@@ -38,7 +38,7 @@
  */
 
 static gboolean
-class_isclass( PElement *instance )
+class_is_class( PElement *instance )
 {
 	if( !PEISCLASS( instance ) ) {
 		BufInfo buf;
@@ -59,7 +59,7 @@ class_isclass( PElement *instance )
 Compile *
 class_get_compile( PElement *instance )
 {
-	if( !class_isclass( instance ) )
+	if( !class_is_class( instance ) )
 		return( NULL );
 
 	return( PEGETCLASSCOMPILE( instance ) );
@@ -79,6 +79,40 @@ class_get_super( PElement *instance, PElement *out )
 	return( class_get_symbol( instance, compile->super, out ) );
 }
 
+void *
+class_map( PElement *instance, class_map_fn fn, void *a, void *b )
+{
+	PElement member;
+	HeapNode *p;
+
+	if( !PEISCLASS( instance ) ) 
+		return( NULL );
+
+	/* Loop over the instance member list.
+	 */
+	PEGETCLASSMEMBER( &member, instance );
+	if( !PEISELIST( &member ) )
+		for( p = PEGETVAL( &member ); p; p = GETRIGHT( p ) ) {
+			HeapNode *hn;
+			PElement s, v;
+			Symbol *sym;
+			void *result;
+
+			/* Get the sym/value pair, get the sym.
+			 */
+			hn = GETLEFT( p );
+			PEPOINTLEFT( hn, &s );
+			PEPOINTRIGHT( hn, &v );
+			sym = PEGETSYMREF( &s );
+
+			if( (result = fn( sym, &v, a, b )) )
+				return( result );
+
+		}
+
+	return( NULL );
+}
+
 /* Look up a member in a class instance by name. If lookup fails in this 
  * instance, try the superclass. Don't search secrets. Point sym and value 
  * at the symbol we found and its value. sym can be NULL for no result
@@ -97,7 +131,7 @@ class_get_member( PElement *instance, const char *name,
 	pgraph( instance );
 #endif /*DEBUG_MEMBER*/
 
-	if( !class_isclass( instance ) )
+	if( !class_is_class( instance ) )
 		return( FALSE );
 
 	/* Search this instance member list.
@@ -177,7 +211,7 @@ class_get_symbol( PElement *instance, Symbol *sym, PElement *out )
 	pgraph( instance );
 #endif /*DEBUG_MEMBER*/
 
-	if( !class_isclass( instance ) )
+	if( !class_is_class( instance ) )
 		return( FALSE );
 
 	PEGETCLASSSECRET( &secret, instance );
@@ -237,7 +271,7 @@ class_get_exact( PElement *instance, const char *name, PElement *out )
 	PElement pe;
 
 	pe = *instance;
-	while( !reduce_isinstanceof_exact( reduce_context, name, &pe ) ) {
+	while( !reduce_is_instanceof_exact( reduce_context, name, &pe ) ) {
 		if( !class_get_super( &pe, &pe ) || PEISELIST( &pe ) ) 
 			return( FALSE );
 	}
@@ -1051,7 +1085,7 @@ class_get_member_class( PElement *instance, const char *name,
 		(ClassGetFn) heap_get_class, out ) )
 		return( FALSE );
 
-	if( !heap_isinstanceof( type, out, &result ) ) 
+	if( !heap_is_instanceof( type, out, &result ) ) 
 		return( FALSE );
 	if( !result ) {
 		class_typecheck_error( instance, name, type );
@@ -1100,7 +1134,7 @@ class_get_member_instance( PElement *instance,
 	gboolean result;
 
 	return( class_get_member( instance, name, NULL, out ) &&
-		heap_isinstanceof( klass, out, &result ) &&
+		heap_is_instanceof( klass, out, &result ) &&
 		result );
 }
 
