@@ -42,7 +42,9 @@ static const char *colour_colour_space[] = {
 	"lch", 		/* index 3 */
 	"ucs", 		/* index 4 */
 	"rgb", 		/* index 5 */
-	"srgb"		/* index 6 */
+	"srgb",		/* index 6 */
+	"rgb16",	/* index 7 */
+	"grey16"	/* index 8 */
 };
 
 /* For each allowed colourspace, the corresponding VIPS Type value.
@@ -54,7 +56,9 @@ static const int colour_type[] = {
 	IM_TYPE_LCH,
 	IM_TYPE_UCS,
 	IM_TYPE_RGB,
-	IM_TYPE_sRGB
+	IM_TYPE_sRGB,
+	IM_TYPE_RGB16,
+	IM_TYPE_GREY16
 };
 
 static ClassmodelClass *parent_class = NULL;
@@ -131,7 +135,7 @@ colour_set_colour( Colour *colour,
 
 	for( i = 0; i < 3; i++ )
 		colour->value[i] = value[i];
-	SETSTR( colour->colour_space, colour_space );
+	IM_SETSTR( colour->colour_space, colour_space );
 
 	colour_refresh( colour );
         classmodel_update( CLASSMODEL( colour ) );
@@ -262,7 +266,7 @@ colour_edit( GtkWidget *parent, Model *model )
 		iwindow_true_cb, NULL, idialog_free_client, eds );
 	idialog_add_ok( IDIALOG( idlg ), 
 		colour_done_cb, _( "Set Color" ) );
-	idialog_set_parent( IDIALOG( idlg ), parent );
+	iwindow_set_parent( IWINDOW( idlg ), parent );
 	idialog_set_iobject( IDIALOG( idlg ), IOBJECT( model ) );
 	idialog_set_pinup( IDIALOG( idlg ), TRUE );
 	iwindow_build( IWINDOW( idlg ) );
@@ -270,124 +274,42 @@ colour_edit( GtkWidget *parent, Model *model )
 	gtk_widget_show( GTK_WIDGET( idlg ) );
 }
 
-static xmlNode *
-colour_save( Model *model, xmlNode *xnode )
+static View *
+colour_view_new( Model *model, View *parent )
 {
-        Colour *colour = COLOUR( model );
-
-	xmlNode *xthis;
-
-	if( !(xthis = MODEL_CLASS( parent_class )->save( model, xnode )) )
-		return( NULL );
-
-	if( CLASSMODEL( model )->edited ) {
-		if( !set_prop( xthis, "colour_space", "%s",
-			colour->colour_space ) ||
-			!set_prop( xthis, "value0", "%g", colour->value[0] ) ||
-			!set_prop( xthis, "value1", "%g", colour->value[1] ) ||
-			!set_prop( xthis, "value2", "%g", colour->value[2] ) )
-			return( NULL );
-	}
-
-	return( xthis );
+	return( colourview_new() );
 }
 
-static gboolean
-colour_load( Model *model, 
-	ModelLoadState *state, Model *parent, xmlNode *xnode )
+static void *
+colour_update_model( Heapmodel *heapmodel )
 {
-        Colour *colour = COLOUR( model );
-	char colour_space[MAX_STRSIZE];
+	Colour *colour = COLOUR( heapmodel );
 
-#ifdef DEBUG
-	printf( "colour_load: " );
-	row_name_print( HEAPMODEL( colour )->row );
-	printf( "\n" );
-#endif /*DEBUG*/
-
-	assert( IS_RHS( parent ) );
-
-	if( get_sprop( xnode, "colour_space", colour_space, MAX_STRSIZE ) &&
-		get_dprop( xnode, "value0", &colour->value[0] ) &&
-		get_dprop( xnode, "value1", &colour->value[1] ) &&
-		get_dprop( xnode, "value2", &colour->value[2] ) ) {
-		SETSTR( colour->colour_space, colour_space );
-		colour_refresh( colour );
-		classmodel_set_edited( CLASSMODEL( model ), TRUE );
-	}
-
-	return( MODEL_CLASS( parent_class )->load( model, 
-		state, parent, xnode ) );
-}
-
-/* Update Colour from heap.
- */
-static gboolean
-colour_class_get( Classmodel *classmodel, PElement *root )
-{
-        Colour *colour = COLOUR( classmodel );
-	char colour_space[MAX_STRSIZE];
-	double value[3];
-	int l;
-	int i;
-
-#ifdef DEBUG
-	printf( "colour_class_get: " );
-	row_name_print( HEAPMODEL( colour )->row );
-	printf( "\n" );
-#endif /*DEBUG*/
-
-	if( !class_get_member_string( root, MEMBER_COLOUR_SPACE, 
-		colour_space, MAX_STRSIZE ) ||
-		!class_get_member_realvec( root, MEMBER_VALUE, value, 3, &l ) )
-		return( FALSE );
-	if( l != 3 ) {
-		error_top( _( "Bad value." ) );
-		error_sub( _( "3 band value only" ) );
-		return( FALSE );
-	}
-	SETSTR( colour->colour_space, colour_space );
-	for( i = 0; i < 3; i++ )
-		colour->value[i] = value[i];
+	if( HEAPMODEL_CLASS( parent_class )->update_model( heapmodel ) )
+		return( heapmodel );
 
 	colour_refresh( colour );
 
-	return( CLASSMODEL_CLASS( parent_class )->class_get( 
-		classmodel, root ) );
+	return( NULL );
 }
 
-/* Make a new "fn value" application.
+/* Members of colour we automate.
  */
-static gboolean
-colour_class_new( Classmodel *classmodel, PElement *fn, PElement *out )
-{
-	Heap *heap = reduce_context->heap;
-	Colour *colour = COLOUR( classmodel );
-	PElement rhs;
-
-#ifdef DEBUG
-	printf( "colour_class_new: " );
-	row_name_print( HEAPMODEL( colour )->row );
-	printf( "\n" );
-#endif /*DEBUG*/
-
-	/* Make application nodes.
-	 */
-	heap_appl_init( out, fn );
-	if( !heap_appl_add( heap, out, &rhs ) ||
-		!heap_string_new( heap, colour->colour_space, &rhs ) ||
-		!heap_appl_add( heap, out, &rhs ) ||
-		!heap_realvec_new( heap, 3, colour->value, &rhs ) )
-		return( FALSE );
-
-	return( TRUE );
-}
+static ClassmodelMember colour_members[] = {
+	{ CLASSMODEL_MEMBER_STRING, NULL, 0, 
+		MEMBER_COLOUR_SPACE, "colour_space", N_( "Color Space" ),
+		G_STRUCT_OFFSET( Colour, colour_space ) },
+	{ CLASSMODEL_MEMBER_REALVEC_FIXED, NULL, 3,
+		MEMBER_VALUE, "value", N_( "Value" ),
+		G_STRUCT_OFFSET( Colour, value ) }
+};
 
 static void
 colour_class_init( ColourClass *class )
 {
 	GObjectClass *gobject_class = (GObjectClass *) class;
 	ModelClass *model_class = (ModelClass *) class;
+	HeapmodelClass *heapmodel_class = (HeapmodelClass *) class;
 	ClassmodelClass *classmodel_class = (ClassmodelClass *) class;
 
 	parent_class = g_type_class_peek_parent( class );
@@ -399,17 +321,17 @@ colour_class_init( ColourClass *class )
 	 */
 	gobject_class->finalize = colour_finalize;
 
-	model_class->view_new = colourview_new;
+	model_class->view_new = colour_view_new;
 	model_class->edit = colour_edit;
-	model_class->save = colour_save;
-	model_class->load = colour_load;
 
-	classmodel_class->class_get = colour_class_get;
-	classmodel_class->class_new = colour_class_new;
+	heapmodel_class->update_model = colour_update_model;
 
 	/* Static init.
 	 */
 	model_register_loadable( MODEL_CLASS( class ) );
+
+	classmodel_class->members = colour_members;
+	classmodel_class->n_members = IM_NUMBER( colour_members );
 }
 
 static void
@@ -448,14 +370,3 @@ colour_get_type( void )
 
 	return( type );
 }
-
-Classmodel *
-colour_new( Rhs *rhs )
-{
-	Colour *colour;
-
-	colour = COLOUR( g_object_new( TYPE_COLOUR, NULL ) );
-	icontainer_child_add( ICONTAINER( rhs ), ICONTAINER( colour ), -1 );
-
-	return( CLASSMODEL( colour ) );
-}
