@@ -144,6 +144,15 @@ column_parent_add( iContainer *child )
 	col->ws = column_get_workspace( col );
 }
 
+static View *
+column_view_new( Model *model, View *parent )
+{
+	if( IS_PREFWORKSPACEVIEW( parent ) )
+		return( prefcolumnview_new() );
+	else
+		return( columnview_new() );
+}
+
 static xmlNode *
 column_save( Model *model, xmlNode *xnode )
 {
@@ -164,7 +173,9 @@ column_save( Model *model, xmlNode *xnode )
 		!set_prop( xthis, "open", bool_to_char( col->open ) ) ||
 		!set_prop( xthis, "selected", bool_to_char( col->selected ) ) ||
 		!set_prop( xthis, "sform", bool_to_char( FALSE ) ) ||
-		!set_prop( xthis, "next", "%d", col->next ) ) 
+		!set_prop( xthis, "next", "%d", col->next ) || 
+		!set_sprop( xthis, "name", IOBJECT( col )->name ) || 
+		!set_sprop( xthis, "caption", IOBJECT( col )->caption ) ) 
 		return( NULL );
 
 	return( xthis );
@@ -190,10 +201,11 @@ column_load( Model *model,
 	ModelLoadState *state, Model *parent, xmlNode *xnode )
 {
 	Column *col = COLUMN( model );
-
 	int x = col->x;
 	int y = col->y;
 
+	char buf[256];
+
 	assert( IS_WORKSPACE( parent ) );
 
 	if( !get_iprop( xnode, "x", &x ) ||
@@ -206,6 +218,16 @@ column_load( Model *model,
 	col->x = x + column_left_offset;
 	col->y = y + column_top_offset;
 
+	/* Don't use iobject_set(): we don't want to trigger _changed during
+	 * load.
+	 */
+	if( get_sprop( xnode, "caption", buf, 256 ) ) {
+		IM_SETSTR( IOBJECT( col )->caption, buf );
+	}
+	if( get_sprop( xnode, "name", buf, 256 ) ) {
+		IM_SETSTR( IOBJECT( col )->name, buf );
+	}
+
 	return( MODEL_CLASS( parent_class )->load( model, 
 		state, parent, xnode ) );
 }
@@ -231,7 +253,7 @@ column_class_init( ColumnClass *class )
 	icontainer_class->child_remove = column_child_remove;
 	icontainer_class->parent_add = column_parent_add;
 
-	model_class->view_new = columnview_new;
+	model_class->view_new = column_view_new;
 	model_class->save = column_save;
 	model_class->save_test = column_save_test;
 	model_class->load = column_load;
@@ -373,7 +395,7 @@ column_add_n_names( Column *col, const char *name, BufInfo *buf, int nparam )
 /* Is a column empty?
  */
 gboolean
-column_isempty( Column *col )
+column_is_empty( Column *col )
 {
 	Subcolumn *scol = col->scol;
 	GSList *children = ICONTAINER( scol )->children;
@@ -453,15 +475,17 @@ column_open_timeout_cb( Column *col )
 void 
 column_set_open( Column *col, gboolean open )
 {
-	Workspace *ws = col->ws;
+        if( col->open != open ) {
+		Workspace *ws = col->ws;
 
-        col->open = !col->open;
-	filemodel_set_modified( FILEMODEL( ws ), TRUE );
-        iobject_changed( IOBJECT( col ) );
+		col->open = open;
+		filemodel_set_modified( FILEMODEL( ws ), TRUE );
+		iobject_changed( IOBJECT( col ) );
 
-	if( !col->open_timeout ) {
-		col->open_frames = 0;
-		col->open_timeout = g_timeout_add( column_open_ms, 
-			(GSourceFunc) column_open_timeout_cb, col );
+		if( !col->open_timeout ) {
+			col->open_frames = 0;
+			col->open_timeout = g_timeout_add( column_open_ms, 
+				(GSourceFunc) column_open_timeout_cb, col );
+		}
 	}
 }
