@@ -44,6 +44,14 @@
 #define DEBUG_RESULT
  */
 
+/* trace list comp compile
+#define DEBUG_LCOMP
+ */
+
+/* trace pattern LHS generation
+#define DEBUG_PATTERN
+ */
+
 /* 
 #define DEBUG
  */
@@ -109,19 +117,19 @@ compile_map_all_sub( Symbol *sym, map_compile_fn fn, void *a )
 		return( compile_map_all( sym->expr->compile, fn, a ) );
 }
 
-/* Apply a function to a compile ... and any local compiles.
+/* Apply a function to a compile ... and any local compiles. Do top-down.
  */
 Compile *
 compile_map_all( Compile *compile, map_compile_fn fn, void *a )
 {
 	Compile *res;
 
-	/* Apply to this compile.
+	/* Us first.
 	 */
 	if( (res = fn( compile, a )) )
 		return( res );
 
-	/* And over any locals.
+	/* Then any children.
 	 */
 	if( (res = (Compile *) icontainer_map( ICONTAINER( compile ),
 		(icontainer_map_fn) compile_map_all_sub, (void *) fn, a )) )
@@ -300,15 +308,43 @@ compile_finalize( GObject *gobject )
 	G_OBJECT_CLASS( parent_class )->finalize( gobject );
 }
 
+static void
+compile_child_add( iContainer *parent, iContainer *child, int pos )
+{
+	Compile *compile = COMPILE( parent );
+	Symbol *sym = SYMBOL( child );
+
+	if( sym ) 
+		compile->last_sym = sym;
+
+	ICONTAINER_CLASS( parent_class )->child_add( parent, child, pos );
+}
+
+static void 
+compile_child_remove( iContainer *parent, iContainer *child )
+{
+	Compile *compile = COMPILE( parent );
+	Symbol *sym = SYMBOL( child );
+
+	if( sym && compile->last_sym == sym )
+		compile->last_sym = NULL;
+
+	ICONTAINER_CLASS( parent_class )->child_remove( parent, child );
+}
+
 static void
 compile_class_init( CompileClass *class )
 {
 	GObjectClass *gobject_class = (GObjectClass *) class;
+	iContainerClass *icontainer_class = (iContainerClass *) class;
 
 	parent_class = g_type_class_peek_parent( class );
 
 	gobject_class->finalize = compile_finalize;
 
+	icontainer_class->child_add = compile_child_add;
+	icontainer_class->child_remove = compile_child_remove;
+
 	/* Create signals.
 	 */
 
@@ -334,6 +370,7 @@ compile_init( Compile *compile )
 
 	compile->tree = NULL;
 	compile->treefrag = NULL;
+	compile->last_sym = NULL;
 
 	compile->nparam = 0;
 	compile->param = NULL;
@@ -1507,11 +1544,9 @@ compile_remove_subexpr( Compile *compile, PElement *root )
 /* Top-level compiler driver.
  */
 
-static void *compile_symbol( Symbol *sym );
-
 /* Compile a symbol into a heap. 
  */
-void *
+static void *
 compile_heap( Compile *compile )
 {
 	PElement base;
@@ -1520,13 +1555,12 @@ compile_heap( Compile *compile )
 	char txt[1024];
 #endif /*DEBUG*/
 
-	PEPOINTE( &base, &compile->base );
-
-	/* Compile all our sub-definitions first.
+	/* Don't generate code for parser temps.
 	 */
-	if( icontainer_map( ICONTAINER( compile ),
-		(icontainer_map_fn) compile_symbol, NULL, NULL ) )
-		return( compile->sym );
+	if( compile->sym->placeholder )
+		return( NULL );
+
+	PEPOINTE( &base, &compile->base );
 
 	/* Is there an existing function base? GC it away.
 	 */
@@ -1603,25 +1637,81 @@ compile_heap( Compile *compile )
 	return( NULL );
 }
 
-/* Compile a symbol.
- */
+static void *compile_object_sub( Compile *compile );
+
 static void *
-compile_symbol( Symbol *sym )
+compile_symbol_sub( Symbol *sym )
 {
-	if( sym->expr && sym->expr->compile )
-		return( compile_heap( sym->expr->compile ) );
+	Compile *compile;
+
+	if( sym->expr && (compile = sym->expr->compile) )
+		if( compile_object_sub( compile ) )
+			return( sym );
 
 	return( NULL );
 }
 
+static void *
+compile_object_sub( Compile *compile )
+{
+	if( icontainer_map( ICONTAINER( compile ),
+		(icontainer_map_fn) compile_symbol_sub, NULL, NULL ) )
+		return( compile );
+
+	if( compile_heap( compile ) )
+		return( compile );
+
+	return( NULL );
+}
+
+/* Top-level compile a thing entry point.
+ */
+void *
+compile_object( Compile *compile )
+{
+	/* Walk this tree of symbols computing the secret lists.
+	 */
+	secret_build( compile );
+
+	/* Compile all definitions from the inside out.
+	 */
+	if( compile_object_sub( compile ) )
+		return( compile );
+
+	return( NULL );
+}
+
+static void *
+compile_toolkit_sub( Tool *tool )
+{
+	Compile *compile;
+
+	if( tool->sym && tool->sym->expr && 
+		(compile = tool->sym->expr->compile )) 
+		/* Only if we have no code.
+		 */
+		if( compile->base.type == ELEMENT_NOVAL ) 
+			if( compile_object( compile ) )
+				return( tool );
+
+	return( NULL );
+}
+
+/* Scan a toolkit and make sure all the symbols have been compiled.
+ */
+void *
+compile_toolkit( Toolkit *kit )
+{
+	return( toolkit_map( kit,
+		(tool_map_fn) compile_toolkit_sub, NULL, NULL ) );
+}
+
 /* Parse support.
  */
 
-static void
+static ParseNode *
 compile_check_i18n( Compile *compile, ParseNode *pn )
 {
-	GSList *l;
-
 	switch( pn->type ) {
 	case NODE_APPLY:
 		if( pn->arg1->type == NODE_LEAF && 
@@ -1659,49 +1749,45 @@ compile_check_i18n( Compile *compile, ParseNode *pn )
 		}
 		break;
 
-	case NODE_GENERATOR:
-		compile_check_i18n( compile, pn->arg1 );
-		if( pn->arg2 )
-			compile_check_i18n( compile, pn->arg2 );
-		if( pn->arg3 )
-			compile_check_i18n( compile, pn->arg3 );
+	default:
 		break;
+	}
 
+	return( NULL );
+}
+
+static ParseNode *
+compile_check_more( Compile *compile, ParseNode *pn )
+{
+	switch( pn->type ) {
 	case NODE_BINOP:
-	case NODE_COMPOSE:
-		compile_check_i18n( compile, pn->arg1 );
-		compile_check_i18n( compile, pn->arg2 );
-		break;
+		switch( pn->biop ) {
+		case BI_MORE:
+			pn->biop = BI_LESS;
+			SWAPP( pn->arg1, pn->arg2 );
+			break;
 
-	case NODE_UOP:
-		compile_check_i18n( compile, pn->arg1 );
-		break;
+		case BI_MOREEQ:
+			pn->biop = BI_LESSEQ;
+			SWAPP( pn->arg1, pn->arg2 );
+			break;
 
-	case NODE_SUPER:
-	case NODE_LISTCONST:
-		for( l = pn->elist; l; l = l->next ) {
-			ParseNode *arg = (ParseNode *) l->data;
-
-			compile_check_i18n( compile, arg );
+		default:
+			break;
 		}
 		break;
 
-	case NODE_LEAF:
-	case NODE_CLASS:
-	case NODE_TAG:
-	case NODE_CONST:
-		break;
-
-	case NODE_NONE:
 	default:
-		assert( FALSE );
+		break;
 	}
+
+	return( NULL );
 }
 
 /* Do end-of-parse checks. Called after every 'A=...' style definition (not 
  * just top-level syms). Used to do lots of checks, not much left now.
  */
-void
+gboolean
 compile_check( Compile *compile )
 {
 	Symbol *sym = compile->sym;
@@ -1711,21 +1797,34 @@ compile_check( Compile *compile )
 	 */ 
 	if( is_member( sym ) &&
 		strcmp( IOBJECT( sym )->name, MEMBER_CHECK ) == 0 ) {
-		if( compile->nparam != 0 ) 
-			yyerror( _( "Member \"%s\" of class "
+		if( compile->nparam != 0 ) {
+			error_top( _( "Too many arguments." ) );
+			error_sub( _( "Member \"%s\" of class "
 				"\"%s\" should have no arguments." ),
 				MEMBER_CHECK, symbol_name( parent ) );
+
+			return( FALSE );
+		}
 	}
 
-	/* Magic stuff for i18n of menus. Look for (_ "string constant") and
-	 * pump it through gettext.
+	/* Look for (_ "string constant") and pump it through gettext. We can
+	 * do a lot of i18n at compile-time.
 	 */
 #ifdef DEBUG
 	printf( "compile_check_i18n: " );
 	compile_name_print( compile );
 	printf( "\n" );
 #endif /*DEBUG*/
-	compile_check_i18n( compile, compile->tree );
+	(void) tree_map( compile, 
+		(tree_map_fn) compile_check_i18n, compile->tree, NULL, NULL );
+
+	/* Swap MORE and MOREEQ for LESS and LESSEQ. Reduces the number of
+	 * cases for the compiler. 
+	 */
+	(void) tree_map( compile, 
+		(tree_map_fn) compile_check_more, compile->tree, NULL, NULL );
+
+	return( TRUE );
 }
 
 /* Mark error on all exprs using this compile.
@@ -1740,8 +1839,8 @@ compile_error_set( Compile *compile )
  */
 static void *
 compile_patch_pointers_sub( void **pnt, void *nsym, void *osym )
-{	
-	assert( *pnt == osym );
+{
+	g_assert( *pnt == osym );
 
 	*pnt = nsym;
 
@@ -1956,3 +2055,760 @@ compile_get_member_string( Compile *compile, const char *name )
 
 	return( NULL );
 }
+
+static void *
+compile_find_generated_node( Compile *compile, ParseNode *node,
+	GSList **list )
+{
+	Symbol *sym = node->leaf;
+
+        if( node->type == NODE_LEAF &&
+		sym->generated &&
+		symbol_get_parent( sym ) &&
+		symbol_get_parent( sym )->expr->compile == compile ) 
+		*list = g_slist_prepend( *list, sym );
+
+	return( NULL );
+}
+
+/* Search a scrap of tree and build a list of all the lambdas/lcomps/etc. it
+ * generated.
+ */
+static GSList *
+compile_find_generated( Compile *compile, ParseNode *tree )
+{
+	GSList *list;
+
+	list = NULL;
+	tree_map( compile,
+		(tree_map_fn) compile_find_generated_node, tree, &list, NULL );
+
+	return( list );
+}
+
+/* Make a copy of sym (and all it's children and trees) in the destination 
+ * scope. This only works for stuff from the parse stage. Symbols which have
+ * values and stuff attached are too complicated to copy easily.
+ */
+static void *
+compile_copy_sym( Symbol *sym, Compile *dest )
+{
+	const char *name = IOBJECT( sym )->name;
+	Symbol *copy_sym;
+
+#ifdef DEBUG
+	printf( "compile_copy_sym: copying " );
+	symbol_name_print( sym );
+	printf( " to scope of " );
+	compile_name_print( dest );
+	printf( "\n" );
+#endif /*DEBUG*/
+
+	/* Must be a different place.
+	 */
+	assert( symbol_get_parent( sym )->expr->compile != dest );
+
+	/* Must not be an existing sym of that name. Or if there is, it has to
+	 * be a zombie.
+	 */
+	assert( !compile_lookup( dest, name ) ||
+		compile_lookup( dest, name )->type == SYM_ZOMBIE );
+
+	switch( sym->type ) {
+	case SYM_VALUE:
+		copy_sym = symbol_new_defining( dest, name );
+		copy_sym->generated = sym->generated;
+		(void) symbol_user_init( copy_sym );
+		(void) compile_new_local( copy_sym->expr );
+
+		/* Copy any locals over. We have to do this before we copy the
+		 * tree so that the new tree links to the new syms.
+		 */
+		icontainer_map( ICONTAINER( sym->expr->compile ),
+			(icontainer_map_fn) compile_copy_sym, 
+			copy_sym->expr->compile, NULL );
+
+		copy_sym->expr->compile->tree = tree_copy( 
+			copy_sym->expr->compile, sym->expr->compile->tree );
+
+		/* Copying the tree may have made some zombies. Resolve 
+		 * outwards.
+		 */
+		compile_resolve_names( copy_sym->expr->compile, dest );
+
+		break;
+
+	case SYM_PARAM:
+		copy_sym = symbol_new_defining( dest, name );
+		copy_sym->generated = sym->generated;
+		symbol_parameter_init( copy_sym );
+		break;
+
+	case SYM_ZOMBIE:
+		break;
+
+	case SYM_WORKSPACE:
+	case SYM_WORKSPACEGROUP:
+	case SYM_ROOT:
+	case SYM_EXTERNAL:
+	case SYM_BUILTIN:
+	default:
+		assert( 0 );
+	}
+
+	return( NULL );
+}
+
+/* tree is a scrap of graph in fromscope's context. It may have caused the
+ * generation of a number of lambdas, lcomps etc. in fromscope. Make a copy
+ * of the tree in toscope and copy over any generated syms too. fromscope and
+ * toscope can be the same, in which case we can just copy the tree.
+ */
+ParseNode *
+compile_copy_tree( Compile *fromscope, ParseNode *tree, Compile *toscope )
+{
+	ParseNode *copy_tree;
+
+#ifdef DEBUG
+	printf( "compile_copy_tree: copying tree from " );
+	compile_name_print( fromscope );
+	printf( " to " );
+	compile_name_print( toscope );
+	printf( "\n" );
+#endif /*DEBUG*/
+
+	/* A new context? Copy generated syms over.
+	 */
+	if( fromscope != toscope ) {
+		GSList *generated;
+
+		generated = compile_find_generated( fromscope, tree );
+
+#ifdef DEBUG
+		printf( "with generated children: " ); 
+		(void) slist_map( generated, (SListMapFn) dump_tiny, NULL );
+		printf( "\n" ); 
+#endif /*DEBUG*/
+
+		slist_map( generated, 
+			(SListMapFn) compile_copy_sym, toscope );
+
+		g_slist_free( generated );
+	}
+
+	copy_tree = tree_copy( toscope, tree );
+
+	/* Copying the tree may have made some zombies. Resolve 
+	 * outwards.
+	 */
+	compile_resolve_names( toscope, compile_get_parent( toscope ) );
+
+	return( copy_tree );
+}
+
+/* Generate the parse tree for this list comprehension. 
+
+  Example: after parse we have:
+
+	[(x, y) | x <- [1..3]; y <- [x..3]; x + y > 3];
+
+	... $$lcomp1 ...
+	{
+		$$lcomp1 = NULL
+		{
+			$$result = (x, y);
+			// elements in left-to-right order
+			// in compile->children
+			x = [1..3]
+			y = [x..3]
+			$$filter1 = x + y > 3
+		}
+	}
+
+  and we generate this code:
+
+  	z = $$lcomp1
+	{
+		$$lcomp1 = foldr $f1 [] [1..3]
+		{
+			$f1 x $sofar = foldr $f2 $sofar [x..3]
+			{
+				$f2 y $sofar = if x + y > 3 then $f3 else $sofar
+				{
+					$f3 = (x, y) : $sofar;
+				}
+			}
+		}
+	}
+
+ */
+
+/* Find the placeholders generated by the parser. Filters, generators,
+ * patterns and $$result. 
+ */
+static void *
+compile_lcomp_find( Symbol *sym, GSList **children )
+{
+	if( sym->placeholder )
+		*children = g_slist_append( *children, sym );
+
+	return( NULL );
+}
+
+static Symbol *
+compile_lcomp_find_pattern( GSList *children, const char *generator )
+{
+	int n;
+	char pattern[256];
+	GSList *p;
+
+	if( sscanf( generator, "$$generator%d", &n ) != 1 )
+		return( NULL );
+	im_snprintf( pattern, 256, "$$pattern%d", n );
+
+	for( p = children; p; p = p->next ) {
+		Symbol *sym = (Symbol *) p->data;
+
+		if( strcmp( IOBJECT( sym )->name, pattern ) == 0 )
+			return( sym );
+	}
+
+	return( NULL );
+}
+
+void 
+compile_lcomp( Compile *compile )
+{
+	/* Number nested locals with this. Keep numbering global so debugging
+	 * nested lcomps is easier.
+	 */
+	static int count = 1;
+
+	GSList *children;
+	gboolean sofar;
+	Compile *scope;
+	Symbol *result;
+	GSList *p;
+	Symbol *child;
+	char name[256];
+	ParseNode *n1, *n2, *n3;
+
+#ifdef DEBUG_LCOMP
+	printf( "before compile_lcomp:\n" );
+	dump_compile( compile );
+#endif /*DEBUG_LCOMP*/
+
+	/* Find all the elements of the lcomp: generators, filters, patterns 
+	 * and $$result.
+	 */
+	children = NULL;
+	(void) icontainer_map( ICONTAINER( compile ), 
+		(icontainer_map_fn) compile_lcomp_find, &children, NULL );
+
+#ifdef DEBUG_LCOMP
+	printf( "list comp " );
+	compile_name_print( compile );
+	printf( " has children: " ); 
+	(void) slist_map( children, (SListMapFn) dump_tiny, NULL );
+	printf( "\n" ); 
+#endif /*DEBUG_LCOMP*/
+
+	/* As yet no list to build on.
+	 */
+	sofar = FALSE;
+
+	/* Start by building a tree in this scope.
+	 */
+	scope = compile;
+
+	/* Not seen the result element yet, but we should.
+	 */
+	result = NULL;
+
+	/* Now generate code for each element, either a filter or a generator.
+	 * If we do a generator, we need to search for the associated pattern
+	 * and expand it.
+	 */
+	for( p = children; p; p = p->next ) {
+		Symbol *element = (Symbol *) p->data;
+
+		/* Just note the result element ... we use it right at the end.
+		 */
+		if( strcmp( "$$result", IOBJECT( element )->name ) == 0 ) {
+			result = element;
+			continue;
+		}
+
+		/* And only process filter/gen.
+		 */
+		if( !is_prefix( "$$filter", IOBJECT( element )->name ) &&
+			!is_prefix( "$$gen", IOBJECT( element )->name ) ) 
+			continue;
+
+		/* Start the next nest in. child is the local we will make for
+		 * this scope.
+		 */
+		im_snprintf( name, 256, "$$fn%d", count++ );
+		child = symbol_new_defining( scope, name );
+		child->generated = TRUE;
+		(void) symbol_user_init( child );
+		(void) compile_new_local( child->expr );
+
+		if( is_prefix( "$$filter", IOBJECT( element )->name ) ) {
+			/* A filter.
+			 */
+			n1 = compile_copy_tree( compile, 
+				element->expr->compile->tree,
+				scope );
+			n2 = tree_leafsym_new( scope, child );
+			n3 = tree_leaf_new( scope, "$$sofar" );
+			n1 = tree_ifelse_new( scope, n1, n2, n3 );
+			scope->tree = n1;
+		}
+		else if( is_prefix( "$$gen", IOBJECT( element )->name ) ) {
+			Symbol *param1;
+			Symbol *param2;
+			Symbol *pattern;
+			GSList *built_syms;
+
+			/* A generator. 
+			 */
+			param1 = symbol_new_defining( child->expr->compile, 
+				IOBJECT( element )->name );
+			param1->generated = TRUE;
+			symbol_parameter_init( param1 );
+			param2 = symbol_new_defining( child->expr->compile, 
+				"$$sofar" );
+			param2->generated = TRUE;
+			symbol_parameter_init( param2 );
+
+			/* Now expand the pattern: it will access parts of the
+			 * $$generator argument.
+			 */
+			pattern = compile_lcomp_find_pattern( children, 
+				IOBJECT( element )->name );
+			g_assert( pattern );
+			built_syms = compile_pattern_lhs( child->expr->compile, 
+				param1, pattern->expr->compile->tree );
+			g_slist_free( built_syms );
+
+			/* Make the "foldr $$fn $sofar expr" tree.
+			 */
+			n1 = tree_leaf_new( scope, "foldr" );
+			n2 = tree_leafsym_new( scope, child );
+			n3 = tree_appl_new( scope, n1, n2 );
+			if( sofar )
+				n2 = tree_leaf_new( scope, "$$sofar" );
+			else {
+				ParseConst con;
+
+				con.type = PARSE_CONST_ELIST;
+				n2 = tree_const_new( scope, con );
+			}
+			n3 = tree_appl_new( scope, n3, n2 );
+			n2 = compile_copy_tree( compile,
+				element->expr->compile->tree, 
+				scope );
+			n3 = tree_appl_new( scope, n3, n2 );
+			scope->tree = n3;
+
+			/* There's now an enclosing sofar we can use.
+			 */
+			sofar = TRUE;
+		}
+
+		/* Nest in again.
+		 */
+		scope = child->expr->compile;
+	}
+
+	/* Copy the code for the final result.
+	 */
+	assert( result );
+
+	n1 = compile_copy_tree( result->expr->compile, 
+		result->expr->compile->tree, scope );
+	n2 = tree_leaf_new( scope, "$$sofar" );
+	n3 = tree_binop_new( compile, BI_CONS, n1, n2 );
+	scope->tree = n3;
+
+	/* Loop outwards again, closing the scopes we made.
+	 */
+	while( scope != compile ) {
+		/* We know check can't fail on generated code.
+
+		   	FIXME ... yuk, maybe compile_lcomp should be 
+			failable too
+
+		 */
+		(void) compile_check( scope );
+		compile_resolve_names( scope, compile_get_parent( scope ) );
+
+		scope = compile_get_parent( scope );
+	}
+
+#ifdef DEBUG_LCOMP
+	printf( "after compile_lcomp:\n" );
+	dump_compile( compile );
+#endif /*DEBUG_LCOMP*/
+
+	g_slist_free( children );
+}
+
+/* Compile a pattern LHS. Generate a sym for each pattern variable, each of
+ * which checks and accesses sym. For example:
+ *
+ *	[a] = x;
+ *
+ * compiles to:
+ *
+ * 	sym = x;
+ * 	a = if is_list sym && len sym == 1 then sym?0 else error "..";
+ */
+
+/* Generate code to access element n of a pattern trail. Eg, pattern is
+ * 	[[[a]]]
+ * the trail will be 
+ * 	0) LISTCONST 1) LISTCONST 2) LISTCONST 3) LEAF
+ * then access(0) will be
+ * 	leaf
+ * and access(1) will be
+ * 	leaf?0
+ * and access(3) (to get the value for a) will be
+ * 	leaf?0?0?0
+ */
+static ParseNode *
+compile_pattern_access( Compile *compile, 
+	Symbol *leaf, ParseNode **trail, int n )
+{
+	ParseNode *node;
+	ParseNode *left;
+	ParseNode *right;
+	ParseConst c;
+	int i;
+
+	/* The initial leaf ref we access from.
+	 */
+	node = tree_leafsym_new( compile, leaf );
+
+	for( i = 0; i < n; i++ )
+		switch( trail[i]->type ) {
+		case NODE_CONST:
+		case NODE_PATTERN_CLASS:
+		case NODE_LEAF:
+			break;
+
+		case NODE_BINOP:
+			switch( trail[i]->biop ) {
+			case BI_COMMA:
+				/* Generate re or im?
+				 */
+				if( trail[i]->arg1 == trail[i + 1] )
+					left = tree_leaf_new( compile, "re" );
+				else
+					left = tree_leaf_new( compile, "im" );
+				node = tree_appl_new( compile, left, node );
+				break;
+
+			case BI_CONS:
+				/* Generate hd or tl?
+				 */
+				if( trail[i]->arg1 == trail[i + 1] )
+					left = tree_leaf_new( compile, "hd" );
+				else
+					left = tree_leaf_new( compile, "tl" );
+				node = tree_appl_new( compile, left, node );
+				break;
+
+			default:
+				g_assert( 0 );
+			}
+			break;
+
+		case NODE_LISTCONST:
+			/* Which list element do we need? Look for the next
+			 * item in the trail in the list of elements.
+			 */
+			c.type = PARSE_CONST_NUM;
+			c.val.num = g_slist_index( trail[i]->elist, 
+				trail[i + 1] );
+			right = tree_const_new( compile, c );
+			node = tree_binop_new( compile, 
+				BI_SELECT, node, right );
+			break;
+
+		default:
+			g_assert( 0 );
+		}
+
+	return( node );
+}
+
+/* Generate a parsetree for the condition test. The array of nodes represents
+ * the set of conditions we have to test, left to right. 
+ */
+static ParseNode *
+compile_pattern_condition( Compile *compile, 
+	Symbol *leaf, ParseNode **trail, int depth )
+{
+	ParseConst n;
+	ParseNode *node;
+	ParseNode *node2;
+	ParseNode *left;
+	ParseNode *right;
+	int i;
+
+	n.type = PARSE_CONST_BOOL;
+	n.val.bool = TRUE;
+	node = tree_const_new( compile, n );
+
+	for( i = depth - 1; i >= 0; i-- ) {
+		switch( trail[i]->type ) {
+		case NODE_LEAF:
+			break;
+
+		case NODE_BINOP:
+			switch( trail[i]->biop ) {
+			case BI_COMMA:
+				/* Generate is_complex x.
+				 */
+				left = tree_leaf_new( compile, "is_complex" );
+				right = compile_pattern_access( compile, 
+					leaf, trail, i );
+				node2 = tree_appl_new( compile, left, right );
+
+				node = tree_binop_new( compile, 
+					BI_LAND, node2, node );
+				break;
+
+			case BI_CONS:
+				/* Generate is_list x && x != [].
+				 */
+				left = tree_leaf_new( compile, "is_list" );
+				right = compile_pattern_access( compile, 
+					leaf, trail, i );
+				node2 = tree_appl_new( compile, left, right );
+
+				node = tree_binop_new( compile, 
+					BI_LAND, node2, node );
+
+				left = compile_pattern_access( compile, 
+					leaf, trail, i );
+				n.type = PARSE_CONST_ELIST;
+				right = tree_const_new( compile, n );
+				node2 = tree_binop_new( compile, 
+					BI_NOTEQ, left, right );
+
+				node = tree_binop_new( compile, 
+					BI_LAND, node, node2 );
+				break;
+
+			default:
+				g_assert( 0 );
+			}
+			break;
+
+		case NODE_LISTCONST:
+			/* Generate is_list x && is_list_len n x.
+			 */
+			left = tree_leaf_new( compile, "is_list" );
+			right = compile_pattern_access( compile, 
+				leaf, trail, i );
+			node2 = tree_appl_new( compile, left, right );
+
+			node = tree_binop_new( compile, BI_LAND, node2, node );
+
+			left = tree_leaf_new( compile, "is_list_len" );
+			n.type = PARSE_CONST_NUM;
+			n.val.num = g_slist_length( trail[i]->elist );
+			right = tree_const_new( compile, n );
+			left = tree_appl_new( compile, left, right );
+			right = compile_pattern_access( compile, 
+				leaf, trail, i );
+			node2 = tree_appl_new( compile, left, right );
+
+			node = tree_binop_new( compile, BI_LAND, node, node2 );
+			break;
+
+		case NODE_CONST:
+			/* Generate x == n.
+			 */
+			left = compile_pattern_access( compile, 
+				leaf, trail, i );
+			right = tree_const_new( compile, trail[i]->con );
+			node2 = tree_binop_new( compile, BI_EQ, left, right );
+
+			node = tree_binop_new( compile, BI_LAND, node2, node );
+			break;
+
+		case NODE_PATTERN_CLASS:
+			/* Generate is_instanceof "class-name" x.
+			 */
+			left = tree_leaf_new( compile, "is_instanceof" );
+			n.type = PARSE_CONST_STR;
+			n.val.str = im_strdupn( trail[i]->tag );
+			right = tree_const_new( compile, n );
+			node2 = tree_appl_new( compile, left, right );
+			right = compile_pattern_access( compile, 
+				leaf, trail, i );
+			node2 = tree_appl_new( compile, node2, right );
+
+			node = tree_binop_new( compile, BI_LAND, node2, node );
+			break;
+
+		default:
+			g_assert( 0 );
+		}
+	}
+
+	return( node );
+}
+
+/* Generate a parsetree for a "pattern match failed" error.
+ */
+static ParseNode *
+compile_pattern_error( Compile *compile, Symbol *leaf )
+{
+	ParseNode *left;
+	ParseConst n;
+	ParseNode *right;
+	ParseNode *node;
+
+	left = tree_leaf_new( compile, "error" );
+	n.type = PARSE_CONST_STR;
+	n.val.str = im_strdupn( _( "pattern match failed" ) );
+	right = tree_const_new( compile, n );
+	node = tree_appl_new( compile, left, right );
+
+	return( node );
+}
+
+/* Depth of trail we keep as we walk the pattern.
+ */
+#define MAX_TRAIL (10)
+
+typedef struct _PatternLhs {
+	Compile *compile;	/* Scope in which we generate new symbols */
+	Symbol *sym;		/* Thing we access */
+
+	/* The trail of nodes representing this slice of the pattern.
+	 */
+	ParseNode *trail[MAX_TRAIL];
+	int depth;
+	GSList *built_syms;
+} PatternLhs;
+
+/* Generate one reference. leaf is the new sym we generate.
+ */
+static void
+compile_pattern_lhs_leaf( PatternLhs *lhs, Symbol *leaf )
+{
+	Symbol *sym;
+	Compile *compile;
+
+	sym = symbol_new_defining( lhs->compile, IOBJECT( leaf )->name );
+	sym->generated = TRUE;
+	(void) symbol_user_init( sym );
+	(void) compile_new_local( sym->expr );
+	lhs->built_syms = g_slist_prepend( lhs->built_syms, sym );
+	compile = sym->expr->compile;
+
+	compile->tree = tree_ifelse_new( compile, 
+		compile_pattern_condition( compile, 
+			lhs->sym, lhs->trail, lhs->depth ),
+		compile_pattern_access( compile, 
+			lhs->sym, lhs->trail, lhs->depth ),
+		compile_pattern_error( compile, leaf ) );
+
+#ifdef DEBUG_PATTERN
+	printf( "compile_pattern_lhs_leaf: generated\n" );
+	dump_compile( compile );
+#endif /*DEBUG_PATTERN*/
+}
+
+/* Recurse over the pattern generating references.
+ */
+static void *
+compile_pattern_lhs_sub( ParseNode *node, PatternLhs *lhs )
+{
+	lhs->trail[lhs->depth++] = node;
+
+	switch( node->type ) {
+	case NODE_LEAF:
+		compile_pattern_lhs_leaf( lhs, node->leaf );
+		break;
+
+	case NODE_PATTERN_CLASS:
+		compile_pattern_lhs_sub( node->arg1, lhs );
+		break;
+
+	case NODE_BINOP:
+		compile_pattern_lhs_sub( node->arg1, lhs );
+		compile_pattern_lhs_sub( node->arg2, lhs );
+		break;
+
+	case NODE_LISTCONST:
+		slist_map( node->elist,
+			(SListMapFn) compile_pattern_lhs_sub, lhs );
+		break;
+
+	case NODE_CONST:
+		break;
+
+	default:
+		g_assert( 0 );
+	}
+
+	lhs->depth--;
+
+	return( NULL );
+}
+
+/* Something like "[a] = [1];". sym is the $$pattern we are generating access 
+ * syms for, node is the pattern tree, compile is the scope in which we
+ * generate the new defining symbols. Return a list of the syms we built: they
+ * will need any final finishing up and then having symbol_made() called on 
+ * them. You need to free the list, too.
+ */
+GSList *
+compile_pattern_lhs( Compile *compile, Symbol *sym, ParseNode *node )
+{
+	PatternLhs lhs;
+
+#ifdef DEBUG_PATTERN
+	printf( "compile_pattern_lhs: building access fns for %s\n", 
+		symbol_name( sym ) );
+#endif /*DEBUG_PATTERN*/
+
+	lhs.compile = compile;
+	lhs.sym = sym;
+	lhs.depth = 0;
+	lhs.built_syms = NULL;
+
+	compile_pattern_lhs_sub( node, &lhs );
+
+	g_assert( lhs.depth == 0 );
+
+	return( lhs.built_syms );
+}
+
+static ParseNode *
+compile_pattern_has_leaf_sub( Compile *compile, 
+	ParseNode *node, void *a, void *b )
+{
+	if( node->type == NODE_LEAF )
+		return( node );
+
+	return( NULL );
+}
+
+/* Does a pattern contain a leaf? We don't allow const-only patterns in
+ * definitions.
+ */
+gboolean
+compile_pattern_has_leaf( ParseNode *node )
+{
+	return( tree_map( NULL, 
+		(tree_map_fn) compile_pattern_has_leaf_sub, node, 
+		NULL, NULL ) != NULL );
+}
