@@ -48,8 +48,6 @@
 struct _Compile {
 	iContainer parent_object;
 
-	/* My instance vars.
-	 */
 	Symbol *sym;		/* We are part of this symbol, scopewise */
 
 	GSList *exprs;		/* We are used by these expressions */
@@ -63,6 +61,7 @@ struct _Compile {
 
 	ParseNode *tree;	/* Parse tree we built */
 	GSList *treefrag;	/* List of tree bits for easy freeing */
+	Symbol *last_sym;	/* The last child we added in this context */
 
 	int nparam;		/* Number of real parameters */
 	GSList *param;		/* Pointers into locals for real params */
@@ -106,10 +105,11 @@ Compile *compile_new( Expr *expr );
 Compile *compile_new_toplevel( Expr *expr );
 Compile *compile_new_local( Expr *expr );
 
-void *compile_heap( Compile *compile );
+void *compile_object( Compile *compile );
+void *compile_toolkit( Toolkit *kit );
 
 void compile_error_set( Compile *compile );
-void compile_check( Compile *compile );
+gboolean compile_check( Compile *compile );
 
 void compile_resolve_names( Compile *inner, Compile *outer );
 Symbol *compile_resolve_top( Symbol *sym );
@@ -117,3 +117,12 @@ void compile_resolve_dynamic( Compile *tab, Compile *context );
 
 Symbol *compile_get_member( Compile *compile, const char *name );
 const char *compile_get_member_string( Compile *compile, const char *name );
+
+ParseNode *compile_copy_tree( Compile *fromscope, ParseNode *tree,
+	Compile *toscope );
+
+void compile_lcomp( Compile *compile );
+
+GSList *compile_pattern_lhs( Compile *compile, Symbol *sym, ParseNode *node );
+gboolean compile_pattern_has_leaf( ParseNode *node );
+gboolean compile_pattern_has_args( Compile *compile );
