@@ -86,7 +86,7 @@ conversionview_scale_cb( GtkWidget *wid, Conversionview *cv )
 	}
 	set_pointer();
 
-        if( max - min == 0 ) {
+        if( max - min < 1e-20 ) {
                 error_top( _( "Unable to scale image." ) );
                 error_sub( _( "Maximum and minimum pixel values are equal." ) );
                 box_alert( wid );
@@ -123,8 +123,17 @@ conversionview_reset_cb( GtkWidget *wid, Conversionview *cv )
 {
 	Imagemodel *imagemodel = cv->imagemodel;
 
-	imagemodel->scale = 1.0;
-	imagemodel->offset = 0.0;
+	if( imagemodel->iimage ) {
+		Row *row = HEAPMODEL( imagemodel->iimage )->row;
+
+		imagemodel->scale = row->ws->scale;
+		imagemodel->offset = row->ws->offset;
+	}
+	else {
+		imagemodel->scale = 1.0;
+		imagemodel->offset = 0.0;
+	}
+
 	imagemodel->falsecolour = FALSE;
 	imagemodel->type = TRUE;
 	iobject_changed( IOBJECT( imagemodel ) );
