@@ -54,10 +54,10 @@ static guint expr_signals[SIG_LAST] = { 0 };
 
 /* Set of expressions containing errors.
  */
-static GSList *expr_error_all = NULL;
+GSList *expr_error_all = NULL;
 
-static void *
-expr_error_print_sub( Expr *expr, BufInfo *buf )
+void *
+expr_error_print( Expr *expr, BufInfo *buf )
 {
 	assert( expr->err );
 
@@ -65,14 +65,8 @@ expr_error_print_sub( Expr *expr, BufInfo *buf )
 		buf_appends( buf, "\n" );
 
 	buf_appendf( buf, _( "error in \"%s\"" ), IOBJECT( expr->sym )->name );
-	if( expr->sym->tool ) {
-		Tool *tool = expr->sym->tool;
-
-		if( tool->lineno != -1 && FILEMODEL( tool->kit )->filename )
-			buf_appendf( buf, " (%s:%d)", 
-				FILEMODEL( tool->kit )->filename, 
-				tool->lineno );
-	}
+	if( expr->sym->tool ) 
+		tool_error( expr->sym->tool, buf );
 	else if( expr->row ) {
 		Workspace *ws = expr->row->ws;
 
@@ -88,13 +82,6 @@ expr_error_print_sub( Expr *expr, BufInfo *buf )
 	return( NULL );
 }
 
-void
-expr_error_print_all( BufInfo *buf )
-{
-	slist_map( expr_error_all, 
-		(SListMapFn) expr_error_print_sub, buf );
-}
-
 static Expr *
 expr_map_all_sub( Symbol *sym, map_expr_fn fn, void *a )
 {
