@@ -46,6 +46,14 @@ typedef struct _FileselFileType {
   const char **suffixes;
 } FileselFileType;
 
+/* Basic types.
+ */
+extern FileselFileType filesel_tfile_type, filesel_jfile_type, 
+       filesel_nfile_type, filesel_vfile_type, filesel_pfile_type, 
+       filesel_afile_type, filesel_wfile_type, filesel_rfile_type, 
+       filesel_mfile_type, filesel_cfile_type, filesel_xfile_type, 
+       filesel_dfile_type, filesel_ifile_type, filesel_sfile_type; 
+
 /* Suffix sets we support.
  */
 extern FileselFileType *filesel_type_workspace[];
@@ -110,6 +118,8 @@ typedef struct _FileselClass {
 	 */
 } FileselClass;
 
+gboolean is_file_type( const FileselFileType *type, const char *filename );
+
 typedef void *(*FileselMapFn)( Filesel *, const char *, void *, void * );
 
 void filesel_add_mode( char *filename );
