@@ -45,83 +45,22 @@ fontname_finalize( GObject *gobject )
 	G_OBJECT_CLASS( parent_class )->finalize( gobject );
 }
 
-static xmlNode *
-fontname_save( Model *model, xmlNode *xnode )
+static View *
+fontname_view_new( Model *model, View *parent )
 {
-	Fontname *fontname = FONTNAME( model );
-
-	xmlNode *xthis;
-
-	if( !(xthis = MODEL_CLASS( parent_class )->save( model, xnode )) )
-		return( NULL );
-
-	if( CLASSMODEL( model )->edited ) {
-		if( !set_prop( xthis, "value", "%s", fontname->value ) )
-			return( NULL );
-	}
-
-	return( xthis );
+	return( fontnameview_new() );
 }
 
-static gboolean
-fontname_load( Model *model, 
-	ModelLoadState *state, Model *parent, xmlNode *xnode )
-{
-	Fontname *fontname = FONTNAME( model );
-	char buf[MAX_STRSIZE];
-
-	assert( IS_RHS( parent ) );
-
-	if( get_sprop( xnode, "value", buf, MAX_STRSIZE ) ) {
-		classmodel_set_edited( CLASSMODEL( model ), TRUE );
-		SETSTR( fontname->value, buf );
-	}
-
-	return( MODEL_CLASS( parent_class )->load( model, 
-		state, parent, xnode ) );
-}
-
-/* Update Fontname from heap.
+/* Members of fontname we automate.
  */
-static gboolean
-fontname_class_get( Classmodel *classmodel, PElement *root )
-{
-	Fontname *fontname = FONTNAME( classmodel );
-	char caption[MAX_STRSIZE];
-	char value[MAX_STRSIZE];
-
-	if( !class_get_member_string( root, 
-		MEMBER_CAPTION, caption, MAX_STRSIZE ) ||
-		!class_get_member_string( root, 
-			MEMBER_VALUE, value, MAX_STRSIZE ) )
-		return( FALSE );
-	iobject_set( IOBJECT( classmodel ), NULL, caption );
-	SETSTR( fontname->value, value );
-
-	return( TRUE );
-}
-
-/* Make a new "fn value" application.
- */
-static gboolean
-fontname_class_new( Classmodel *classmodel, PElement *fn, PElement *out )
-{
-	Heap *heap = reduce_context->heap;
-	Fontname *fontname = FONTNAME( classmodel );
-	PElement rhs;
-
-	/* Make application nodes.
-	 */
-	heap_appl_init( out, fn );
-	if( !heap_appl_add( heap, out, &rhs ) ||
-		!heap_string_new( heap, 
-			IOBJECT( classmodel )->caption, &rhs ) ||
-		!heap_appl_add( heap, out, &rhs ) ||
-		!heap_string_new( heap, fontname->value, &rhs ) )
-		return( FALSE );
-
-	return( TRUE );
-}
+static ClassmodelMember fontname_members[] = {
+	{ CLASSMODEL_MEMBER_STRING, NULL, 0,
+		MEMBER_CAPTION, "caption", N_( "Caption" ),
+		G_STRUCT_OFFSET( iObject, caption ) },
+	{ CLASSMODEL_MEMBER_STRING, NULL, 0,
+		MEMBER_VALUE, "value", N_( "Value" ),
+		G_STRUCT_OFFSET( Fontname, value ) }
+};
 
 static void
 fontname_class_init( FontnameClass *class )
@@ -134,16 +73,14 @@ fontname_class_init( FontnameClass *class )
 
 	gobject_class->finalize = fontname_finalize;
 
-	model_class->view_new = fontnameview_new;
-	model_class->save = fontname_save;
-	model_class->load = fontname_load;
-
-	classmodel_class->class_get = fontname_class_get;
-	classmodel_class->class_new = fontname_class_new;
+	model_class->view_new = fontname_view_new;
 
 	/* Static init.
 	 */
 	model_register_loadable( MODEL_CLASS( class ) );
+
+	classmodel_class->members = fontname_members;
+	classmodel_class->n_members = IM_NUMBER( fontname_members );
 }
 
 static void
@@ -152,6 +89,7 @@ fontname_init( Fontname *fontname )
 	/* Overridden later. Just something sensible.
 	 */
 	fontname->value = NULL;
+	IM_SETSTR( fontname->value, "Sans" );
 
 	iobject_set( IOBJECT( fontname ), CLASS_FONTNAME, NULL );
 }
@@ -180,14 +118,3 @@ fontname_get_type( void )
 
 	return( type );
 }
-
-Classmodel *
-fontname_new( Rhs *rhs )
-{
-	Fontname *fontname;
-
-	fontname = FONTNAME( g_object_new( TYPE_FONTNAME, NULL ) );
-	icontainer_child_add( ICONTAINER( rhs ), ICONTAINER( fontname ), -1 );
-
-	return( CLASSMODEL( fontname ) );
-}
