@@ -48,6 +48,8 @@ typedef struct _Formula {
         gboolean edit;              	/* In edit mode */
         gboolean sensitive;            	/* Flick to edit on click */
         gboolean changed;            	/* ->entry changed since we set it */
+	gboolean refresh_queued;	/* Awaiting refresh */
+	gboolean needs_focus;		/* Grab focus on refresh */
 
 	/* Widgets.
 	 */
@@ -72,6 +74,7 @@ typedef struct _FormulaClass {
 
 void formula_set_edit( Formula *formula, gboolean edit );
 void formula_set_sensitive( Formula *formula, gboolean sensitive );
+void formula_set_needs_focus( Formula *formula, gboolean needs_focus );
 gboolean formula_scan( Formula *formula );
 
 GtkType formula_get_type( void );
