@@ -487,7 +487,7 @@ graph_pointer( PElement *root )
 	printf( "%s\n", buf_all( &buf ) );
 }
 
-static void
+static void *
 save_image( Imageinfo *ii, char *filename )
 {
 #ifdef DEBUG
@@ -495,9 +495,11 @@ save_image( Imageinfo *ii, char *filename )
 #endif /*DEBUG*/
 
 	if( !imageinfo_write( ii, NULL, filename ) ) 
-		error( error_get_top() );
+		return( ii );
 
 	increment_filename( filename );
+
+	return( NULL );
 }
 
 /* Write all images to files. Generate filenames with increment-file,
@@ -509,16 +511,18 @@ save_objects( PElement *base, char *filename )
 	gboolean result;
 	Imageinfo *ii;
 
-	if( PEISIMAGE( base ) && (ii = PEGETII( base )) ) 
-		save_image( ii, filename );
+	if( PEISIMAGE( base ) && (ii = PEGETII( base )) ) {
+		if( save_image( ii, filename ) )
+			return( base );
+	}
 	else if( PEISCLASS( base ) ) {
 		if( !heap_is_instanceof( CLASS_IMAGE, base, &result ) )
 			error( error_get_top() );
 		if( result ) {
 			if( !class_get_member_image( base, MEMBER_VALUE, &ii ) )
 				error( error_get_top() );
-			if( ii )
-				save_image( ii, filename );
+			if( ii && save_image( ii, filename ) )
+				return( base );
 		}
 
 		if( !heap_is_instanceof( CLASS_MATRIX, base, &result ) )
@@ -533,9 +537,11 @@ save_objects( PElement *base, char *filename )
 			increment_filename( filename );
 		}
 	}
-	else if( PEISLIST( base ) )
-		heap_map_list( base, 
-			(heap_map_list_fn) save_objects, filename, NULL );
+	else if( PEISLIST( base ) ) {
+		if( heap_map_list( base, 
+			(heap_map_list_fn) save_objects, filename, NULL ) )
+			return( base );
+	}
 
 	return( NULL );
 }
