@@ -38,4 +38,4 @@ void graph_value( PElement *root );
 
 /* Make sure all images are written to disc, and won't be deleted on exit.
  */
-void *set_output( PElement *base );
+void *save_objects( PElement *base, char *filename );
