@@ -33,7 +33,7 @@
 
 #include "ip.h"
 
-static ClassmodelClass *parent_class = NULL;
+static ValueClass *parent_class = NULL;
 
 static void
 group_finalize( GObject *gobject )
@@ -51,26 +51,10 @@ group_finalize( GObject *gobject )
 
 	/* My instance finalize stuff.
 	 */
-	buf_destroy( &group->caption_buffer );
 
 	G_OBJECT_CLASS( parent_class )->finalize( gobject );
 }
 
-static const char *
-group_generate_caption( iObject *iobject )
-{
-	Group *group = GROUP( iobject );
-	Expr *expr;
-
-	if( !HEAPMODEL( group )->row ||
-		!(expr = HEAPMODEL( group )->row->expr) )
-		return( _( "No group" ) );
-	buf_rewind( &group->caption_buffer );
-	itext_make_value_string( expr, &group->caption_buffer );
-
-	return( buf_all( &group->caption_buffer ) );
-}
-
 static gboolean
 group_save_list( PElement *list, GtkWidget *parent, char *filename );
 
@@ -80,7 +64,7 @@ group_save_item( PElement *item, GtkWidget *parent, char *filename )
 	gboolean result;
 	Imageinfo *ii;
 
-	if( !heap_isinstanceof( CLASS_IMAGE, item, &result ) )
+	if( !heap_is_instanceof( CLASS_IMAGE, item, &result ) )
 		return( FALSE );
 	if( result ) {
 		PElement value;
@@ -91,7 +75,7 @@ group_save_item( PElement *item, GtkWidget *parent, char *filename )
 			return( FALSE );
 	}
 
-	if( !heap_isinstanceof( CLASS_MATRIX, item, &result ) )
+	if( !heap_is_instanceof( CLASS_MATRIX, item, &result ) )
 		return( FALSE );
 	if( result ) {
 		DOUBLEMASK *dmask;
@@ -172,8 +156,6 @@ static void
 group_class_init( GroupClass *class )
 {
 	GObjectClass *gobject_class = (GObjectClass *) class;
-	iObjectClass *iobject_class = (iObjectClass *) class;
-	ModelClass *model_class = (ModelClass *) class;
 	ClassmodelClass *classmodel_class = (ClassmodelClass *) class;
 
 	parent_class = g_type_class_peek_parent( class );
@@ -183,10 +165,6 @@ group_class_init( GroupClass *class )
 
 	gobject_class->finalize = group_finalize;
 
-	iobject_class->generate_caption = group_generate_caption;
-
-	model_class->view_new = groupview_new;
-
 	classmodel_class->graphic_save = group_graphic_save;
 
 	model_register_loadable( MODEL_CLASS( class ) );
@@ -196,7 +174,6 @@ static void
 group_init( Group *group )
 {
 	iobject_set( IOBJECT( group ), CLASS_GROUP, NULL );
-	buf_init_dynamic( &group->caption_buffer, MAX_LINELENGTH );
 }
 
 GType
@@ -217,20 +194,9 @@ group_get_type( void )
 			(GInstanceInitFunc) group_init,
 		};
 
-		type = g_type_register_static( TYPE_CLASSMODEL, 
+		type = g_type_register_static( TYPE_VALUE, 
 			"Group", &info, 0 );
 	}
 
 	return( type );
 }
-
-Classmodel *
-group_new( Rhs *rhs )
-{
-	Group *group;
-
-	group = GROUP( g_object_new( TYPE_GROUP, NULL ) );
-	icontainer_child_add( ICONTAINER( rhs ), ICONTAINER( group ), -1 );
-
-	return( CLASSMODEL( group ) );
-}
