@@ -36,19 +36,15 @@
 	(GTK_CHECK_CLASS_TYPE( (klass), TYPE_GROUP ))
 
 typedef struct _Group {
-	Classmodel model;
+	Value parent_object;
 
-	/* Private ... build iobject caption here.
-	 */
-	BufInfo caption_buffer;
 } Group;
 
 typedef struct _GroupClass {
-	ClassmodelClass parent_class;
+	ValueClass parent_class;
 
 	/* My methods.
 	 */
 } GroupClass;
 
 GType group_get_type( void );
-Classmodel *group_new( Rhs *rhs );
