@@ -728,17 +728,6 @@ get_geditable_pint( GtkWidget *text, int *n )
 	return( TRUE );
 }
 
-/* Set a name for a widget and all it's children ... so they pick up a style.
- */
-void 
-set_name( GtkWidget *widget, const char *name )
-{
-	gtk_widget_set_name( widget, name );
-	if( GTK_IS_CONTAINER( widget ) )
-		gtk_container_foreach( GTK_CONTAINER( widget ), 
-                           (GtkCallback) set_name, (char *) name );
-}
-
 /* Indent widget, label above.
  */
 GtkWidget *
@@ -961,6 +950,40 @@ filedrop_register( GtkWidget *widget, FiledropFunc fn, void *client )
 		GTK_SIGNAL_FUNC( filedrop_drag_data_received ), fdi );
 }
 
+/* Add symbol drag to the target list.
+ */
+void
+set_symbol_drag_type( GtkWidget *widget )
+{
+	static const GtkTargetEntry targets[] = {
+		{ "text/symbol", 0, TARGET_SYMBOL }
+	};
+
+	GtkTargetList *target_list;
+
+	if( !GTK_WIDGET_REALIZED( widget ) ) 
+		return;
+
+	/* We can't always set the dest types, since we're probably already a
+	 * filedrop. Just add to the target list.
+	 */
+	if( (target_list = 
+		gtk_drag_dest_get_target_list( widget )) )
+		gtk_target_list_add_table( target_list, 
+			targets, IM_NUMBER( targets ) );
+	else
+		gtk_drag_dest_set( widget,
+			GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_MOTION | 
+				GTK_DEST_DEFAULT_DROP,
+			targets, IM_NUMBER( targets ),
+			GDK_ACTION_COPY );
+
+	gtk_drag_source_set( widget,
+		GDK_BUTTON1_MASK | GDK_BUTTON3_MASK,
+		targets, IM_NUMBER( targets ),
+		GDK_ACTION_COPY | GDK_ACTION_MOVE );
+}
+
 typedef struct _Listen {
 	GObject *gobject;		/* This object */
 	GObject *source;		/* Listens for signals from this */
@@ -1100,3 +1123,119 @@ vfatal( GError **error )
 
 	exit( -1 );
 } 
+
+char *
+text_view_get_text( GtkTextView *text_view )
+{ 
+	GtkTextBuffer *text_buffer = gtk_text_view_get_buffer( text_view );
+	GtkTextIter start_iter;
+	GtkTextIter end_iter;
+	char *text;
+
+	gtk_text_buffer_get_start_iter( text_buffer, &start_iter );
+	gtk_text_buffer_get_end_iter( text_buffer, &end_iter );
+	text = gtk_text_buffer_get_text( text_buffer, 
+		&start_iter, &end_iter, FALSE ); 
+
+	return( text );
+}
+
+void
+text_view_set_text( GtkTextView *text_view, 
+	const char *text, gboolean editable )
+{
+	gtk_text_buffer_set_text( gtk_text_view_get_buffer( text_view ),
+		text ? text : "", -1 );
+
+	gtk_text_view_set_editable( text_view, editable );
+	gtk_text_view_set_cursor_visible( text_view, editable );
+}
+
+void
+text_view_select_text( GtkTextView *text_view, int start, int end )
+{
+	GtkTextBuffer *text_buffer = gtk_text_view_get_buffer( text_view );
+	GtkTextMark *mark = gtk_text_buffer_get_insert( text_buffer );
+	GtkTextIter start_iter;
+	GtkTextIter end_iter;
+
+	gtk_text_buffer_get_iter_at_offset( text_buffer, &start_iter, start );
+	gtk_text_buffer_get_iter_at_offset( text_buffer, &end_iter, end );
+	gtk_text_buffer_select_range( text_buffer, &start_iter, &end_iter );
+	gtk_text_view_scroll_mark_onscreen( text_view, mark );
+}
+
+/* If parent dies, kill us too. Parent can be anything, but child must be an
+ * iobject.
+ */
+typedef struct _DestroyIfDestroyed {
+	GObject *child;
+	GObject *parent;
+	DestroyFn destroy_fn;
+} DestroyIfDestroyed;
+
+static void destroy_if_destroyed_parent_cb( DestroyIfDestroyed *difd, 
+	GObject *parent );
+static void destroy_if_destroyed_child_cb( DestroyIfDestroyed *difd, 
+	GObject *child );
+
+static void
+destroy_if_destroyed_parent_cb( DestroyIfDestroyed *difd, GObject *parent )
+{
+	GObject *child;
+	DestroyFn destroy_fn;
+
+#ifdef DEBUG
+	printf( "destroy_if_destroyed_parent_cb: %p\n", difd );
+#endif /*DEBUG*/
+
+	/* Destroying the child will trigger the other half of difd, make sure
+	 * we remove the link first.
+	 */
+	child = difd->child;
+	destroy_fn = difd->destroy_fn;
+	g_object_weak_unref( difd->child, 
+		(GWeakNotify) destroy_if_destroyed_child_cb, difd );
+	destroy_fn( child );
+
+	difd->child = NULL;
+	difd->parent = NULL;
+	difd->destroy_fn = NULL;
+	g_free( difd );
+}
+
+static void
+destroy_if_destroyed_child_cb( DestroyIfDestroyed *difd, GObject *child )
+{
+#ifdef DEBUG
+	printf( "destroy_if_destroyed_child_cb: %p\n", difd );
+#endif /*DEBUG*/
+
+	g_object_weak_unref( difd->parent, 
+		(GWeakNotify) destroy_if_destroyed_parent_cb, difd );
+
+	difd->child = NULL;
+	difd->parent = NULL;
+	difd->destroy_fn = NULL;
+	g_free( difd );
+}
+
+void
+destroy_if_destroyed( GObject *child, GObject *parent, DestroyFn destroy_fn )
+{
+	DestroyIfDestroyed *difd = g_new( DestroyIfDestroyed, 1 );
+
+#ifdef DEBUG
+	printf( "destroy_if_destroyed %p: parent=%p, child=%p\n", 
+		difd, parent, child );
+#endif /*DEBUG*/
+
+	difd->child = child;
+	difd->parent = parent;
+	difd->destroy_fn = destroy_fn;
+
+	g_object_weak_ref( parent, 
+		(GWeakNotify) destroy_if_destroyed_parent_cb, difd );
+	g_object_weak_ref( child, 
+		(GWeakNotify) destroy_if_destroyed_child_cb, difd );
+}
