@@ -110,10 +110,6 @@ gboolean get_geditable_int( GtkWidget *text, int *n );
 gboolean get_geditable_uint( GtkWidget *text, int *n );
 gboolean get_geditable_pint( GtkWidget *text, int *n );
 
-/* Set a name for a widget and all its children.
- */
-void set_name( GtkWidget *widget, const char *name );
-
 /* Make widget groups.
  */
 GtkWidget *build_glabelframe2( GtkWidget *box, const char *label );
@@ -128,6 +124,15 @@ GtkWidget *build_goption( GtkWidget *box, GtkSizeGroup *group,
 typedef gboolean (*FiledropFunc)( void *client, const char *file );
 void filedrop_register( GtkWidget *widget, FiledropFunc fn, void *client );
 
+/* Tag our thumbnail drag-n-drops with these. Start up a bit to leave room for 
+ * filedrop.
+ */
+enum {
+	TARGET_SYMBOL = 99
+};
+
+void set_symbol_drag_type( GtkWidget *widget );
+
 void listen_add( GObject *gobject, GObject **zap, 
 	const char *name, GCallback gcallback );
 
@@ -140,3 +145,12 @@ int get_dpi( void );
 GtkWidget *image_new_from_file( const char *name );
 
 void vfatal( GError **error );
+
+char *text_view_get_text( GtkTextView *text_view );
+void text_view_set_text( GtkTextView *text_view, 
+	const char *text, gboolean editable );
+void text_view_select_text( GtkTextView *text_view, int start, int end );
+
+typedef void (*DestroyFn)( GObject * );
+void destroy_if_destroyed( GObject *child, GObject *parent, 
+	DestroyFn destroy_fn );
