@@ -40,6 +40,10 @@
 #define DEBUG_STATIC
  */
 
+/* Count GCs and %full, handy for tuning.
+#define DEBUG_GETMEM
+ */
+
 #include "ip.h"
 
 static iObjectClass *parent_class = NULL;
@@ -806,6 +810,9 @@ heap_getmem( Heap *heap )
 {
 	HeapNode *hn;
 	int pcused;
+#ifdef DEBUG_GETMEM
+	static int n_heap_getmem = 0;
+#endif /*DEBUG_GETMEM*/
 
 	/* Easy case ... this should be handled by the NEWNODE macro, but do
 	 * it here as well just in case.
@@ -828,9 +835,21 @@ heap_getmem( Heap *heap )
 	/* Is heap over x% full? Add another heap block if we can.
 	 */
 	pcused = 100 * (heap->ncells - heap->nfree) / heap->ncells;
-	if( pcused > 70 ) {
-		if( !heapblock_create( heap, heap->rsz ) )
-			return( NULL );
+#ifdef DEBUG_GETMEM
+	n_heap_getmem += 1;
+	printf( "heap_getmem: %d%% (%d)\n", pcused, n_heap_getmem );
+#endif /*DEBUG_GETMEM*/
+
+	if( pcused > 50 ) {
+		int nblocks = 1 + (heap->ncells - heap->nfree) / heap->rsz;
+		int i;
+
+#ifdef DEBUG_GETMEM
+		printf( "heap_getmem: %d more blocks added\n", nblocks );
+#endif /*DEBUG_GETMEM*/
+		for( i = 0; i < nblocks; i++ )
+			if( !heapblock_create( heap, heap->rsz ) )
+				return( NULL );
 	}
 
 	if( !heap->free ) {
@@ -1145,6 +1164,33 @@ heap_realvec_new( Heap *heap, int n, double *vec, PElement *out )
 	return( TRUE );
 }
 
+/* Make a realvec, but from an int*.
+ */
+gboolean
+heap_intvec_new( Heap *heap, int n, int *vec, PElement *out )
+{
+	PElement list = *out;
+	int i;
+
+	/* Make first RHS ... the end of the list. 
+	 */
+	heap_list_init( &list );
+
+	/* Build a CONS node for each element. 
+	 */
+	for( i = 0; i < n; i++ ) {
+		PElement t;
+
+		if( !heap_list_add( heap, &list, &t ) )
+			return( FALSE );
+		if( !heap_real_new( heap, (double) vec[i], &t ) )
+			return( FALSE );
+		(void) heap_list_next( &list );
+	}
+
+	return( TRUE );
+}
+
 /* Make a matrix.
  */
 gboolean
@@ -1265,28 +1311,35 @@ heap_map_dict_entry( PElement *head, HeapMapDict *map_dict )
 	void *result;
 
 	if( !reduce_pelement( rc, reduce_spine, head ) )
-		return( FALSE );
-	if( !PEISFLIST( head ) ) 
+		return( head );
+	if( !PEISFLIST( head ) ) {
 		heap_error_typecheck( head, "heap_map_dict", "list" );
+		return( head );
+	}
 	PEGETHD( &p1, head );
 	if( !heap_get_string( &p1, key, 256 ) )
-		return( FALSE );
+		return( head );
 
 	PEGETTL( &p2, head );
 	if( !reduce_pelement( rc, reduce_spine, &p2 ) )
-		return( FALSE );
-	if( !PEISFLIST( &p2 ) ) 
+		return( head );
+	if( !PEISFLIST( &p2 ) ) {
 		heap_error_typecheck( &p2, "heap_map_dict", "list" );
+		return( head );
+	}
 	PEGETHD( &p1, &p2 );
-	result = map_dict->fn( key, &p1, map_dict->a, map_dict->b );
+	if( (result = map_dict->fn( key, &p1, map_dict->a, map_dict->b )) )
+		return( result );
 
 	PEGETTL( &p1, &p2 );
 	if( !reduce_pelement( rc, reduce_spine, &p1 ) )
-		return( FALSE );
-	if( !PEISELIST( &p1 ) ) 
+		return( head );
+	if( !PEISELIST( &p1 ) ) {
 		heap_error_typecheck( &p1, "heap_map_dict", "[]" );
+		return( head );
+	}
 
-	return( result );
+	return( NULL );
 }
 
 /* Map over a list of ["key", value] pairs.
@@ -1549,6 +1602,19 @@ heap_list_length( PElement *base )
 	return( result );
 }
 
+int
+heap_list_length_max( PElement *base, int max_length )
+{
+	Reduce *rc = reduce_context;
+	int result;
+
+	REDUCE_CATCH_START( -1 );
+	result = reduce_list_length_max( rc, base, max_length );
+	REDUCE_CATCH_STOP; 
+
+	return( result );
+}
+
 gboolean
 heap_list_index( PElement *base, int n, PElement *out )
 {
@@ -1715,8 +1781,12 @@ heap_copy( Heap *heap, Compile *compile, PElement *out )
 		break;
 
 	case ELEMENT_NOVAL:
-		/* Nothing to copy. 
+		/* Not compiled yet: compile now, then copy.
 		 */
+		if( compile_object( compile ) )
+			return( FALSE );
+		if( !heap_copy( heap, compile, out ) )
+			return( FALSE );
 		break;
 
 	case ELEMENT_MANAGED:
