@@ -226,7 +226,7 @@ typedef struct pelement {
 	IS_MANAGEDGOBJECT( PEGETVAL(P) ))
 #define PEISIMAGE(P) (PEISMANAGED(P) && IS_IMAGEINFO( PEGETVAL(P) ))
 #define PEISVIPSOBJECT(P) \
-		(PEISMANAGEDGOBJECT(P) && VIPS_IS_OBJECT( PEGETMANAGEDGOBJECT(P) )) 
+	(PEISMANAGEDGOBJECT(P) && VIPS_IS_OBJECT( PEGETMANAGEDGOBJECT(P) )) 
 #define PEISFILE(P) (PEISMANAGED(P) && IS_MANAGEDFILE( PEGETVAL(P) ))
 #define PEISLIST(P) (PEISELIST(P) || PEISFLIST(P))
 #define PEISNOVAL(P) (PEGETTYPE(P) == ELEMENT_NOVAL)
@@ -390,6 +390,7 @@ gboolean heap_complex_element_new( Heap *heap,
 	PElement *rp, PElement *ip, PElement *out );
 gboolean heap_complex_new( Heap *heap, double rp, double ip, PElement *out );
 gboolean heap_realvec_new( Heap *heap, int n, double *vec, PElement *out );
+gboolean heap_intvec_new( Heap *heap, int n, int *vec, PElement *out );
 void heap_list_init( PElement *list );
 gboolean heap_list_add( Heap *heap, PElement *list, PElement *data );
 gboolean heap_list_next( PElement *list );
@@ -432,6 +433,7 @@ gboolean heap_is_instanceof_exact( const char *name, PElement *klass,
 gboolean heap_is_instanceof( const char *name, PElement *klass, gboolean *out );
 
 int heap_list_length( PElement *base );
+int heap_list_length_max( PElement *base, int max_length );
 gboolean heap_list_index( PElement *base, int n, PElement *out );
 gboolean heap_reduce_strict( PElement *base );
 
