@@ -39,24 +39,12 @@
 #define IIMAGE_GET_CLASS( obj ) \
 	(G_TYPE_INSTANCE_GET_CLASS( (obj), TYPE_IIMAGE, iImageClass ))
 
-/* Our instance vars ... packaged up for code sharing.
- */
-typedef struct {
-        Imageinfo *ii;
-
-	/* Can get "changed" for reload if the file changes behind our backs.
-	 * Recalc the classmodel if this happens.
-	 */
-	guint file_changed_sid;
-	Classmodel *classmodel;
-} iImageInstance;
-
 struct _iImage {
 	Classmodel parent_class;
 
 	/* Class fields.
 	 */
-	iImageInstance instance;
+	ImageValue value;
 
 	/* List of classmodel which have displays on us.
 	 */
@@ -68,8 +56,6 @@ struct _iImage {
 
 	/* Track display pos/size/etc. here.
 	 */
-	int window_x, window_y;		/* Last window position at quit */
-	int window_width, window_height;
 	int image_left, image_top;	/* Scroll position */
 	int image_mag;			/* Scale */
 
@@ -80,9 +66,11 @@ struct _iImage {
 	gboolean show_convert;
 	gboolean show_rulers;	
 
-	/* Bar settings.
+	/* Bar settings we remember.
 	 */
 	double scale, offset;
+	gboolean falsecolour;
+	gboolean type;
 
 	/* Private ... build iobject caption here.
 	 */
@@ -98,4 +86,3 @@ typedef struct _iImageClass {
 
 GType iimage_get_type( void );
 gboolean iimage_replace( iImage *iimage, const char *filename );
-Classmodel *iimage_new( Rhs *rhs );
