@@ -420,19 +420,33 @@ imageinfo_undo_free( Imageinfo *imageinfo )
 }
 
 static void
-imageinfo_dispose_eval( Imageinfo *imageinfo )
+imageinfo_set_eval_parent( Imageinfo *imageinfo, GtkWidget *eval_parent )
 {
-	if( imageinfo->eval_progress ) {
-		gtk_widget_destroy( GTK_WIDGET( imageinfo->eval_progress ) );
-		imageinfo->eval_progress = NULL;
-	}
-	IM_FREEF( g_timer_destroy, imageinfo->eval_timer );
 	if( imageinfo->eval_parent ) {
 		g_object_remove_weak_pointer( 
 			G_OBJECT( imageinfo->eval_parent ), 
 			(gpointer *) &imageinfo->eval_parent );
 		imageinfo->eval_parent = NULL;
 	}
+	imageinfo->eval_parent = eval_parent;
+	if( imageinfo->eval_parent ) 
+		g_object_add_weak_pointer( G_OBJECT( eval_parent ), 
+			(gpointer *) &imageinfo->eval_parent );
+}
+
+static void
+imageinfo_dispose_eval( Imageinfo *imageinfo )
+{
+	DESTROY_GTK( imageinfo->eval_progress );
+	imageinfo_set_eval_parent( imageinfo, NULL );
+	imageinfo->monitored = FALSE;
+
+	/* Make sure any callbacks from the IMAGE stop working.
+	 */
+	if( imageinfo->proxy ) {
+		imageinfo->proxy->imageinfo = NULL;
+		imageinfo->proxy = NULL;
+	}
 }
 
 static void
@@ -600,6 +614,8 @@ imageinfo_init( Imageinfo *imageinfo )
 	imageinfo->mapped_im = NULL;
 	imageinfo->identity_lut = NULL;
 	imageinfo->underlying = NULL;
+	imageinfo->proxy = NULL;
+
 	imageinfo->dfile = FALSE;
 	imageinfo->from_file = FALSE;
 	imageinfo->mtime = 0;
@@ -609,7 +625,7 @@ imageinfo_init( Imageinfo *imageinfo )
 	imageinfo->redo = NULL;
 	imageinfo->cundo = NULL;
 
-	imageinfo->eval_timer = NULL;
+	imageinfo->monitored = FALSE;
 	imageinfo->eval_progress = NULL;
 
 	imageinfo->check_mtime = 0;
@@ -641,6 +657,135 @@ imageinfo_get_type( void )
 	return( type );
 }
 
+static int
+imageinfo_progress_start( ImageinfoIMAGE *proxy )
+{
+	Imageinfo *imageinfo = proxy->imageinfo;
+
+	if( imageinfo )
+		/* Note the start time.
+		 */
+		imageinfo->eval_last = 
+			g_timer_elapsed( imageinfo->im->time->start, NULL );
+
+	return( 0 );
+}
+
+static int
+imageinfo_progress_eval( ImageinfoIMAGE *proxy )
+{
+	const double update_threshold = 0.2;
+	Imageinfo *imageinfo = proxy->imageinfo;
+
+	if( imageinfo ) {
+		double latest = 
+			g_timer_elapsed( imageinfo->im->time->start, NULL );
+		double elapsed = latest - imageinfo->eval_last;
+
+		if( imageinfo->im->time->eta > 1 &&
+			!imageinfo->eval_progress ) {
+			imageinfo->eval_progress = progress_new( imageinfo );
+			iwindow_set_parent( IWINDOW( imageinfo->eval_progress ),
+				imageinfo->eval_parent );
+			idialog_set_iobject( 
+				IDIALOG( imageinfo->eval_progress ), 
+				IOBJECT( imageinfo ) );
+			iwindow_build( IWINDOW( imageinfo->eval_progress ) );
+			gtk_widget_show( 
+				GTK_WIDGET( imageinfo->eval_progress ) );
+		}
+
+		if( elapsed > update_threshold ) {
+			if( imageinfo->eval_progress ) 
+				progress_update( imageinfo->eval_progress,
+					imageinfo->im->time->percent,
+					imageinfo->im->time->eta );
+
+			while( g_main_context_iteration( NULL, FALSE ) )
+				;
+
+			if( imageinfo->eval_progress &&
+				imageinfo->eval_progress->cancelled )
+				imageinfo->im->kill = 1;
+
+			imageinfo->eval_last = latest;
+		}
+	}
+
+	return( 0 );
+}
+
+static int
+imageinfo_progress_stop( ImageinfoIMAGE *proxy )
+{
+	Imageinfo *imageinfo = proxy->imageinfo;
+
+	/* Pop down the dialog.
+	 */
+	if( imageinfo ) 
+		DESTROY_GTK( imageinfo->eval_progress );
+
+	return( 0 );
+}
+
+static int
+imageinfo_progress_close( ImageinfoIMAGE *proxy )
+{
+	Imageinfo *imageinfo = proxy->imageinfo;
+
+	/* Remove everything related to progress.
+	 */
+	if( imageinfo ) 
+		imageinfo_dispose_eval( imageinfo );
+
+	return( 0 );
+}
+
+/* Add a progress box for an image evaluation.
+ */
+static void
+imageinfo_progress_add( Imageinfo *imageinfo, GtkWidget *eval_parent )
+{
+	/* Only if we're running interactively.
+	 */
+	if( !main_window_top )
+		return;
+
+	/* Always update eval_parent. 
+	 */
+	imageinfo_set_eval_parent( imageinfo, eval_parent );
+
+	/* Already being monitored?
+	 */
+	if( imageinfo->monitored ) 
+		return;
+	imageinfo->monitored = TRUE;
+
+        /* Need a proxy on IMAGE.
+         */ 
+	g_assert( !imageinfo->proxy );
+	if( !(imageinfo->proxy = IM_NEW( imageinfo->im, ImageinfoIMAGE )) )
+		return;
+	imageinfo->proxy->im = imageinfo->im;
+	imageinfo->proxy->imageinfo = imageinfo;
+
+	(void) im_add_evalstart_callback( imageinfo->im, 
+		(im_callback_fn) imageinfo_progress_start, 
+		imageinfo->proxy, NULL );
+	(void) im_add_eval_callback( imageinfo->im, 
+		(im_callback_fn) imageinfo_progress_eval, 
+		imageinfo->proxy, NULL );
+	(void) im_add_evalend_callback( imageinfo->im, 
+		(im_callback_fn) imageinfo_progress_stop, 
+		imageinfo->proxy, NULL );
+
+	/* On close, remove everything.
+	 */
+	(void) im_add_close_callback( imageinfo->im, 
+		(im_callback_fn) imageinfo_progress_close, 
+		imageinfo->proxy, NULL );
+}
+
 /* Make a basic imageinfo. No refs, will be destroyed on next GC. If name is
  * NULL, make a temp name up; otherwise name needs to be unique.
  */
@@ -675,28 +820,10 @@ imageinfo_new( Imageinfogroup *imageinfogroup,
 
 	icontainer_child_add( ICONTAINER( imageinfogroup ),
 		ICONTAINER( imageinfo ), -1 );
-
-	return( imageinfo );
-}
-
-/* Make a temp image. Deleted on close. No refs: closed on next GC. If you
- * want it to stick around, ref it!
- */
-Imageinfo *
-imageinfo_new_temp( Imageinfogroup *imageinfogroup, 
-	Heap *heap, const char *name, const char *mode )
-{
-	IMAGE *im;
-	Imageinfo *imageinfo;
-	char tname[FILENAME_MAX];
-
-	if( !temp_name( tname, "v" ) || !(im = im_open( tname, mode )) )
-		return( NULL );
-	if( !(imageinfo = imageinfo_new( imageinfogroup, heap, im, name )) ) {
-		im_close( im );
-		return( NULL );
-	}
-	imageinfo->dfile = TRUE;
+	
+	/* Sometimes overridden later to set a real parent.
+	 */
+	imageinfo_progress_add( imageinfo, NULL );
 
 	return( imageinfo );
 }
@@ -721,93 +848,26 @@ imageinfo_new_modlut( Imageinfogroup *imageinfogroup,
 	return( top_imageinfo );
 }
 
-static int
-imageinfo_progress_close( Imageinfo *imageinfo )
-{
-	imageinfo_dispose_eval( imageinfo );
-
-	return( 0 );
-}
-
-static int
-imageinfo_progress_eval( Imageinfo *imageinfo )
-{
-	const double start_threshold = 0.5;
-        const double update_threshold = 0.2;
-
-	double latest = g_timer_elapsed( imageinfo->eval_timer, NULL );
-	double elapsed = latest - imageinfo->eval_last;
-
-	if( imageinfo->eval_progress ) {
-		if( elapsed > update_threshold ) {
-			progress_update( imageinfo->eval_progress,
-				imageinfo->im->time->percent,
-				imageinfo->im->time->eta );
-
-			while( g_main_context_iteration( NULL, FALSE ) )
-				;
-
-			imageinfo->eval_last = latest;
-
-			if( imageinfo->eval_progress->cancelled )
-				imageinfo->im->kill = 1;
-		}
-	}
-	else {
-		if( elapsed > start_threshold ) {
-			imageinfo->eval_progress = progress_new( imageinfo );
-			iwindow_set_parent( IWINDOW( imageinfo->eval_progress ),
-				imageinfo->eval_parent );
-			idialog_set_iobject( 
-				IDIALOG( imageinfo->eval_progress ), 
-				IOBJECT( imageinfo ) );
-			iwindow_build( IWINDOW( imageinfo->eval_progress ) );
-			progress_update( imageinfo->eval_progress, 0, 99 );
-			gtk_widget_show( 
-				GTK_WIDGET( imageinfo->eval_progress ) );
-
-			while( g_main_context_iteration( NULL, FALSE ) )
-				;
-
-			imageinfo->eval_last = latest;
-		}
-	}
-
-	return( 0 );
-}
-
-/* Add a progress box for an image evaluation.
+/* Make a temp image. Deleted on close. No refs: closed on next GC. If you
+ * want it to stick around, ref it!
  */
-static void
-imageinfo_progress_add( Imageinfo *imageinfo, GtkWidget *eval_parent )
+Imageinfo *
+imageinfo_new_temp( Imageinfogroup *imageinfogroup, 
+	Heap *heap, const char *name, const char *mode )
 {
-	/* Already being monitored.
-	 */
-	if( imageinfo->eval_timer )
-		return;
+	IMAGE *im;
+	Imageinfo *imageinfo;
+	char tname[FILENAME_MAX];
 
-	/* Only if we're running interactively.
-	 */
-	if( !main_window_top )
-		return;
+	if( !temp_name( tname, "v" ) || !(im = im_open( tname, mode )) )
+		return( NULL );
+	if( !(imageinfo = imageinfo_new( imageinfogroup, heap, im, name )) ) {
+		im_close( im );
+		return( NULL );
+	}
+	imageinfo->dfile = TRUE;
 
-	if( !imageinfo->eval_timer ) 
-		imageinfo->eval_timer = g_timer_new();
-	imageinfo->eval_last = g_timer_elapsed( imageinfo->eval_timer, NULL );
-	imageinfo->eval_parent = eval_parent;
-	if( eval_parent )
-		g_object_add_weak_pointer( G_OBJECT( eval_parent ), 
-			(gpointer *) &imageinfo->eval_parent );
-
-        /* Call to tidy up.
-         */ 
-        (void) im_add_close_callback( imageinfo->im, 
-                (im_callback_fn) imageinfo_progress_close, imageinfo, NULL );
-
-        /* Call to animate countdown. 
-         */ 
-	(void) im_add_eval_callback( imageinfo->im, 
-		(im_callback_fn) imageinfo_progress_eval, imageinfo, NULL );
+	return( imageinfo );
 }
 
 /* Need this context during imageinfo_open_image_input().
@@ -859,7 +919,6 @@ imageinfo_open_image_input( const char *filename, ImageinfoOpen *open )
 			open->heap, open->filename, "w" )) )
 			return( NULL );
 		MANAGED_REF( imageinfo );
-		imageinfo_progress_add( imageinfo, NULL );
 		if( im_tiff2vips( filename, imageinfo->im ) ||
 			im_histlin( imageinfo->im, "im_tiff2vips %s %s",
 				filename, imageinfo->im->filename ) ) {
@@ -878,7 +937,6 @@ imageinfo_open_image_input( const char *filename, ImageinfoOpen *open )
 			open->heap, open->filename, "w" )) )
 			return( NULL );
 		MANAGED_REF( imageinfo );
-		imageinfo_progress_add( imageinfo, NULL );
 		if( im_jpeg2vips( filename, imageinfo->im ) ||
 			im_histlin( imageinfo->im, "im_jpeg2vips %s %s",
 				filename, imageinfo->im->filename ) ) {
@@ -897,7 +955,6 @@ imageinfo_open_image_input( const char *filename, ImageinfoOpen *open )
 			open->heap, open->filename, "w" )) )
 			return( NULL );
 		MANAGED_REF( imageinfo );
-		imageinfo_progress_add( imageinfo, NULL );
 		if( im_png2vips( filename, imageinfo->im ) ||
 			im_histlin( imageinfo->im, "im_png2vips %s %s",
 				filename, imageinfo->im->filename ) ) {
@@ -978,7 +1035,6 @@ imageinfo_open_image_input( const char *filename, ImageinfoOpen *open )
 			open->heap, open->filename, "w" )) )
 			return( NULL );
 		MANAGED_REF( imageinfo );
-		imageinfo_progress_add( imageinfo, NULL );
 		if( im_magick2vips( filename, imageinfo->im ) ||
 			im_histlin( imageinfo->im, "im_magick2vips %s %s",
 				filename, imageinfo->im->filename ) ) {
@@ -1013,8 +1069,8 @@ imageinfo_open_image_input( const char *filename, ImageinfoOpen *open )
 	}
 
 	else {
-		im_errormsg( "\"%s\" is not in a supported image file format",
-			filename );
+		im_error( "open", _( "\"%s\" is not in a supported "
+			"image file format" ), filename );
 		return( NULL );
 	}
 
@@ -1023,7 +1079,12 @@ imageinfo_open_image_input( const char *filename, ImageinfoOpen *open )
 	if( im_pincheck( imageinfo->im ) ) 
 		return( NULL );
 
-	/* Attach the origiinal filename ... pick this up again later as a
+	/* The rewind will have removed everything from the IMAGE. Reattach
+	 * progress.
+	 */
+	imageinfo_progress_add( imageinfo, NULL );
+
+	/* Attach the original filename ... pick this up again later as a
 	 * save default.
 	 */
 	if( im_meta_set_string( imageinfo->im, ORIGINAL_FILENAME, filename ) )
