@@ -85,13 +85,22 @@ static void
 imagepresent_destroy( GtkObject *object )
 {
 	Imagepresent *ip = IMAGEPRESENT( object );
-	iImage *iimage = ip->imagemodel->iimage;
+
+#ifdef DEBUG
+	printf( "imagepresent_destroy\n" );
+#endif /*DEBUG*/
 
 	IM_FREEF( g_source_remove, ip->scroll_tid );
 	IM_FREEF( iwindow_cursor_context_destroy, ip->cntxt );
 	DESTROY_GTK( ip->ruler_menu );
-	if( iimage ) 
-		iimage->views = g_slist_remove( iimage->views, ip );
+
+	if( ip->imagemodel ) {
+		iImage *iimage = ip->imagemodel->iimage;
+
+		if( iimage ) 
+			iimage->views = g_slist_remove( iimage->views, ip );
+		UNREF( ip->imagemodel );
+	}
 
 	GTK_OBJECT_CLASS( parent_class )->destroy( object );
 
@@ -1024,7 +1033,7 @@ imagepresent_left_release( Imagepresent *ip, GdkEvent *ev, int x, int y )
 			/* In compatibility mode, we need to un-offset, since
 			 * mark/etc. will re-offset.
 			 */
-			if( row->ws->compatibility ) {
+			if( row->ws->compat_78 ) {
 				Conversion *conv = ip->id->conv;
 				IMAGE *im = imageinfo_get( FALSE, conv->ii );
 
@@ -1039,7 +1048,7 @@ imagepresent_left_release( Imagepresent *ip, GdkEvent *ev, int x, int y )
 				/* How annoying, this used to be called
 				 * "Point".
 				 */
-				if( row->ws->compatibility )
+				if( row->ws->compat_78 )
 					buf_appends( &buf, "Point " );
 				else
 					buf_appendf( &buf, "%s ", CLASS_MARK );
@@ -1379,7 +1388,11 @@ imagepresent_key_press_event_cb( GtkWidget *widget, GdkEventKey *event,
 		handled = TRUE;
 		break;
 
+	/* FIXME + and = are not always on the same key, of course :( 
+	 */
 	case GDK_i:
+	case GDK_plus:
+	case GDK_equal:
 		if( ip->inside ) 
 			imagepresent_zoom_in( ip, ip->last_x, ip->last_y );
 		else 
@@ -1387,13 +1400,14 @@ imagepresent_key_press_event_cb( GtkWidget *widget, GdkEventKey *event,
 		handled = TRUE;
 		break;
 
-	case GDK_0:
-		conversion_set_mag( conv, 0 );
+	case GDK_o: 
+	case GDK_minus:
+		imagepresent_zoom_out( ip );
 		handled = TRUE;
 		break;
 
-	case GDK_o: 
-		imagepresent_zoom_out( ip );
+	case GDK_0:
+		conversion_set_mag( conv, 0 );
 		handled = TRUE;
 		break;
 
@@ -1645,6 +1659,9 @@ static void
 imagepresent_link( Imagepresent *ip, Imagemodel *imagemodel )
 {
 	ip->imagemodel = imagemodel;
+	g_object_ref( G_OBJECT( ip->imagemodel ) );
+	iobject_sink( IOBJECT( ip->imagemodel ) );
+
 	g_signal_connect( G_OBJECT( imagemodel ), "changed", 
 		G_CALLBACK( imagepresent_imagemodel_changed_cb ), ip );
 	g_signal_connect( G_OBJECT( imagemodel ), "imageinfo_changed", 
