@@ -197,14 +197,6 @@ imageview_imagemodel_changed_cb( Imagemodel *imagemodel, Imageview *iv )
 	imageview_refresh_title( iv );
 }
 
-/* The model has been destroyed ... kill us too.
- */
-static void
-imageview_imagemodel_destroy_cb( Imagemodel *imagemodel, Imageview *iv )
-{
-	gtk_widget_destroy( GTK_WIDGET( iv ) );
-}
-
 /* Region class names indexed by iRegionType.
  */
 static const char *imageview_region_name[] = {
@@ -365,22 +357,13 @@ imageview_header_action_cb( GtkAction *action, Imageview *iv )
 	iImage *iimage = imagemodel->iimage;
 	Row *row = HEAPMODEL( iimage )->row;
 	Workspace *ws = row_get_workspace( row );
-	GtkWidget *imageheader = imageheader_new( imagemodel->conv );
 	BufInfo buf;
 	char txt[512];
 
 	buf_init_static( &buf, txt, 512 );
 	row_qualified_name_relative( ws->sym, row, &buf );
-
-	iwindow_set_title( IWINDOW( imageheader ), _( "Header for \"%s\"" ), 
-		buf_all( &buf ) );
-	idialog_set_callbacks( IDIALOG( imageheader ), 
-		NULL, NULL, NULL, iv );
-	idialog_add_ok( IDIALOG( imageheader ), iwindow_true_cb, _( "OK" ) );
-	iwindow_set_parent( IWINDOW( imageheader ), GTK_WIDGET( iv ) );
-	iwindow_build( IWINDOW( imageheader ) );
-
-	gtk_widget_show( imageheader );
+	conversion_header_dialog( imagemodel->conv, 
+		buf_all( &buf ), GTK_WIDGET( iv ) );
 }
 
 static void
@@ -731,9 +714,6 @@ imageview_build( Imageview *iv, GtkWidget *vbox, iImage *iimage )
 	iv->imagemodel_changed_sid = g_signal_connect( 
 		G_OBJECT( iv->imagemodel ), "changed", 
 		G_CALLBACK( imageview_imagemodel_changed_cb ), iv );
-	iv->imagemodel_destroy_sid = g_signal_connect( 
-		G_OBJECT( iv->imagemodel ), "destroy", 
-		G_CALLBACK( imageview_imagemodel_destroy_cb ), iv );
 
         /* Make main menu bar
          */
@@ -866,6 +846,8 @@ imageview_build( Imageview *iv, GtkWidget *vbox, iImage *iimage )
 	focus_chain = g_list_append( focus_chain, iv->cv );
 	focus_chain = g_list_append( focus_chain, iv->pbv );
 	gtk_container_set_focus_chain( GTK_CONTAINER( vbox ), focus_chain );
+	g_list_free( focus_chain );
+
 	gtk_widget_grab_focus( GTK_WIDGET( iv->ip->id ) );
 }
 
