@@ -142,15 +142,17 @@ iobject_real_changed( iObject *iobject )
 	iObjectClass *iobject_class = IOBJECT_GET_CLASS( iobject );
 
 	if( iobject_class->generate_caption )
-		SETSTR( iobject->caption, 
+		IM_SETSTR( iobject->caption, 
 			iobject_class->generate_caption( iobject ) );
 }
 
 static void
 iobject_real_info( iObject *iobject, BufInfo *buf )
 {
-	buf_appendf( buf, "name = \"%s\"\n", NN( iobject->name ) );
-	buf_appendf( buf, "caption = \"%s\"\n", NN( iobject->caption ) );
+	if( iobject->name )
+		buf_appendf( buf, "name = \"%s\"\n", iobject->name );
+	if( iobject->caption )
+		buf_appendf( buf, "caption = \"%s\"\n", iobject->caption );
 	buf_appendf( buf, "iObject :: \"%s\"\n", 
 		G_OBJECT_TYPE_NAME( iobject ) );
 }
@@ -246,10 +248,10 @@ iobject_test_name( iObject *iobject, const char *name )
 void *
 iobject_print( iObject *iobject )
 {
-	g_print( "%s \"%s\" (0x%x)\n", 
+	g_print( "%s \"%s\" (%p)\n", 
 		G_OBJECT_TYPE_NAME( iobject ),
 		NN( iobject->name ), 
-		GPOINTER_TO_UINT( iobject ) ); 
+		iobject ); 
 
 	return( NULL );
 }
@@ -263,11 +265,11 @@ iobject_set( iObject *iobject, const char *name, const char *caption )
 	g_return_if_fail( IS_IOBJECT( iobject ) );
 
 	if( name && name != iobject->name ) {
-		SETSTR( iobject->name, name );
+		IM_SETSTR( iobject->name, name );
 		changed = TRUE;
 	}
 	if( caption && caption != iobject->caption ) {
-		SETSTR( iobject->caption, caption );
+		IM_SETSTR( iobject->caption, caption );
 		changed = TRUE;
 	}
 
