@@ -27,6 +27,11 @@
 
  */
 
+/* We can get multiple includes sometimes, gah, thank you bison.
+ */
+#ifndef IP_H
+#define IP_H
+
 /* Turn off VIPS's old and broken defines, we don't need them.
  */
 #define IM_NO_VIPS7_COMPAT
@@ -125,8 +130,6 @@ extern int statfs();
 #include <gdk/gdkkeysyms.h>
 
 #include <vips/vips.h>
-#include <vips/history.h>
-#include <vips/time.h>
 
 /* Experimental vips8 links.
  */
@@ -184,7 +187,7 @@ typedef void VipsObject;
 				/* XML namespace ... note, not nip2! */
 #define MAX_LINELENGTH (120)	/* Max chars we display of value */
 #define MAX_RECENT (10)		/* Number of recent items in file menu */
-#define NIP_COPYRIGHT "%s: (c) 2007 Imperial College, London"
+#define NIP_COPYRIGHT "%s: (c) 2008 Imperial College, London"
 
 /* Our stock_ids.
  */
@@ -298,6 +301,8 @@ typedef struct _Toolkitgroupview Toolkitgroupview;
 typedef struct _Toolkitview Toolkitview;
 typedef struct _Toolview Toolview;
 typedef struct _Trace Trace;
+typedef struct _iError iError;
+typedef struct _Log Log;
 typedef struct _vObject vObject;
 typedef struct _View View;
 typedef struct _Workspace Workspace;
@@ -346,6 +351,7 @@ typedef void *(*tool_map_fn)( Tool *, void *, void * );
 #include "floatwindow.h"
 #include "imageview.h"
 #include "tslider.h"
+#include "pane.h"
 
 /* Basic ip includes (order important).
  */
@@ -390,7 +396,10 @@ typedef void *(*tool_map_fn)( Tool *, void *, void * );
 
 /* Per module includes, any order
  */
+#include "workspacedefs.h"
 #include "toolkitbrowser.h"
+#include "log.h"
+#include "error.h"
 #include "trace.h"
 #include "program.h"
 #include "conversionview.h"
@@ -455,3 +464,5 @@ typedef void *(*tool_map_fn)( Tool *, void *, void * );
 #ifdef WITH_DMALLOC
 #include <dmalloc.h>
 #endif /*WITH_DMALLOC*/
+
+#endif /*IP_H*/
