@@ -69,10 +69,10 @@ itextview_refresh( vObject *vobject )
 	printf( " (%p)\n", vobject );
 #endif /*DEBUG*/
 
-	/* Only reset edit mode if the text isn't being edited. We
+	/* Only reset edit mode if the text hasn't been changed. We
 	 * don't want the user to lose work.
 	 */
-	if( !itextview->formula->edit ) 
+	if( !itextview->formula->changed ) 
 		switch( row->ws->mode ) {
 		case WORKSPACE_MODE_REGULAR:
 			formula_set_edit( itextview->formula, FALSE );
