@@ -86,6 +86,7 @@ static gboolean main_option_no_load_args = FALSE;
 static gboolean main_option_stdin_ws = FALSE;
 static gboolean main_option_stdin_def = FALSE;
 static char *main_option_output = NULL;
+static char **main_option_set = NULL;
 static gboolean main_option_benchmark = FALSE;
 gboolean main_option_time_save = FALSE;
 gboolean main_option_i18n = FALSE;
@@ -104,6 +105,8 @@ static GOptionEntry main_option[] = {
 		N_( "write value of 'main' to FILE" ), "FILE" },
 	{ "batch", 'b', 0, G_OPTION_ARG_NONE, &main_option_batch, 
 		N_( "run in batch mode" ), NULL },
+	{ "set", '=', 0, G_OPTION_ARG_STRING_ARRAY, &main_option_set, 
+		N_( "set values" ), NULL },
 	{ "verbose", 'V', 0, G_OPTION_ARG_NONE, &main_option_verbose, 
 		N_( "verbose error output" ), NULL },
 	{ "no-load-menus", 'm', 0, G_OPTION_ARG_NONE, 
@@ -176,9 +179,15 @@ main_log_null( const char *log_domain, GLogLevelFlags log_level,
 /* Print all errors and quit. Batch mode only.
  */
 static void
-main_error_exit( const char *msg )
+main_error_exit( const char *fmt, ... )
 {
-	fprintf( stderr, "%s %s\n", 
+	va_list args;
+
+        va_start( args, fmt );
+        (void) vfprintf( stderr, fmt, args );
+        va_end( args );
+
+	fprintf( stderr, "\n%s %s\n", 
 		error_get_top(), error_get_sub() );
 
 	if( main_option_verbose ) {
@@ -186,14 +195,54 @@ main_error_exit( const char *msg )
 		char txt[MAX_STRSIZE];
 
 		buf_init_static( &buf, txt, MAX_STRSIZE );
-		expr_error_print_all( &buf );
+		slist_map( expr_error_all,
+			(SListMapFn) expr_error_print, &buf );
 		fprintf( stderr, "%s\n", buf_all( &buf ) );
-		error( msg );
 	}
 
 	exit( 1 );
 }
 
+/* Output a single main.
+ */
+static void
+main_print_main( Symbol *sym )
+{
+	PElement *root;
+
+	root = &sym->expr->root;
+	if( symbol_recalculate_check( sym ) || 
+		!reduce_pelement( reduce_context, reduce_spine_strict, root ) ) 
+		main_error_exit( _( "error calculating \"%s\"" ), 
+			symbol_name( sym ) );
+
+	if( main_option_output ) {
+		char filename[FILENAME_MAX];
+
+		im_strncpy( filename, main_option_output, FILENAME_MAX );
+		if( save_objects( root, filename ) )
+			main_error_exit( _( "error calculating \"%s\"" ), 
+				symbol_name( sym ) );
+	}
+
+	if( main_option_print_main )
+		graph_value( root );
+}
+
+static void *
+main_print_ws( Workspace *ws, gboolean *found )
+{
+	Symbol *sym;
+
+	if( (sym = compile_lookup( 
+		ws->sym->expr->compile, "main" )) ) {
+		main_print_main( sym );
+		*found = TRUE;
+	}
+
+	return( NULL );
+}
+
 /* Clean up our application and quit. Not interactive! Do any "has been
  * modified, OK to quit?" stuff before this.
  */
@@ -210,30 +259,23 @@ main_quit( void )
 
 	if( main_option_print_main || main_option_output ) {
 		Symbol *sym;
-		PElement *root;
+		gboolean found;
 
 		symbol_recalculate_all();
 
-		sym = compile_lookup( symbol_root->expr->compile, "main" );
-
-		if( !sym )
-			main_error_exit( _( "symbol \"main\" not found" ) );
-		root = &sym->expr->root;
-		if( symbol_recalculate_check( sym ) || 
-			!reduce_pelement( reduce_context, 
-				reduce_spine_strict, root ) ) 
-			main_error_exit( _( "error calculating \"main\"" ) );
-
-		if( main_option_output ) {
-			char filename[FILENAME_MAX];
-
-			im_strncpy( filename, main_option_output, 
-				FILENAME_MAX );
-			(void) save_objects( root, filename );
+		/* Process all the mains we can find: one at the top level,
+		 * one in each workspace.
+		 */
+		found = FALSE;
+		if( (sym = compile_lookup( 
+			symbol_root->expr->compile, "main" )) ) {
+			main_print_main( sym );
+			found = TRUE;
 		}
+		workspace_map( (workspace_map_fn) main_print_ws, &found, NULL );
 
-		if( main_option_print_main )
-			graph_value( root );
+		if( !found )
+			main_error_exit( "%s", _( "no \"main\" found" ) );
 	}
 
 	/* Force all windows down. 
@@ -851,6 +893,56 @@ main_mkdir( const char *dir )
 				get_savedir(), dir, g_strerror( errno ) );
 }
 
+static gboolean
+main_set( const char *str )
+{
+	Symbol *sym;
+
+#ifdef DEBUG
+	printf( "main_set: %s\n", str );
+#endif /*DEBUG*/
+
+	attach_input_string( str );
+	if( !(sym = parse_set_symbol()) ) 
+		return( FALSE );
+
+	/* Put the input just after the '=', ready to parse a RHS into the
+	 * symbol.
+	 */
+	attach_input_string( str + 
+		IM_CLIP( 0, input_state.charpos - 1, strlen( str ) ) );
+
+	if( !symbol_user_init( sym ) || 
+		!parse_rhs( sym->expr, PARSE_RHS ) ) {
+		/* Another parse error.
+		 */
+		expr_error_get( sym->expr );
+
+		/* Block changes to error_string ... symbol_destroy() 
+		 * can set this for compound objects.
+		 */
+		error_block();
+		IDESTROY( sym );
+		error_unblock();
+
+		return( FALSE );
+	}
+
+	symbol_made( sym );
+
+	/* Is there a row? Make sure any modified text there can't zap our new
+	 * text.
+	 */
+	if( sym->expr->row ) {
+		Row *row = sym->expr->row;
+
+		heapmodel_set_modified( 
+			HEAPMODEL( row->child_rhs->itext ), FALSE );
+	}
+
+	return( TRUE );
+}
+
 /* Start here!
  */
 int
@@ -904,7 +996,7 @@ main( int argc, char *argv[] )
 	g_set_application_name( _( PACKAGE ) );
 
 #ifdef DEBUG
-	printf( "main: sizeof( HeapNode ) == %d\n", sizeof( HeapNode ) );
+	printf( "main: sizeof( HeapNode ) == %zd\n", sizeof( HeapNode ) );
 
 	/* Should be 3 pointers, hopefully.
 	 */
@@ -1191,10 +1283,10 @@ main( int argc, char *argv[] )
 		if( !(ws = workspace_new_from_openfile( 
 			main_workspacegroup, main_stdin )) ) 
 			main_log_add( "%s\n", error_get_sub() );
-		else {
-			filemodel_set_filename( FILEMODEL( ws ), "stdin" );
-			FILEMODEL( ws )->nosave = TRUE;
-		}
+		else 
+			/* Don't want to have "stdin" as the filename.
+			 */
+			filemodel_set_filename( FILEMODEL( ws ), NULL );
 	}
 
 	/* Make sure we have a start workspace.
@@ -1225,15 +1317,15 @@ main( int argc, char *argv[] )
 	 */
 	main_load_startup();
 
-	/* Recalc to build all classes.
+	/* Recalc to build all classes. We have to do this in batch
+	 * mode since we can find dirties through dynamic lookups. Even though
+	 * you might think we could just follow recomps.
 	 */
-	if( !main_option_batch ) {
-		main_splash_update( "%s", _( "First recalculation ..." ) );
-		symbol_recalculate_all();
-	}
+	main_splash_update( "%s", _( "First recalculation ..." ) );
+	symbol_recalculate_all();
 
 	/* Measure amount of stuff in temp area ... need this for checking
-	 * temps later. We pop a diglog if there are too many, so only useful
+	 * temps later. We pop a dialog if there are too many, so only useful
 	 * in interactive mode.
 	 */
 	if( !main_option_batch )
@@ -1280,6 +1372,14 @@ main( int argc, char *argv[] )
 		}
 	}
 
+	if( main_option_set ) {
+		int i;
+
+		for( i = 0; main_option_set[i]; i++ ) 
+			if( !main_set( main_option_set[i] ) )
+				main_log_add( "%s\n", error_get_sub() );
+	}
+
 	/* Recalc to load any args.
 	 */
 	main_splash_update( "%s", _( "Final recalculation ..." ) );
