@@ -152,7 +152,6 @@ mainw_destroy( GtkObject *object )
 	FREESID( mainw->watch_changed_sid, main_watchgroup );
 	DESTROY_GTK( mainw->popup );
 	UNREF( mainw->kitgview );
-	IM_FREEF( g_source_remove, mainw->pane_animate_timeout );
 
 	mainw_all = g_slist_remove( mainw_all, mainw );
 
@@ -230,10 +229,11 @@ mainw_init( Mainw *mainw )
 
 	mainw->toolbar_visible = MAINW_TOOLBAR;
 	mainw->statusbar_visible = MAINW_STATUSBAR;
-	mainw->pane_visible = MAINW_TOOLKITBROWSER;
 
 	mainw->row_last_error = NULL;
 
+	mainw->compat_timeout = 0;
+
 	mainw->kitgview = NULL;
 	mainw->toolkitbrowser = NULL;
 	mainw->wsview = NULL;
@@ -244,11 +244,8 @@ mainw_init( Mainw *mainw )
 	mainw->space_free = NULL;
 	mainw->space_free_eb = NULL;
 
-	mainw->compat_timeout = 0;
-
-	mainw->pane_position = MAINW_PANE_POSITION;
-	mainw->pane_animate_timeout = 0;
-	mainw->pane_last_visible = mainw->pane_visible;
+	mainw->lpane = NULL;
+	mainw->rpane = NULL;
 
 	mainw_all = g_slist_prepend( mainw_all, mainw );
 }
@@ -390,65 +387,6 @@ mainw_status_update( Mainw *mainw )
 			ws->status );
 }
 
-static gboolean
-mainw_pane_animate_timeout_cb( Mainw *mainw )
-{
-	int now = gtk_paned_get_position( GTK_PANED( mainw->pane ) );
-	int target = mainw->target_position;
-	int new;
-	gboolean more;
-
-#ifdef DEBUG
-	printf( "mainw_pane_animate_timeout_cb\n" );
-#endif /*DEBUG*/
-
-	new = now + (target - now) / 2;
-
-	if( ABS( new - target ) < 5 || new == mainw->last_set_position ) {
-		/* Close enough: set exactly the target and stop animating. Or
-		 * the new we set last hasn't taken .. in which case we must
-		 * have hit a stop.
-		 */
-		new = target;
-		mainw->pane_animate_timeout = 0;
-		more = FALSE;
-	}
-	else 
-		/* Keep moving.
-		 */
-		more = TRUE;
-
-	gtk_paned_set_position( GTK_PANED( mainw->pane ), new );
-	mainw->last_set_position = new;
-
-	return( more );
-}
-
-/* Animate the pane to a new position.
- */
-static void
-mainw_pane_animate( Mainw *mainw, int target_position )
-{
-	int max_position;
-	int min_position;
-
-	g_object_get( mainw->pane, 
-		"max_position", &max_position, 
-		"min_position", &min_position, 
-		NULL );
-
-	mainw->target_position = 
-		IM_CLIP( min_position, target_position, max_position );
-
-	/* Invalidate the last set pos.
-	 */
-	mainw->last_set_position = -1;
-
-	if( !mainw->pane_animate_timeout ) 
-		mainw->pane_animate_timeout = g_timeout_add( 50, 
-			(GSourceFunc) mainw_pane_animate_timeout_cb, mainw );
-}
-
 static gint
 mainw_jump_name_compare( iContainer *a, iContainer *b )
 {
@@ -563,40 +501,21 @@ mainw_refresh( Mainw *mainw )
 		mainw->statusbar_visible );
         widget_visible( mainw->statusbar_main, mainw->statusbar_visible );
 
+	action = gtk_action_group_get_action( mainw->action_group, 
+		"WorkspaceDefs" );
+	gtk_toggle_action_set_active( GTK_TOGGLE_ACTION( action ),
+		mainw->lpane->open );
+
 	action = gtk_action_group_get_action( mainw->action_group, 
 		"ToolkitBrowser" );
 	gtk_toggle_action_set_active( GTK_TOGGLE_ACTION( action ),
-		mainw->pane_visible );
+		mainw->rpane->open );
 
 	action = gtk_action_group_get_action( mainw->action_group, 
 		view_mode[ws->mode] );
 	gtk_toggle_action_set_active( GTK_TOGGLE_ACTION( action ),
 		TRUE );
 
-	if( mainw->pane_visible != mainw->pane_last_visible ) {
-		mainw->pane_last_visible = mainw->pane_visible;
-
-		if( mainw->pane_visible ) {
-			/* -1 means pick a position from the browser size.
-			 */
-			if( mainw->pane_position == -1 )
-				mainw->pane_position = mainw->wsview->vp.width -
-					toolkitbrowser_get_width( 
-					mainw->toolkitbrowser );
-
-			mainw_pane_animate( mainw, mainw->pane_position );
-		}
-		else {
-			mainw->pane_position = gtk_paned_get_position( 
-				GTK_PANED( mainw->pane ) );
-
-			prefs_set( "MAINW_PANE_POSITION", 
-				"%d", mainw->pane_position );
-
-			mainw_pane_animate( mainw, 10000 );
-		}
-	}
-
 	mainw_jump_update( mainw, mainw->jump_to_column_menu );
 	mainw_jump_update( mainw, mainw->popup_jump );
 }
@@ -1054,6 +973,12 @@ mainw_open_action_cb( GtkAction *action, Mainw *mainw )
 	mainw_open( mainw );
 }
 
+static void
+mainw_open_action_cb2( GtkWidget *wid, GtkWidget *host, Mainw *mainw )
+{
+	mainw_open_action_cb( NULL, mainw );
+}
+
 /* Open one of the example workspaces ... just a shortcut.
  */
 static void
@@ -1251,7 +1176,8 @@ mainw_workspace_merge( Mainw *mainw )
 {
 	GtkWidget *filesel = filesel_new();
 
-	iwindow_set_title( IWINDOW( filesel ), _( "Merge Workspace" ) );
+	iwindow_set_title( IWINDOW( filesel ), 
+		_( "Merge Workspace from File" ) );
 	filesel_set_flags( FILESEL( filesel ), FALSE, FALSE );
 	filesel_set_filetype( FILESEL( filesel ), filesel_type_workspace, 0 ); 
 	iwindow_set_parent( IWINDOW( filesel ), GTK_WIDGET( mainw ) );
@@ -1288,6 +1214,13 @@ mainw_workspace_merge_action_cb( GtkAction *action, Mainw *mainw )
 	mainw_workspace_merge( mainw );
 }
 
+static void
+mainw_workspace_merge_action_cb2( GtkWidget *wid, GtkWidget *host, 
+	Mainw *mainw )
+{
+	mainw_workspace_merge_action_cb( NULL, mainw );
+}
+
 /* Auto recover.
  */
 static void
@@ -1449,10 +1382,15 @@ mainw_statusbar_action_cb( GtkToggleAction *action, Mainw *mainw )
 static void
 mainw_toolkitbrowser_action_cb( GtkToggleAction *action, Mainw *mainw )
 {
-	mainw->pane_visible = gtk_toggle_action_get_active( action );
-	prefs_set( "MAINW_TOOLKITBROWSER", 
-		"%s", bool_to_char( mainw->pane_visible ) );
-	mainw_refresh( mainw );
+	pane_set_open( mainw->rpane, gtk_toggle_action_get_active( action ) );
+}
+
+/* Expose/hide the workspace defs.
+ */
+static void
+mainw_workspacedefs_action_cb( GtkToggleAction *action, Mainw *mainw )
+{
+	pane_set_open( mainw->lpane, gtk_toggle_action_get_active( action ) );
 }
 
 /* Layout columns.
@@ -1703,7 +1641,7 @@ static GtkActionEntry mainw_actions[] = {
 		G_CALLBACK( mainw_preferences_action_cb ) },
 
 	{ "EditToolkits", 
-		NULL, N_( "Edit _Toolkits" ), NULL,
+		NULL, N_( "_Edit" ), NULL,
 		N_( "Edit toolkits" ), 
 		G_CALLBACK( mainw_program_new_action_cb ) },
 
@@ -1743,6 +1681,11 @@ static GtkToggleActionEntry mainw_toggle_actions[] = {
 		NULL, N_( "Toolkit _Browser" ), NULL,
 		N_( "Show toolkit browser" ),
 		G_CALLBACK( mainw_toolkitbrowser_action_cb ), FALSE },
+
+	{ "WorkspaceDefs",
+		NULL, N_( "Workspace _Definitions" ), NULL,
+		N_( "Show workspace definitions" ),
+		G_CALLBACK( mainw_workspacedefs_action_cb ), FALSE },
 };
 
 static GtkRadioActionEntry mainw_radio_actions[] = {
@@ -1809,6 +1752,7 @@ static const char *mainw_menubar_ui_description =
 "      <menuitem action='Toolbar'/>"
 "      <menuitem action='Statusbar'/>"
 "      <menuitem action='ToolkitBrowser'/>"
+"      <menuitem action='WorkspaceDefs'/>"
 "      <separator/>"
 "      <menuitem action='Normal'/>"
 "      <menuitem action='ShowFormula'/>"
@@ -1841,6 +1785,28 @@ static const char *mainw_toolbar_ui_description =
 "  </toolbar>"
 "</ui>";
 
+static void
+mainw_lpane_changed_cb( Pane *pane, Mainw *mainw )
+{
+	mainw->ws->lpane_open = pane->open;
+	mainw->ws->lpane_position = pane->position;
+
+	/* Give menus an update.
+	 */
+	mainw_refresh( mainw );
+}
+
+static void
+mainw_rpane_changed_cb( Pane *pane, Mainw *mainw )
+{
+	mainw->ws->rpane_open = pane->open;
+	mainw->ws->rpane_position = pane->position;
+
+	/* Give menus an update.
+	 */
+	mainw_refresh( mainw );
+}
+
 static void
 mainw_watch_changed_cb( Watchgroup *watchgroup, Watch *watch, Mainw *mainw )
 {
@@ -1961,9 +1927,6 @@ mainw_build( iWindow *iwnd, GtkWidget *vbox )
 	mainw->heap_changed_sid = g_signal_connect( reduce_context->heap, 
 		"changed",
 		G_CALLBACK( mainw_free_changed_cb ), mainw );
-	mainw->watch_changed_sid = g_signal_connect( main_watchgroup, 
-		"watch_changed",
-		G_CALLBACK( mainw_watch_changed_cb ), mainw );
 
 	/* Make message label.
 	 */
@@ -1986,14 +1949,23 @@ mainw_build( iWindow *iwnd, GtkWidget *vbox )
 	gtk_menu_set_accel_group( GTK_MENU( mainw->kitgview->menu ),
 		iwnd->accel_group );
 
-	mainw->pane = gtk_hpaned_new();
+	mainw->rpane = pane_new( PANE_HIDE_RIGHT );
+	g_signal_connect( mainw->rpane, "changed",
+		G_CALLBACK( mainw_rpane_changed_cb ), mainw );
 	gtk_box_pack_start( GTK_BOX( vbox ), 
-		GTK_WIDGET( mainw->pane ), TRUE, TRUE, 0 );
-	gtk_widget_show( mainw->pane );
+		GTK_WIDGET( mainw->rpane ), TRUE, TRUE, 0 );
+	gtk_widget_show( GTK_WIDGET( mainw->rpane ) );
+
+	mainw->lpane = pane_new( PANE_HIDE_LEFT );
+	g_signal_connect( mainw->lpane, "changed",
+		G_CALLBACK( mainw_lpane_changed_cb ), mainw );
+	gtk_paned_pack1( GTK_PANED( mainw->rpane ), GTK_WIDGET( mainw->lpane ),
+		TRUE, FALSE );
+	gtk_widget_show( GTK_WIDGET( mainw->lpane ) );
 
 	mainw->wsview = WORKSPACEVIEW( 
 		model_view_new( MODEL( mainw->ws ), NULL ) );
-	gtk_paned_pack1( GTK_PANED( mainw->pane ), GTK_WIDGET( mainw->wsview ),
+	gtk_paned_pack2( GTK_PANED( mainw->lpane ), GTK_WIDGET( mainw->wsview ),
 		TRUE, FALSE );
 	gtk_widget_show( GTK_WIDGET( mainw->wsview ) );
 
@@ -2005,6 +1977,11 @@ mainw_build( iWindow *iwnd, GtkWidget *vbox )
 	popup_add_but( mainw->popup, _( "Align _Columns" ),
 		POPUP_FUNC( mainw_layout_action_cb2 ) ); 
 	menu_add_sep( mainw->popup );
+	popup_add_but( mainw->popup, GTK_STOCK_OPEN,
+		POPUP_FUNC( mainw_open_action_cb2 ) ); 
+	popup_add_but( mainw->popup, _( "_Merge Workspace from File" ),
+		POPUP_FUNC( mainw_workspace_merge_action_cb2 ) ); 
+	menu_add_sep( mainw->popup );
 	popup_add_but( mainw->popup, _( "_Group Selected" ),
 		POPUP_FUNC( mainw_group_action_cb2 ) ); 
 	menu_add_sep( mainw->popup );
@@ -2016,7 +1993,7 @@ mainw_build( iWindow *iwnd, GtkWidget *vbox )
 	 * clipped to the pane size.
 	 */
 	ebox = gtk_event_box_new();
-	gtk_paned_pack2( GTK_PANED( mainw->pane ), ebox, TRUE, TRUE );
+	gtk_paned_pack2( GTK_PANED( mainw->rpane ), ebox, TRUE, TRUE );
 	gtk_widget_show( ebox );
 
 	mainw->toolkitbrowser = toolkitbrowser_new();
@@ -2027,9 +2004,23 @@ mainw_build( iWindow *iwnd, GtkWidget *vbox )
 		GTK_WIDGET( mainw->toolkitbrowser ) );
 	gtk_widget_show( GTK_WIDGET( mainw->toolkitbrowser ) );
 
+	/* Workspace-local defs pane.
+	 */
+	mainw->workspacedefs = workspacedefs_new( mainw );
+	vobject_link( VOBJECT( mainw->workspacedefs ), IOBJECT( mainw->ws ) );
+	gtk_paned_pack1( GTK_PANED( mainw->lpane ), 
+		GTK_WIDGET( mainw->workspacedefs ), TRUE, TRUE );
+	gtk_widget_show( GTK_WIDGET( mainw->workspacedefs ) );
+
 	/* Set start state.
 	 */
 	(void) mainw_refresh( mainw );
+
+	/* Any changes to prefs, refresh (yuk!).
+	 */
+	mainw->watch_changed_sid = g_signal_connect( main_watchgroup, 
+		"watch_changed",
+		G_CALLBACK( mainw_watch_changed_cb ), mainw );
 }
 
 static void
@@ -2037,16 +2028,6 @@ mainw_popdown( iWindow *iwnd, void *client, iWindowNotifyFn nfn, void *sys )
 {
 	Mainw *mainw = MAINW( iwnd );
 
-	if( mainw->pane_visible )
-		/* Re-read the position rather than looking at pane_position ...
-		 * pane_position is only updated on hide.
-		 */
-		prefs_set( "MAINW_PANE_POSITION", "%d", 
-			gtk_paned_get_position( GTK_PANED( mainw->pane ) ) );
-	else
-		prefs_set( "MAINW_PANE_POSITION", "%d", 
-			mainw->pane_position );
-
 	IM_FREEF( g_source_remove, mainw->compat_timeout );
 
 	filemodel_inter_savenclose_cb( IWINDOW( mainw ), 
@@ -2089,13 +2070,10 @@ mainw_link( Mainw *mainw, Workspace *ws )
 				gdk_screen_get_height( screen ) ) );
 	}
 
-	/* 10000: just a huge number; enough to get the gutter locked on to
-	 * the right edge.
-	 */
-	gtk_paned_set_position( GTK_PANED( mainw->pane ), 
-		MAINW_TOOLKITBROWSER ? 
-			mainw->pane_position : 
-			10000 );
+	pane_set_open_position( mainw->lpane, 
+		ws->lpane_open, ws->lpane_position );
+	pane_set_open_position( mainw->rpane, 
+		ws->rpane_open, ws->rpane_position );
 }
 
 Mainw *
