@@ -51,40 +51,41 @@ matrix_finalize( GObject *gobject )
 
 	/* My instance finalize stuff.
 	 */
-	IM_FREE( matrix->value );
+	IM_FREE( matrix->value.coeff );
 
 	G_OBJECT_CLASS( parent_class )->finalize( gobject );
 }
 
 /* Rearrange our model for a new width/height.
  */
-static gboolean
-matrix_resize( Matrix *matrix, int width, int height )
+gboolean
+matrix_value_resize( MatrixValue *value, int width, int height )
 {
-	double *value;
+	double *coeff;
 	int x, y, i;
 
-	if( width == matrix->width && height == matrix->height )
+	if( width == value->width && height == value->height )
 		return( TRUE );
 
-	if( !(value = IARRAY( NULL, width * height, double )) )
+	if( !(coeff = IARRAY( NULL, width * height, double )) )
 		return( FALSE );
 
 	/* Set what we can with values from the old matrix.
 	 */
 	for( i = 0, y = 0; y < height; y++ )
 		for( x = 0; x < width; x++, i++ ) 
-			if( y < matrix->height && x < matrix->width ) 
-				value[i] = matrix->value[x + y*matrix->width];
+			if( y < value->height && x < value->width ) 
+				coeff[i] = value->coeff[x + 
+					y * value->width];
 			else 
-				value[i] = 0.0;
+				coeff[i] = 0.0;
 
 	/* Install new values.
 	 */
-	IM_FREE( matrix->value );
-	matrix->value = value;
-	matrix->width = width;
-	matrix->height = height;
+	IM_FREE( value->coeff );
+	value->coeff = coeff;
+	value->width = width;
+	value->height = height;
 
 	return( TRUE );
 }
@@ -118,7 +119,7 @@ matrix_done_cb( iWindow *iwnd, void *client,
 		gtk_combo_box_get_active( GTK_COMBO_BOX( eds->display ) );
 	if( !get_geditable_pint( eds->width, &width ) ||
 		!get_geditable_pint( eds->height, &height ) ||
-		!matrix_resize( eds->matrix, width, height ) ) {
+		!matrix_value_resize( &eds->matrix->value, width, height ) ) {
 		nfn( sys, IWINDOW_ERROR );
 		return;
 	}
@@ -153,10 +154,10 @@ matrix_buildedit( iDialog *idlg, GtkWidget *work, MatrixEdit *eds )
 
         eds->width = build_glabeltext4( work, group, "Width" );
 	idialog_init_entry( idlg, eds->width, 
-		"Width of matrix", "%d", matrix->width );
+		"Width of matrix", "%d", matrix->value.width );
         eds->height = build_glabeltext4( work, group, "Height" );
 	idialog_init_entry( idlg, eds->height, 
-		"Height of matrix", "%d", matrix->height );
+		"Height of matrix", "%d", matrix->value.height );
         eds->display = build_goption( work, group, _( "Display as" ),
                 display_names, IM_NUMBER( display_names ), NULL, NULL );
 	gtk_combo_box_set_active( GTK_COMBO_BOX( eds->display ), 
@@ -167,6 +168,12 @@ matrix_buildedit( iDialog *idlg, GtkWidget *work, MatrixEdit *eds )
         gtk_widget_show_all( work );
 }
 
+static View *
+matrix_view_new( Model *model, View *parent )
+{
+	return( matrixview_new() );
+}
+
 /* Pop up a matrix edit box.
  */
 static void 
@@ -190,126 +197,13 @@ matrix_edit( GtkWidget *parent, Model *model )
 	idialog_set_callbacks( IDIALOG( idlg ), 
 		iwindow_true_cb, NULL, idialog_free_client, eds );
 	idialog_add_ok( IDIALOG( idlg ), matrix_done_cb, _( "Set Matrix" ) );
-	idialog_set_parent( IDIALOG( idlg ), parent );
+	iwindow_set_parent( IWINDOW( idlg ), parent );
 	idialog_set_iobject( IDIALOG( idlg ), IOBJECT( model ) );
 	iwindow_build( IWINDOW( idlg ) );
 
 	gtk_widget_show( GTK_WIDGET( idlg ) );
 }
 
-static xmlNode *
-matrix_save( Model *model, xmlNode *xnode )
-{
-	Matrix *matrix = MATRIX( model );
-
-	xmlNode *xthis;
-	const int n = matrix->width * matrix->height;
-
-	if( !(xthis = MODEL_CLASS( parent_class )->save( model, xnode )) )
-		return( NULL );
-
-	if( CLASSMODEL( model )->edited ) {
-		if( !set_dlprop( xthis, "value", matrix->value, n ) ||
-			!set_prop( xthis, "width", "%d", matrix->width ) ||
-			!set_prop( xthis, "height", "%d", matrix->height ) ||
-			!set_prop( xthis, "display", "%d", 
-				(int) matrix->display ) ||
-			!set_prop( xthis, "scale", "%g", matrix->scale ) ||
-			!set_prop( xthis, "offset", "%g", matrix->offset ) ||
-			!set_sprop( xthis, "filename", 
-				CLASSMODEL( matrix )->filename ) )
-			return( NULL );
-	}
-
-	return( xthis );
-}
-
-static gboolean
-matrix_load( Model *model, 
-	ModelLoadState *state, Model *parent, xmlNode *xnode )
-{
-	Matrix *matrix = MATRIX( model );
-	char buf[MAX_STRSIZE];
-
-	assert( IS_RHS( parent ) );
-
-	assert( sizeof( matrix->display ) == sizeof( int ) );
-
-	IM_FREE( matrix->value );
-	if( get_dlprop( xnode, "value", &matrix->value ) &&
-		get_iprop( xnode, "width", &matrix->width ) &&
-		get_iprop( xnode, "height", &matrix->height ) &&
-		get_iprop( xnode, "display", (int *) &matrix->display ) &&
-		get_dprop( xnode, "scale", &matrix->scale ) &&
-		get_dprop( xnode, "offset", &matrix->offset ) &&
-		get_sprop( xnode, "filename", buf, MAX_STRSIZE ) ) {
-		SETSTR( CLASSMODEL( matrix )->filename, buf );
-		classmodel_set_edited( CLASSMODEL( model ), TRUE );
-	}
-
-	return( MODEL_CLASS( parent_class )->load( model, 
-		state, parent, xnode ) );
-}
-
-/* Update Matrix from heap.
- */
-static gboolean
-matrix_class_get( Classmodel *classmodel, PElement *root )
-{
-	Matrix *matrix = MATRIX( classmodel );
-	int w, h;
-	int display;
-	char buf[MAX_STRSIZE];
-
-	if( !class_get_member_matrix_size( root, MEMBER_VALUE, &w, &h ) ||
-		!matrix_resize( matrix, w, h ) ||
-		!class_get_member_matrix( root, MEMBER_VALUE, 
-			matrix->value, matrix->width * matrix->height, 
-			&w, &h ) )
-		return( FALSE );
-
-	if( class_get_member_int( root, MEMBER_DISPLAY, &display ) ) {
-		matrix->display = (MatrixDisplayType) 
-			IM_CLIP( 0, display, MATRIX_DISPLAY_LAST - 1 );
-	}
-	(void) class_get_member_real( root, MEMBER_SCALE, &matrix->scale );
-	(void) class_get_member_real( root, MEMBER_OFFSET, &matrix->offset );
-	if( class_get_member_string( root, MEMBER_FILENAME, buf, MAX_STRSIZE ) )
-		SETSTR( CLASSMODEL( matrix )->filename, buf );
-
-	return( TRUE );
-}
-
-/* Make a new "fn value" application.
- */
-static gboolean
-matrix_class_new( Classmodel *classmodel, PElement *fn, PElement *out )
-{
-	Heap *heap = reduce_context->heap;
-	Matrix *matrix = MATRIX( classmodel );
-
-	PElement rhs;
-
-	/* Make application nodes.
-	 */
-	heap_appl_init( out, fn );
-	if( !heap_appl_add( heap, out, &rhs ) || 
-		!heap_matrix_new( heap, 
-			matrix->width, matrix->height, matrix->value, &rhs ) ||
-		!heap_appl_add( heap, out, &rhs ) ||
-		!heap_real_new( heap, matrix->scale, &rhs ) ||
-		!heap_appl_add( heap, out, &rhs ) ||
-		!heap_real_new( heap, matrix->offset, &rhs ) ||
-		!heap_appl_add( heap, out, &rhs ) ||
-		!heap_string_new( heap, 
-			CLASSMODEL( matrix )->filename, &rhs ) ||
-		!heap_appl_add( heap, out, &rhs ) ||
-		!heap_real_new( heap, matrix->display, &rhs ) )
-		return( FALSE );
-
-	return( TRUE );
-}
-
 static gboolean
 matrix_graphic_save( Classmodel *classmodel, 
 	GtkWidget *parent, const char *filename )
@@ -362,6 +256,26 @@ matrix_graphic_replace( Classmodel *classmodel,
 	return( TRUE );
 }
 
+/* Members of matrix we automate.
+ */
+static ClassmodelMember matrix_members[] = {
+	{ CLASSMODEL_MEMBER_MATRIX, NULL, 0, 
+		MEMBER_VALUE, NULL, N_( "Value" ),
+		G_STRUCT_OFFSET( Matrix, value ) },
+	{ CLASSMODEL_MEMBER_DOUBLE, NULL, 0,
+		MEMBER_SCALE, "scale", N_( "Scale" ),
+		G_STRUCT_OFFSET( Matrix, scale ) },
+	{ CLASSMODEL_MEMBER_DOUBLE, NULL, 0,
+		MEMBER_OFFSET, "offset", N_( "Offset" ),
+		G_STRUCT_OFFSET( Matrix, offset ) },
+	{ CLASSMODEL_MEMBER_STRING, NULL, 0,
+		MEMBER_FILENAME, "filename", N_( "Filename" ),
+		G_STRUCT_OFFSET( Classmodel, filename ) },
+	{ CLASSMODEL_MEMBER_ENUM, NULL, MATRIX_DISPLAY_LAST - 1,
+		MEMBER_DISPLAY, "display", N_( "Display" ),
+		G_STRUCT_OFFSET( Matrix, display ) }
+};
+
 static void
 matrix_class_init( MatrixClass *class )
 {
@@ -378,13 +292,8 @@ matrix_class_init( MatrixClass *class )
 	 */
 	gobject_class->finalize = matrix_finalize;
 
-	model_class->view_new = matrixview_new;
+	model_class->view_new = matrix_view_new;
 	model_class->edit = matrix_edit;
-	model_class->save = matrix_save;
-	model_class->load = matrix_load;
-
-	classmodel_class->class_get = matrix_class_get;
-	classmodel_class->class_new = matrix_class_new;
 
 	classmodel_class->graphic_save = matrix_graphic_save;
 	classmodel_class->graphic_replace = matrix_graphic_replace;
@@ -395,6 +304,9 @@ matrix_class_init( MatrixClass *class )
 	/* Static init.
 	 */
 	model_register_loadable( MODEL_CLASS( class ) );
+
+	classmodel_class->members = matrix_members;
+	classmodel_class->n_members = IM_NUMBER( matrix_members );
 }
 
 static void
@@ -404,12 +316,13 @@ matrix_init( Matrix *matrix )
 	printf( "matrix_init\n" );
 #endif /*DEBUG*/
 
-	matrix->value = NULL;
-        matrix->width = 0;
-	matrix->height = 0;
+	matrix->value.coeff = NULL;
+        matrix->value.width = 0;
+	matrix->value.height = 0;
 	matrix->display = MATRIX_DISPLAY_TEXT;
 	matrix->scale = 1.0;
 	matrix->offset = 0.0;
+	matrix->selected = FALSE;
 
 	iobject_set( IOBJECT( matrix ), CLASS_MATRIX, NULL );
 }
@@ -439,15 +352,58 @@ matrix_get_type( void )
 	return( type );
 }
 
-Classmodel *
-matrix_new( Rhs *rhs )
+void
+matrix_select( Matrix *matrix, int left, int top, int width, int height )
 {
-	Matrix *matrix;
+	if( !matrix->selected ||
+		matrix->range.left != left ||
+		matrix->range.top != top ||
+		matrix->range.width != width ||
+		matrix->range.height != height ) {
+		Row *row = HEAPMODEL( matrix )->row;
 
-	matrix = MATRIX( g_object_new( TYPE_MATRIX, NULL ) );
-	icontainer_child_add( ICONTAINER( rhs ), ICONTAINER( matrix ), -1 );
+#ifdef DEBUG
+		printf( "matrix_select: "
+			"left=%d, top = %d, width = %d, height = %d\n",
+			left, top, width, height );
+#endif /*DEBUG*/
 
-	return( CLASSMODEL( matrix ) );
+		matrix->selected = TRUE;
+		matrix->range.left = left;
+		matrix->range.top = top;
+		matrix->range.width = width;
+		matrix->range.height = height;
+		iobject_changed( IOBJECT( matrix ) );
+
+		/* Also make sure this row is selected.
+		 */
+		row_select_ensure( row );
+
+		/* The range of cells selected has changed, so the workspace
+		 * must update the status line too. row_select_ensure() only
+		 * spots row on/off selects. Yuk!
+		 */
+		iobject_changed( IOBJECT( row->ws ) );
+	}
+}
+
+void 
+matrix_deselect( Matrix *matrix )
+{
+	if( matrix->selected ) {
+		Row *row = HEAPMODEL( matrix )->row;
+
+#ifdef DEBUG
+		printf( "matrix_deselect\n" );
+#endif /*DEBUG*/
+
+		matrix->selected = FALSE;
+		iobject_changed( IOBJECT( matrix ) );
+
+		/* Also make sure this row is not selected.
+		 */
+		row_deselect( row );
+	}
 }
 
 /* Guess a display type from a filename.
@@ -455,24 +411,24 @@ matrix_new( Rhs *rhs )
 static int
 matrix_guess_display( const char *fname )
 {
-	const char *p;
-	int i;
-
 	/* Choose display type based on filename suffix ... rec 
-	 * displays as 1, mor displays as 2, all others display as 0. Keep in
-	 * sync with MatrixDisplayType.
+	 * displays as 1, mor displays as 2, .con displays as 3, all others 
+	 * display as 0. Keep in sync with MatrixDisplayType.
 	 */
-	static const char *suffs[] = {
-		".con",
-		".rec",
-		".mor"
+	static const FileselFileType *types[] = {
+		&filesel_xfile_type,	// matrix
+		&filesel_rfile_type,	// recombination
+		&filesel_mfile_type,	// morphology
+		&filesel_cfile_type	// convolution
 	};
 
-	if( !fname || !(p = strrchr( fname, '.' )) ) 
+	int i;
+
+	if( !fname )
 		return( 0 );
 
-	for( i = 0; i < IM_NUMBER( suffs ); i++ )
-		if( strcasecmp( p, suffs[i] ) == 0 )
+	for( i = 0; i < IM_NUMBER( types ); i++ )
+		if( is_file_type( types[i], fname ) )
 			return( i );
 
 	return( 0 );
@@ -674,15 +630,15 @@ matrix_model_to_dmask( Matrix *matrix )
 	int i;
 
 	if( !(dmask = im_create_dmask( CLASSMODEL( matrix )->filename, 
-		matrix->width, matrix->height )) ) {
+		matrix->value.width, matrix->value.height )) ) {
 		error_vips_all();
 		return( NULL );
 	}
 
 	dmask->scale = matrix->scale;
 	dmask->offset = matrix->offset;
-	for( i = 0; i < matrix->width * matrix->height; i++ )
-		dmask->coeff[i] = matrix->value[i];
+	for( i = 0; i < matrix->value.width * matrix->value.height; i++ )
+		dmask->coeff[i] = matrix->value.coeff[i];
 
 	return( dmask );
 }
@@ -692,16 +648,17 @@ matrix_dmask_to_model( Matrix *matrix, DOUBLEMASK *dmask )
 {
 	int i;
 
-	if( !matrix_resize( matrix, dmask->xsize, dmask->ysize ) ) 
+	if( !matrix_value_resize( &matrix->value, 
+		dmask->xsize, dmask->ysize ) ) 
 		return( FALSE );
 
 	matrix->scale = dmask->scale;
 	matrix->offset = dmask->offset;
-	for( i = 0; i < matrix->width * matrix->height; i++ )
-		matrix->value[i] = dmask->coeff[i];
+	for( i = 0; i < matrix->value.width * matrix->value.height; i++ )
+		matrix->value.coeff[i] = dmask->coeff[i];
 	matrix->display = 
 		(MatrixDisplayType) matrix_guess_display( dmask->filename );
-	SETSTR( CLASSMODEL( matrix )->filename, dmask->filename );
+	IM_SETSTR( CLASSMODEL( matrix )->filename, dmask->filename );
 
 	return( TRUE );
 }
