@@ -219,10 +219,6 @@ model_loadstate_rewrite_name( char *name )
 	return( NULL );
 }
 
-/* Need to set this! From parse.y.
- */
-extern jmp_buf parse_error_point;
-
 /* Use the lexer to rewrite an expression, swapping all symbols on the rewrite 
  * list.
  */
@@ -245,20 +241,7 @@ model_loadstate_rewrite( ModelLoadState *state, char *old_rhs, char *new_rhs )
 	/* Lex and rewrite.
 	 */
 	while( (yychar = yylex()) > 0 ) 
-		/* Fee any stuff the lexer might have allocated. 
-		 */
-		switch( yychar ) {
-		case TK_CONST:
-			tree_const_destroy( &yylval.yy_const );
-			break;
-
-		case TK_IDENT:
-			IM_FREE( yylval.yy_name );
-			break;
-
-		default:
-			break;
-		}
+		free_lex( yychar );
 
 	model_loadstate = NULL;
 
