@@ -27,6 +27,15 @@
 
  */
 
+/* When scrolling, do we want the top or the bottom of the object visible.
+ * Important for Columns, since we sometimes want to see the title bar and
+ * sometimes the edit box at the bottom.
+ */
+typedef enum {
+	MODEL_SCROLL_TOP,
+	MODEL_SCROLL_BOTTOM
+} ModelScrollPosition;
+
 /* How to rename symbols.
  */
 typedef struct _ModelRename {
@@ -90,13 +99,13 @@ typedef struct _ModelClass {
 
 		view_new	make a view for this model ... make the top
 				view yourself, thereafter view will watch
-				model_child_add etc. and manage subviews
+				child_add etc. and manage subviews
 				automatically ... use model->display to create
 				and destroy views
 
 	 */
 
-	View *(*view_new)( void );
+	View *(*view_new)( Model *model, View *parent );
 
 	/* Change methods
 
@@ -113,7 +122,7 @@ typedef struct _ModelClass {
 	 */
 
 	void (*edit)( GtkWidget *, Model * );
-	void (*scrollto)( Model * );
+	void (*scrollto)( Model *, ModelScrollPosition );
 	void (*reset)( Model * );
 	void (*layout)( Model * );
 
@@ -155,8 +164,8 @@ void model_loadstate_rewrite( ModelLoadState *state,
 
 void model_register_loadable( ModelClass *model_class );
 
-View *model_view_new( Model *model );
-void model_scrollto( Model *model );
+View *model_view_new( Model *model, View *parent );
+void model_scrollto( Model *model, ModelScrollPosition position );
 void model_layout( Model *model );
 void *model_reset( Model *model );
 void *model_edit( GtkWidget *parent, Model *model );
