@@ -27,6 +27,10 @@
 
  */
 
+/*
+#define DEBUG
+ */
+
 #include "ip.h"
 
 static ClassmodelClass *parent_class = NULL;
@@ -48,6 +52,12 @@ option_finalize( GObject *gobject )
 	G_OBJECT_CLASS( parent_class )->finalize( gobject );
 }
 
+static View *
+option_view_new( Model *model, View *parent )
+{
+	return( optionview_new() );
+}
+
 /* Widgets for option edit.
  */
 typedef struct _OptionEdit {
@@ -195,103 +205,26 @@ option_edit( GtkWidget *parent, Model *model )
 	idialog_set_callbacks( IDIALOG( idlg ), 
 		iwindow_true_cb, NULL, idialog_free_client, eds );
 	idialog_add_ok( IDIALOG( idlg ), option_done_cb, _( "Set Option" ) );
-	idialog_set_parent( IDIALOG( idlg ), parent );
+	iwindow_set_parent( IWINDOW( idlg ), parent );
 	idialog_set_iobject( IDIALOG( idlg ), IOBJECT( model ) );
 	iwindow_build( IWINDOW( idlg ) );
 
 	gtk_widget_show( GTK_WIDGET( idlg ) );
 }
 
-static xmlNode *
-option_save( Model *model, xmlNode *xnode )
-{
-	Option *option = OPTION( model );
-
-	xmlNode *xthis;
-
-	if( !(xthis = MODEL_CLASS( parent_class )->save( model, xnode )) )
-		return( NULL );
-
-	if( CLASSMODEL( model )->edited ) {
-		if( !set_slprop( xthis, "labels", option->labels ) ||
-			!set_prop( xthis, "value", "%d", option->value ) )
-			return( NULL );
-	}
-
-	return( xthis );
-}
-
-static gboolean
-option_load( Model *model, 
-	ModelLoadState *state, Model *parent, xmlNode *xnode )
-{
-	Option *option = OPTION( model );
-
-	assert( IS_RHS( parent ) );
-
-	IM_FREEF( slist_free_all, option->labels );
-
-	if( get_slprop( xnode, "labels", &option->labels ) &&
-		get_iprop( xnode, "value", &option->value ) )
-		classmodel_set_edited( CLASSMODEL( model ), TRUE );
-
-	return( MODEL_CLASS( parent_class )->load( model, 
-		state, parent, xnode ) );
-}
-
-/* Update Option from heap.
+/* Members of option we automate.
  */
-static gboolean
-option_class_get( Classmodel *classmodel, PElement *root )
-{
-	Option *option = OPTION( classmodel );
-	char buf[MAX_STRSIZE];
-	GSList *labels;
-	int value;
-
-	/* Reread from heap.
-	 */
-	labels = NULL;
-	if( !class_get_member_string( root, MEMBER_CAPTION, 
-		buf, MAX_STRSIZE ) ||
-		!class_get_member_int( root, MEMBER_VALUE, &value ) ||
-		!class_get_member_lstring( root, MEMBER_LABELS, &labels ) )
-		return( FALSE );
-
-	/* Install.
-	 */
-	IM_FREEF( slist_free_all, option->labels );
-	SETSTR( IOBJECT( option )->caption, buf );
-	option->labels = labels;
-	option->value = value;
-
-	return( TRUE );
-}
-
-/* Make a new "fn caption labels value" application.
- */
-static gboolean
-option_class_new( Classmodel *classmodel, PElement *fn, PElement *out )
-{
-	Heap *heap = reduce_context->heap;
-	Option *option = OPTION( classmodel );
-
-	PElement rhs;
-
-	/* Make application nodes.
-	 */
-	heap_appl_init( out, fn );
-	if( !heap_appl_add( heap, out, &rhs ) ||
-		!heap_string_new( heap, 
-			IOBJECT( classmodel )->caption, &rhs ) ||
-		!heap_appl_add( heap, out, &rhs ) ||
-		!heap_lstring_new( heap, option->labels, &rhs ) ||
-		!heap_appl_add( heap, out, &rhs ) ||
-		!heap_real_new( heap, (double) option->value, &rhs ) )
-		return( FALSE );
-
-	return( TRUE );
-}
+static ClassmodelMember option_members[] = {
+	{ CLASSMODEL_MEMBER_STRING, NULL, 0,
+		MEMBER_CAPTION, "caption", N_( "Caption" ),
+		G_STRUCT_OFFSET( iObject, caption ) },
+	{ CLASSMODEL_MEMBER_STRING_LIST, NULL, 0,
+		MEMBER_LABELS, "labels", N_( "Labels" ),
+		G_STRUCT_OFFSET( Option, labels ) },
+	{ CLASSMODEL_MEMBER_INT, NULL, 0,
+		MEMBER_VALUE, "value", N_( "Value" ),
+		G_STRUCT_OFFSET( Option, value ) }
+};
 
 static void
 option_class_init( OptionClass *class )
@@ -309,24 +242,22 @@ option_class_init( OptionClass *class )
 	 */
 	gobject_class->finalize = option_finalize;
 
-	model_class->view_new = optionview_new;
+	model_class->view_new = option_view_new;
 	model_class->edit = option_edit;
-	model_class->save = option_save;
-	model_class->load = option_load;
-
-	classmodel_class->class_get = option_class_get;
-	classmodel_class->class_new = option_class_new;
 
 	/* Static init.
 	 */
 	model_register_loadable( MODEL_CLASS( class ) );
+
+	classmodel_class->members = option_members;
+	classmodel_class->n_members = IM_NUMBER( option_members );
 }
 
 static void
 option_init( Option *option )
 {
         option->labels = NULL;
-	option->value = 1;
+	option->value = 0;
 
 	iobject_set( IOBJECT( option ), CLASS_OPTION, NULL );
 }
@@ -355,14 +286,3 @@ option_get_type( void )
 
 	return( type );
 }
-
-Classmodel *
-option_new( Rhs *rhs )
-{
-	Option *option;
-
-	option = OPTION( g_object_new( TYPE_OPTION, NULL ) );
-	icontainer_child_add( ICONTAINER( rhs ), ICONTAINER( option ), -1 );
-
-	return( CLASSMODEL( option ) );
-}
