@@ -35,6 +35,40 @@
 
 static GraphicviewClass *parent_class = NULL;
 
+/* Copy a gslist of strings.
+ */
+static GSList *
+lstring_copy( GSList *lstring )
+{
+	GSList *new;
+	GSList *p;
+
+	new = NULL;
+	for( p = lstring; p; p = p->next )
+		new = g_slist_prepend( new, 
+			g_strdup( (const char *) p->data ) );
+
+	new = g_slist_reverse( new );
+
+	return( new );
+}
+
+/* Are two lstrings equal?
+ */
+static gboolean
+lstring_equal( GSList *a, GSList *b )
+{
+	for( ; a && b; a = a->next, b = b->next )
+		if( strcmp( (const char *) a->data, 
+			(const char *) b->data ) != 0 )
+			return( FALSE );
+
+	if( a || b )
+		return( FALSE );
+
+	return( TRUE );
+}
+
 static void
 optionview_destroy( GtkObject *object )
 {
@@ -47,6 +81,7 @@ optionview_destroy( GtkObject *object )
 
 	/* My instance destroy stuff.
 	 */
+	IM_FREEF( slist_free_all, optionview->labels );
 
 	GTK_OBJECT_CLASS( parent_class )->destroy( object );
 }
@@ -92,27 +127,47 @@ optionview_refresh( vObject *vobject )
 	int i;
 
 #ifdef DEBUG
-	printf( "optionview_refresh\n" );
+	printf( "optionview_refresh: " );
+	row_name_print( HEAPMODEL( option )->row );
+	printf( "\n" );
 #endif /*DEBUG*/
 
-	/* If the menu is currently up, we can get strange things happening if
-	 * we destroy it.
+	/* Only rebuild the menu if there's been a change.
 	 */
-	if( optionview->options )
-		gtk_combo_box_popdown( GTK_COMBO_BOX( optionview->options ) );
-	IM_FREEF( gtk_widget_destroy, optionview->options );
+	if( !lstring_equal( optionview->labels, option->labels ) ) {
+		/* If the menu is currently up, we can get strange things 
+		 * happening if we destroy it.
+		 */
+		if( optionview->options )
+			gtk_combo_box_popdown( 
+				GTK_COMBO_BOX( optionview->options ) );
+		IM_FREEF( gtk_widget_destroy, optionview->options );
 
-	optionview->options = gtk_combo_box_new_text();
-	for( p = option->labels, i = 0; p; p = p->next, i++ ) 
-		gtk_combo_box_append_text( GTK_COMBO_BOX( optionview->options ),
-			(const char *) p->data );
-        gtk_box_pack_start( GTK_BOX( optionview->hbox ), 
-		optionview->options, TRUE, TRUE, 0 );
-        gtk_widget_show( optionview->options );
-	gtk_signal_connect( GTK_OBJECT( optionview->options ), "changed", 
-		GTK_SIGNAL_FUNC( optionview_change_cb ), optionview );
-	gtk_combo_box_set_active( GTK_COMBO_BOX( optionview->options ), 
-		option->value );
+		optionview->options = gtk_combo_box_new_text();
+		for( p = option->labels, i = 0; p; p = p->next, i++ ) 
+			gtk_combo_box_append_text( 
+				GTK_COMBO_BOX( optionview->options ),
+				(const char *) p->data );
+		gtk_box_pack_start( GTK_BOX( optionview->hbox ), 
+			optionview->options, TRUE, TRUE, 0 );
+
+		gtk_signal_connect( GTK_OBJECT( optionview->options ), 
+			"changed", 
+			GTK_SIGNAL_FUNC( optionview_change_cb ), optionview );
+		gtk_widget_show( optionview->options );
+
+		IM_FREEF( slist_free_all, optionview->labels );
+		optionview->labels = lstring_copy( option->labels );
+	}
+
+	if( optionview->options ) {
+		gtk_signal_handler_block_by_data( 
+			GTK_OBJECT( optionview->options ), optionview );
+		gtk_combo_box_set_active( GTK_COMBO_BOX( optionview->options ), 
+			option->value );
+		gtk_signal_handler_unblock_by_data( 
+			GTK_OBJECT( optionview->options ), optionview );
+	}
 
 	set_glabel( optionview->label, _( "%s:" ), IOBJECT( option )->caption );
 
@@ -154,6 +209,8 @@ optionview_init( Optionview *optionview )
 
 	optionview->options = NULL;
 
+	optionview->labels = NULL;
+
         gtk_widget_show_all( optionview->hbox );
 }
 
