@@ -45,6 +45,10 @@
 #include <vips/vips.h>
 #include <vips/rect.h>
 
+#include <libxml/tree.h>
+#include <libxml/parser.h>
+
+#include "util.h"
 #include "gtkutil.h"
 #include "orderitem.h"
 #include "orderlist.h"
@@ -217,7 +221,7 @@ orderlist_scan( Orderlist *ol )
 		GtkWidget *par = GTK_WIDGET( ol )->parent->parent;
 
 		orderlist_add_label( ORDERLIST( ol ), cname, len - 1 );
-		set_gentry( ol->cname, "" );
+		set_gentry( ol->cname, "%s", "" );
 
 		/* Scroll to bottom.
 		 */
@@ -278,10 +282,10 @@ orderlist_init( Orderlist *ol )
 GtkType
 orderlist_get_type( void )
 {
-	static GtkType orderlist_type = 0;
+	static GtkType type = 0;
 
-	if (!orderlist_type) {
-		static const GtkTypeInfo ol_info = {
+	if( !type ) {
+		static const GtkTypeInfo info = {
 			"Orderlist",
 			sizeof( Orderlist ),
 			sizeof( OrderlistClass ),
@@ -292,10 +296,10 @@ orderlist_get_type( void )
 			(GtkClassInitFunc) NULL,
 		};
 
-		orderlist_type = gtk_type_unique (GTK_TYPE_LIST, &ol_info);
+		type = gtk_type_unique( GTK_TYPE_LIST, &info );
 	}
 
-	return( orderlist_type );
+	return( type );
 }
 
 GtkWidget *
