@@ -50,7 +50,7 @@ paintboxview_destroy( GtkObject *object )
 	pbv = PAINTBOXVIEW( object );
 
 #ifdef DEBUG
-	printf( "paintboxview_destroy: 0x%x\n", (unsigned int) pbv );
+	printf( "paintboxview_destroy: %p\n", pbv );
 #endif /*DEBUG*/
 
 	/* My instance destroy stuff.
@@ -160,7 +160,7 @@ paintboxview_font_changed_cb( GtkWidget *widget, Paintboxview *pbv )
 
 	font_name = fontbutton_get_font_name( fontbutton ); 
 	if( strcmp( font_name, imagemodel->font_name ) != 0 ) {
-		SETSTR( imagemodel->font_name, font_name );
+		IM_SETSTR( imagemodel->font_name, font_name );
 		iobject_changed( IOBJECT( imagemodel ) );
 	}
 }
@@ -211,7 +211,7 @@ paintboxview_text_changed_cb( GtkWidget *widget, Paintboxview *pbv )
 {
 	const char *text = gtk_entry_get_text( GTK_ENTRY( pbv->text ) );
 
-	SETSTR( pbv->imagemodel->text, text );
+	IM_SETSTR( pbv->imagemodel->text, text );
 }
 
 static void
@@ -433,7 +433,7 @@ paintboxview_changed_cb( Imagemodel *imagemodel, Paintboxview *pbv )
 	int i;
 
 #ifdef DEBUG
-	printf( "paintboxview_conv_changed_cb: 0x%x\n", (unsigned int) conv );
+	printf( "paintboxview_conv_changed_cb: %p\n", conv );
 #endif /*DEBUG*/
 
 	/* Has the ii changed? Link to it for undo/redo changes.
@@ -485,7 +485,7 @@ static void
 paintboxview_link( Paintboxview *pbv, Imagemodel *imagemodel )
 {
 #ifdef DEBUG
-	printf( "paintboxview_link: 0x%x\n", (unsigned int) pbv );
+	printf( "paintboxview_link: %p\n", pbv );
 #endif /*DEBUG*/
 
 	pbv->imagemodel = imagemodel;
