@@ -1,4 +1,4 @@
-/* main processing window
+/* preferences dialog
  */
 
 /*
@@ -38,112 +38,32 @@ static iDialogClass *parent_class = NULL;
 static void
 prefs_destroy( GtkObject *object )
 {
-	Prefs *prefs;
-
-	g_return_if_fail( object != NULL );
-	g_return_if_fail( IS_PREFS( object ) );
-
-	prefs = PREFS( object );
+	Prefs *prefs = PREFS( object );
 
 #ifdef DEBUG
 	printf( "prefs_destroy\n" );
 #endif /*DEBUG*/
 
-	/* My instance destroy stuff.
-	 */
-	FREESID( prefs->destroy_sid, prefs->ws );
-	IM_FREEF( g_source_remove, prefs->auto_layout_timeout );
-
-	GTK_OBJECT_CLASS( parent_class )->destroy( object );
-}
-
-static gboolean
-prefs_auto_layout_timeout_cb( Prefs *prefs )
-{
-	prefs->auto_layout_timeout = 0;
-
-	model_layout( MODEL( prefs->ws ) );
-
-	return( FALSE );
-}
-
-static void
-prefs_show( GtkWidget *widget )
-{
-	Prefs *prefs = PREFS( widget );
-
-	GTK_WIDGET_CLASS( parent_class )->show( widget );
-
-	if( !prefs->done_auto_layout ) {
-		prefs->done_auto_layout = TRUE;
-
-		/* Yuk. We autolayout some time after the window is shown,
-		 * very unreliable. Unfortunately we can't layout until all of
-		 * the widget have their final size, and that's done in an
-		 * idle loop with no clear "size allocation now finished" 
-		 * event to link to
-
-			FIXME ... there must be a better way to do this. Maybe
-			schedule a layout 200ms after every size_allocate on a
-			columnview?
+	if( prefs->ws ) {
+		Filemodel *filemodel = FILEMODEL( prefs->ws );
 
+		/* Force a recalc, in case we've changed the autorecalc 
+		 * settings. Also does a scan on any widgets.
 		 */
-		prefs->auto_layout_timeout = g_timeout_add( 2000, 
-			(GSourceFunc) prefs_auto_layout_timeout_cb, prefs );
-	}
-}
+		symbol_recalculate_all_force( TRUE );
 
-static void
-prefs_popdown( iWindow *iwnd, void *client, iWindowNotifyFn nfn, void *sys ) 
-{ 
-	Prefs *prefs = PREFS( iwnd );
-	Filemodel *filemodel = FILEMODEL( prefs->ws );
-
-	/* Force a recalc, in case we've changed the autorecalc settings. Also
-	 * does a scan on any widgets.
-	 */
-	symbol_recalculate_all_force();
-
-	if( filemodel ) {
 		if( filemodel->modified &&
-			!filemodel_save_all( filemodel, filemodel->filename ) ) 
-			nfn( sys, IWINDOW_ERROR );
-		else 
+			filemodel_save_all( filemodel, filemodel->filename ) ) 
 			filemodel_set_modified( filemodel, FALSE );
 	}
 
-	nfn( sys, IWINDOW_TRUE );
-}
+	/* My instance destroy stuff.
+	 */
+	FREESID( prefs->destroy_sid, prefs->ws );
+	IM_FREE( prefs->caption_filter );
+	prefs->ws = NULL;
 
-void 
-prefs_revert_ok_cb( iWindow *iwnd, void *client, 
-	iWindowNotifyFn nfn, void *sys ) 
-{
-	Prefs *prefs = PREFS( client );
-	Filemodel *filemodel = FILEMODEL( prefs->ws );
-	
-	if( filemodel->filename ) {
-		(void) unlinkf( "%s", filemodel->filename );
-		main_reload();
-		symbol_recalculate_all();
-	}
-
-	nfn( sys, IWINDOW_TRUE );
-}
-
-void 
-prefs_revert_cb( iWindow *iwnd, void *client, iWindowNotifyFn nfn, void *sys ) 
-{
-	Prefs *prefs = PREFS( iwnd );
-
-	box_yesno( GTK_WIDGET( iwnd ),
-		prefs_revert_ok_cb, iwindow_true_cb, prefs,
-		nfn, sys,
-		_( "Revert to Defaults" ), 
-		_( "Revert to installation defaults?" ),
-		_( "Would you like to reset all preferences to their factory "
-		"settings? This will delete any changes you have ever made "
-		"to your preferences and may take a few seconds." ) );
+	GTK_OBJECT_CLASS( parent_class )->destroy( object );
 }
 
 static void
@@ -153,43 +73,59 @@ prefs_build( GtkWidget *widget )
 	GtkWidget *work;
 
 #ifdef DEBUG
-	printf( "prefs_build: 0x%x\n", (unsigned int) prefs );
+	printf( "prefs_build: %p\n", prefs );
 #endif /*DEBUG*/
 
-	iwindow_set_title( IWINDOW( prefs ), _( "Preferences" ) );
-	idialog_set_callbacks( IDIALOG( prefs ), 
-		NULL, prefs_popdown, NULL, prefs );
-	idialog_add_ok( IDIALOG( prefs ), 
-		prefs_revert_cb, _( "Revert to Defaults" ) );
-	idialog_add_ok( IDIALOG( prefs ), iwindow_true_cb, GTK_STOCK_CLOSE );
-
 	/* Call all builds in superclasses.
 	 */
 	IWINDOW_CLASS( parent_class )->build( widget );
 
 	work = IDIALOG( prefs )->work;
 
-	prefs->wsview = WORKSPACEVIEW( model_view_new( MODEL( prefs->ws ) ) );
-	view_link( VIEW( prefs->wsview ), MODEL( prefs->ws ), NULL );
-	gtk_box_pack_start( GTK_BOX( work ), 
-		GTK_WIDGET( prefs->wsview ), TRUE, TRUE, 0 );
-	gtk_window_set_default_size( GTK_WINDOW( prefs ), 665, 480 );
-	gtk_widget_show( GTK_WIDGET( prefs->wsview ) );
+	prefs->pwview = PREFWORKSPACEVIEW( prefworkspaceview_new() );
+	prefworkspaceview_set_caption_filter( prefs->pwview, 
+		prefs->caption_filter );
+	view_link( VIEW( prefs->pwview ), MODEL( prefs->ws ), NULL );
+
+	if( prefs->caption_filter ) {
+		gtk_box_pack_start( GTK_BOX( work ), 
+			GTK_WIDGET( prefs->pwview ), TRUE, TRUE, 0 );
+
+		gtk_widget_show( GTK_WIDGET( prefs->pwview ) );
+	}
+	else {
+		/* No caption_filter set, so this is probably a big prefs
+		 * window. Build a scrolledwindow for the content.
+		 */
+		GtkWidget *window;
+
+		window = gtk_scrolled_window_new( NULL, NULL );
+		gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( window ), 
+			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
+		gtk_scrolled_window_add_with_viewport( 
+			GTK_SCROLLED_WINDOW( window ), 
+			GTK_WIDGET( prefs->pwview ) );
+		gtk_viewport_set_shadow_type( 
+			GTK_VIEWPORT( GTK_BIN( window )->child ), 
+			GTK_SHADOW_NONE );
+		gtk_box_pack_start( GTK_BOX( work ), 
+			GTK_WIDGET( window ), TRUE, TRUE, 0 );
+
+		gtk_widget_show( GTK_WIDGET( prefs->pwview ) );
+		gtk_widget_show( window );
+	}
 }
 
 static void
 prefs_class_init( PrefsClass *class )
 {
 	GtkObjectClass *gobject_class = (GtkObjectClass *) class;
-	GtkWidgetClass *widget_class = (GtkWidgetClass *) class;
 	iWindowClass *iwindow_class = (iWindowClass *) class;
 
 	parent_class = g_type_class_peek_parent( class );
 
 	gobject_class->destroy = prefs_destroy;
 
-	widget_class->show = prefs_show;
-
 	iwindow_class->build = prefs_build;
 
 	/* Create signals.
@@ -204,8 +140,6 @@ prefs_init( Prefs *prefs )
 {
 	prefs->ws = NULL;
 	prefs->destroy_sid = 0;
-	prefs->auto_layout_timeout = 0;
-	prefs->done_auto_layout = FALSE;
 }
 
 GType
@@ -253,8 +187,14 @@ prefs_link( Prefs *prefs, Workspace *ws )
 		G_CALLBACK( prefs_workspace_destroy_cb ), prefs );
 }
 
+static gint 
+prefs_column_compare( Column *a, Column *b )
+{
+	return( b->y - a->y );
+}
+
 Prefs *
-prefs_new( void )
+prefs_new( const char *caption_filter )
 {
 	Symbol *wsg_sym = main_workspacegroup->sym;
 	Symbol *ws_sym = SYMBOL( icontainer_child_lookup( 
@@ -272,7 +212,10 @@ prefs_new( void )
 		return( NULL );
 	}
 
+	icontainer_custom_sort( ICONTAINER( ws_sym->ws ),
+		(GCompareFunc) prefs_column_compare );
 	prefs = PREFS( g_object_new( TYPE_PREFS, NULL ) );
+	IM_SETSTR( prefs->caption_filter, caption_filter );
 	prefs_link( prefs, ws_sym->ws );
 
 	return( prefs );
