@@ -43,13 +43,11 @@ typedef struct _Prefs {
 	Workspace *ws;
 	guint destroy_sid;
 
-	Workspaceview *wsview;
+	Prefworkspaceview *pwview;
 
-	/* We layout the workspace a 'short while' after it appears. Should
-	 * reduce problems due to fonts etc. changing between platforms.
+	/* (optionally) filter prefs with this.
 	 */
-	guint auto_layout_timeout;
-	gboolean done_auto_layout;
+	char *caption_filter;
 } Prefs;
 
 typedef struct _PrefsClass {
@@ -60,6 +58,6 @@ typedef struct _PrefsClass {
 } PrefsClass;
 
 GType prefs_get_type( void );
-Prefs *prefs_new( void );
-gboolean prefs_set( const char *name, const char *fmt, ... );
-
+Prefs *prefs_new( const char *caption_filter );
+gboolean prefs_set( const char *name, const char *fmt, ... )
+	__attribute__((format(printf, 2, 3)));
