@@ -773,6 +773,7 @@ program_init( Program *program )
 
 	program->text = NULL;
 	program->dirty = FALSE;
+	program->text_hash = 0;
 	program->tree = NULL;
 	program->store = NULL;
 	program->pane_position = PROGRAM_PANE_POSITION;
@@ -838,15 +839,13 @@ program_kit_destroy( Toolkit *kit, Program *program )
 static void
 program_set_text( Program *program, const char *text, gboolean editable )
 {
-	GtkTextView *text_view = GTK_TEXT_VIEW( program->text );
+	guint text_hash = g_str_hash( text );
 
-	gtk_text_buffer_set_text( 
-		gtk_text_view_get_buffer( 
-			GTK_TEXT_VIEW( program->text ) ),
-		text ? text : "", -1 );
-
-	gtk_text_view_set_editable( text_view, editable );
-	gtk_text_view_set_cursor_visible( text_view, editable );
+	if( text_hash != program->text_hash ) {
+		text_view_set_text( GTK_TEXT_VIEW( program->text ), 
+			text, editable );
+		program->text_hash = text_hash;
+	}
 
 	program->dirty = FALSE;
 }
@@ -980,23 +979,6 @@ program_get_text( Program *program )
 	return( text );
 }
 
-/* Select a range in byte offset units. end == -1 means to end of buffer.
- */
-static void
-program_select_text( Program *program, int start, int end )
-{
-	GtkTextView *text_view = GTK_TEXT_VIEW( program->text );
-	GtkTextBuffer *text_buffer = gtk_text_view_get_buffer( text_view );
-	GtkTextMark *mark = gtk_text_buffer_get_insert( text_buffer );
-	GtkTextIter start_iter;
-	GtkTextIter end_iter;
-
-	gtk_text_buffer_get_iter_at_offset( text_buffer, &start_iter, start );
-	gtk_text_buffer_get_iter_at_offset( text_buffer, &end_iter, end );
-	gtk_text_buffer_select_range( text_buffer, &start_iter, &end_iter );
-	gtk_text_view_scroll_mark_onscreen( text_view, mark );
-}
-
 /* Read and parse the text.
  */
 static gboolean
@@ -1004,13 +986,16 @@ program_parse( Program *program )
 {
 	char *txt;
 	char buffer[MAX_STRSIZE];
+	Compile *compile;
 
 	if( !program->dirty )
 		return( TRUE );
 
-	/* Irritatingly, we need to append a ';'.
+	/* Irritatingly, we need to append a ';'. Also, update the hash, so we
+	 * don't set the same text back again if we can help it.
 	 */
 	txt = program_get_text( program );
+	program->text_hash = g_str_hash( txt );
 	im_snprintf( buffer, MAX_STRSIZE, "%s;", txt );
 	IM_FREEF( g_free, txt );
 
@@ -1021,6 +1006,7 @@ program_parse( Program *program )
 	 */
 	if( !program->kit )
 		program_select_kit_sub( program, program->kit );
+	compile = program->kit->kitg->root->expr->compile;
 
 #ifdef DEBUG
 	printf( "program_parse: parsing to kit \"%s\", pos %d\n",
@@ -1031,7 +1017,7 @@ program_parse( Program *program )
 	 */
 	attach_input_string( buffer );
 	if( !parse_onedef( program->kit, program->pos ) ) {
-		program_select_text( program, 
+		text_view_select_text( GTK_TEXT_VIEW( program->text ), 
 			input_state.charpos - yyleng, input_state.charpos );
 		return( FALSE );
 	}
@@ -1039,10 +1025,10 @@ program_parse( Program *program )
 	program->dirty = FALSE;
 	filemodel_set_modified( FILEMODEL( program->kit ), TRUE );
 
-	/* Reselect last_top_sym, the last thing the parser saw. 
+	/* Reselect last_sym, the last thing the parser saw. 
 	 */
-	if( last_top_sym && last_top_sym->tool ) 
-		program_select_tool( program, last_top_sym->tool );
+	if( compile->last_sym && compile->last_sym->tool ) 
+		program_select_tool( program, compile->last_sym->tool );
 
 	symbol_recalculate_all();
 
@@ -1372,7 +1358,7 @@ program_delete_action_cb( GtkAction *action, Program *program )
 static void
 program_select_all_action_cb( GtkAction *action, Program *program )
 {
-	program_select_text( program, 0, -1 );
+	text_view_select_text( GTK_TEXT_VIEW( program->text ), 0, -1 );
 }
 
 static void
@@ -1451,7 +1437,7 @@ program_find_done_cb( iWindow *iwnd, void *client,
 
 	if( program_find( program ) ) {
 		program_select_tool( program, program->find_sym->tool );
-		program_select_text( program, 
+		text_view_select_text( GTK_TEXT_VIEW( program->text ), 
 			program->find_start, program->find_end );
 	}
 	else {
@@ -1500,7 +1486,7 @@ program_find_again_action_cb( GtkAction *action, Program *program )
 
 	if( program_find( program ) ) {
 		program_select_tool( program, program->find_sym->tool );
-		program_select_text( program, 
+		text_view_select_text( GTK_TEXT_VIEW( program->text ), 
 			program->find_start, program->find_end );
 	}
 	else
@@ -1599,37 +1585,13 @@ program_trace_action_cb( GtkAction *action, Program *program )
 	(void) trace_new();
 }
 
-static void
-program_linkreport_action_cb( GtkAction *action, Program *program )
-{
-	BufInfo buf;
-	char txt[MAX_STRSIZE];
-	gboolean bad_links;
-
-	buf_init_static( &buf, txt, MAX_STRSIZE );
-	bad_links = FALSE;
-	(void) toolkitgroup_map( program->kitg,
-		(toolkit_map_fn) toolkit_linkreport, &buf, &bad_links );
-	if( !bad_links )
-		buf_appendf( &buf, _( "No unresolved symbols found." ) );
-
-	box_info( GTK_WIDGET( program ), _( "Link report." ), 
-		"%s", buf_all( &buf ) );
-}
-
 static void
 program_errorreport_action_cb( GtkAction *action, Program *program )
 {
-	BufInfo buf;
-	char txt[MAX_STRSIZE];
+	iError *ierror;
 
-	buf_init_static( &buf, txt, MAX_STRSIZE );
-	expr_error_print_all( &buf );
-	if( buf_is_empty( &buf ) )
-		buf_appendf( &buf, _( "No errors found." ) );
-
-	box_info( GTK_WIDGET( program ), _( "Error report." ), 
-		"%s", buf_all( &buf ) );
+	ierror = ierror_new( program->kitg );
+	gtk_widget_show( GTK_WIDGET( ierror ) ); 
 
 #ifdef DEBUG
 	/* Dump VIPS memory usage info for debugging.
@@ -1809,11 +1771,6 @@ static GtkActionEntry program_actions[] = {
 		N_( "Make a new trace window" ), 
 		G_CALLBACK( program_trace_action_cb ) },
 
-	{ "Link", 
-		NULL, N_( "_Link" ), NULL,
-		N_( "Show a link report" ), 
-		G_CALLBACK( program_linkreport_action_cb ) },
-
 	{ "Errors", 
 		NULL, N_( "_Errors" ), NULL,
 		N_( "Show all errors" ), 
@@ -1877,7 +1834,6 @@ static const char *program_menubar_ui_description =
 "    </menu>"
 "    <menu action='DebugMenu'>"
 "      <menuitem action='Trace'/>"
-"      <menuitem action='Link'/>"
 "      <menuitem action='Errors'/>"
 "    </menu>"
 "    <menu action='HelpMenu'>"
@@ -2161,6 +2117,24 @@ program_tabs_new( void )
 	return( tabs );
 }
 
+GtkWidget *
+program_text_new( void )
+{
+	PangoFontDescription *font_desc;
+	PangoTabArray *tabs;
+	GtkWidget *text;
+
+	text = gtk_text_view_new();
+	font_desc = pango_font_description_from_string( "Mono" );
+	gtk_widget_modify_font( text, font_desc );
+	pango_font_description_free( font_desc );
+	tabs = program_tabs_new();
+	gtk_text_view_set_tabs( GTK_TEXT_VIEW( text ), tabs );
+	pango_tab_array_free( tabs );
+
+	return( text );
+}
+
 static void
 program_build( Program *program, GtkWidget *vbox )
 {
@@ -2173,8 +2147,6 @@ program_build( Program *program, GtkWidget *vbox )
 	GtkTreeViewColumn *column;
 	GtkTreeSelection *select;
 	GtkWidget *swin;
-	PangoFontDescription *font_desc;
-	PangoTabArray *tabs;
 
         /* Make main menu bar
          */
@@ -2271,13 +2243,7 @@ program_build( Program *program, GtkWidget *vbox )
 	gtk_paned_pack2( GTK_PANED( program->pane ), swin, TRUE, TRUE );
 	gtk_widget_show( swin );
 
-	program->text = gtk_text_view_new();
-	font_desc = pango_font_description_from_string( "Mono" );
-	gtk_widget_modify_font( program->text, font_desc );
-	pango_font_description_free( font_desc );
-	tabs = program_tabs_new();
-	gtk_text_view_set_tabs( GTK_TEXT_VIEW( program->text ), tabs );
-	pango_tab_array_free( tabs );
+	program->text = program_text_new();
         g_signal_connect( 
 		gtk_text_view_get_buffer( GTK_TEXT_VIEW( program->text ) ),
 		"changed",
