@@ -46,6 +46,7 @@ struct _Program {
 	GtkUIManager *ui_manager;
 	GtkWidget *text;
 	gboolean dirty;		/* Has the text changed since we set it */
+	guint text_hash;	/* Hash of the last text we set */
 	GtkWidget *tree;
 	GtkWidget *pane;
 	int pane_position;
@@ -105,6 +106,7 @@ typedef struct _ProgramClass {
 } ProgramClass;
 
 GtkType program_get_type( void );
+GtkWidget *program_text_new( void );
 Program *program_new( Toolkitgroup *kitg );
 
 gboolean program_select( Program *program, Model *model );
