@@ -52,7 +52,7 @@ rhs_finalize( GObject *gobject )
 #ifdef DEBUG
 	printf( "rhs_destroy: " );
 	row_name_print( HEAPMODEL( rhs )->row );
-	printf( " (0x%x)\n", (unsigned int) rhs );
+	printf( " (%p)\n", rhs );
 #endif /*DEBUG*/
 
 	G_OBJECT_CLASS( parent_class )->finalize( gobject );
@@ -104,6 +104,12 @@ rhs_parent_add( iContainer *child )
 	ICONTAINER_CLASS( parent_class )->parent_add( child );
 }
 
+static View *
+rhs_view_new( Model *model, View *parent )
+{
+	return( rhsview_new() );
+}
+
 static gboolean
 rhs_load( Model *model, 
 	ModelLoadState *state, Model *parent, xmlNode *xnode )
@@ -148,27 +154,31 @@ rhs_save( Model *model, xmlNode *xnode )
 typedef struct {
 	const char *name;
 	GType (*type)( void );
-	Classmodel *(*build)( Rhs * );
 } RhsGraphic;
 
 /* All our graphicdisplay widgets. Order is important! Most-derived classes
  * first.
  */
 static RhsGraphic rhs_graphic[] = {
-	{ CLASS_EXPRESSION, expression_get_type, expression_new },
-	{ CLASS_GROUP, group_get_type, group_new },
-	{ CLASS_PATHNAME, pathname_get_type, pathname_new },
-	{ CLASS_FONTNAME, fontname_get_type, fontname_new },
-	{ CLASS_TOGGLE, toggle_get_type, toggle_new },
-	{ CLASS_SLIDER, slider_get_type, slider_new },
-	{ CLASS_COLOUR, colour_get_type, colour_new },
-	{ CLASS_OPTION, option_get_type, option_new },
-	{ CLASS_MATRIX, matrix_get_type, matrix_new },
-	{ CLASS_ARROW, iarrow_get_type, iarrow_new },
-	{ CLASS_REGION, iregion_get_type, iregion_new },
-	{ CLASS_IMAGE, iimage_get_type, iimage_new },
-	{ CLASS_NUMBER, number_get_type, number_new },
-	{ CLASS_STRING, string_get_type, string_new }
+	{ CLASS_CLOCK, clock_get_type },
+	{ CLASS_EXPRESSION, expression_get_type },
+	{ CLASS_GROUP, group_get_type },
+	{ CLASS_LIST, group_get_type },
+	{ CLASS_PATHNAME, pathname_get_type },
+	{ CLASS_FONTNAME, fontname_get_type },
+	{ CLASS_TOGGLE, toggle_get_type },
+	{ CLASS_SLIDER, slider_get_type },
+	{ CLASS_COLOUR, colour_get_type },
+	{ CLASS_OPTION, option_get_type },
+	{ CLASS_MATRIX, matrix_get_type },
+	{ CLASS_ARROW, iarrow_get_type },
+	{ CLASS_REGION, iregion_get_type },
+	{ CLASS_PLOT, plot_get_type },
+	{ CLASS_IMAGE, iimage_get_type },
+	{ CLASS_NUMBER, number_get_type },
+	{ CLASS_REAL, real_get_type },
+	{ CLASS_VECTOR, vector_get_type },
+	{ CLASS_STRING, string_get_type }
 };
 
 /* Create/destroy the graphic display.
@@ -180,7 +190,7 @@ rhs_refresh_graphic( Rhs *rhs, PElement *root )
 	Row *row = HEAPMODEL( rhs )->row;
 	int i;
 
-	if( !heap_isclass( root, &result ) ) 
+	if( !heap_is_class( root, &result ) ) 
 		return( FALSE );
 
 	/* Only for non-parameter class objects.
@@ -189,7 +199,7 @@ rhs_refresh_graphic( Rhs *rhs, PElement *root )
 		for( i = 0; i < IM_NUMBER( rhs_graphic ); i++ ) {
 			const char *name = rhs_graphic[i].name;
 
-			if( !heap_isinstanceof( name, root, &result ) ) 
+			if( !heap_is_instanceof( name, root, &result ) ) 
 				return( FALSE );
 			if( result ) 
 				break;
@@ -198,8 +208,8 @@ rhs_refresh_graphic( Rhs *rhs, PElement *root )
 		if( i != IM_NUMBER( rhs_graphic ) ) {
 			GType type = rhs_graphic[i].type();
 
-			if( !rhs->graphic || !TYPE_EXACT( rhs->graphic, type )) 
-				rhs_graphic[i].build( rhs );
+			if( !rhs->graphic || !TYPE_EXACT( rhs->graphic, type ) )
+				classmodel_new_classmodel( type, rhs );
 		}
 		else
 			/* Not a class we know about.
@@ -234,7 +244,7 @@ rhs_new_heap( Heapmodel *heapmodel, PElement *root )
 
 	/* Create/reuse/destroy class display. Only for non-param symbols.
 	 */
-	if( !heap_isclass( root, &result ) ) 
+	if( !heap_is_class( root, &result ) ) 
 		return( rhs );
 	if( result && row->sym->type != SYM_PARAM ) {
 		if( !rhs->scol || !IS_SUBCOLUMN( rhs->scol ) ) 
@@ -374,7 +384,7 @@ rhs_class_init( RhsClass *class )
 	icontainer_class->child_remove = rhs_child_remove;
 	icontainer_class->parent_add = rhs_parent_add;
 
-	model_class->view_new = rhsview_new;
+	model_class->view_new = rhs_view_new;
 	model_class->load = rhs_load;
 	model_class->save = rhs_save;
 
@@ -436,8 +446,7 @@ rhs_new( Row *row )
 	icontainer_child_add( ICONTAINER( row ), ICONTAINER( rhs ), -1 );
 
 #ifdef DEBUG
-	printf( "rhs_new: %s (0x%x)\n", 
-		MODEL( row )->name, (unsigned int) rhs );
+	printf( "rhs_new: %s (%p)\n", MODEL( row )->name, rhs );
 #endif /*DEBUG*/
 
 	return( rhs );
