@@ -200,7 +200,7 @@ statusview_changed_cb( Imagemodel *imagemodel, Statusview *sv )
 
 	Conversion *conv = imagemodel->conv;
 	iImage *iimage = imagemodel->iimage;
-	IMAGE *im = imageinfo_get( FALSE, iimage->instance.ii );
+	IMAGE *im = imageinfo_get( FALSE, iimage->value.ii );
 	double size = (double) im->Ysize * IM_IMAGE_SIZEOF_LINE( im );
 	unsigned int nb;
 	int fmt;
@@ -208,8 +208,7 @@ statusview_changed_cb( Imagemodel *imagemodel, Statusview *sv )
 	char txt[MAX_LINELENGTH];
 
 #ifdef DEBUG
-	printf( "statusview_changed_cb: 0x%x\n", 
-		(unsigned int) imagemodel );
+	printf( "statusview_changed_cb: %p\n", imagemodel );
 #endif /*DEBUG*/
 
 	widget_visible( GTK_WIDGET( sv ), imagemodel->show_status );
