@@ -200,6 +200,33 @@ symbol_name_print( Symbol *sym )
 	printf( "%s", symbol_name( sym ) );
 }
 
+/* Print a symbol's name, including the enclosing static scope. Return value
+ * is a pointer to a static buffer :(
+ */
+const char *
+symbol_name_scope( Symbol *sym )
+{
+	Symbol *scope = symbol_get_scope( sym );
+
+	static BufInfo buf;
+	static char txt[200];
+
+	buf_init_static( &buf, txt, 200 );
+	buf_appends( &buf, NN( IOBJECT( scope )->name ) );
+	buf_appends( &buf, "." );
+	symbol_qualified_name_relative( scope, sym, &buf );
+
+	return( buf_all( &buf ) );
+}
+
+/* Convenience ... print a qual name to stdout.
+ */
+void
+symbol_name_scope_print( Symbol *sym )
+{
+	printf( "%s", symbol_name_scope( sym ) );
+}
+
 void 
 symbol_new_value( Symbol *sym )
 {
@@ -238,6 +265,9 @@ symbol_clear( Symbol *sym )
 	sym->ndirtychildren = 0;
 	sym->leaf = FALSE;
 
+	sym->generated = FALSE;
+	sym->placeholder = FALSE;
+
 	sym->tool = NULL;
 
 	sym->function = NULL;
@@ -340,16 +370,16 @@ symbol_strip( Symbol *sym )
 	printf( "\n" );
 #endif /*DEBUG_MAKE*/
 
+	/* Anything that refers to us will need a recomp.
+	 */
+	if( is_top( sym ) ) 
+		symbol_dirty_intrans( sym, link_serial_new() );
+
 	/* Clean out old exprinfo.
 	 */
 	icontainer_map( ICONTAINER( sym ),
 		(icontainer_map_fn) expr_strip, NULL, NULL );
 
-	/* Set inital state again. 
-	 */
-	if( is_top( sym ) ) 
-		symbol_dirty_clear( sym );
-
 	/* Free any top-links we made.
 	 */
 	(void) slist_map( sym->topchildren, (SListMapFn) link_destroy, NULL );
@@ -430,20 +460,9 @@ static void *
 symbol_not_defined_sub( Link *link, BufInfo *buf )
 {
 	symbol_qualified_name( link->parent, buf );
-	buf_appends( buf, " " );
 
-	if( link->parent->tool ) {
-		Tool *tool = link->parent->tool;
-
-		buf_appends( buf, "(" );
-		buf_appendf( buf, _( "defined in toolkit \"%s\"" ),
-			IOBJECT( tool->kit )->name );
-		if( tool->lineno != -1 ) {
-			buf_appends( buf, ", " );
-			buf_appendf( buf, _( "line %d" ), tool->lineno );
-		}
-		buf_appends( buf, ") " );
-	}
+	if( link->parent->tool ) 
+		tool_error( link->parent->tool, buf );
 
 	return( NULL );
 }
@@ -711,7 +730,7 @@ symbol_new_defining( Compile *compile, const char *name )
 			/* Redef of existing symbol? Only allowed at top
 			 * level.
 			 */
-			if( compile->sym != symbol_root )
+			if( !is_scope( compile->sym ) )
 				symbol_error_redefine( sym );
 			break;
 
@@ -794,13 +813,15 @@ symbol_user_init( Symbol *sym )
 /* Specialise into a parameter on an expression.
  */
 gboolean
-symbol_parameter_init( Symbol *sym, Compile *compile )
+symbol_parameter_init( Symbol *sym )
 {
+	Compile *parent = COMPILE( ICONTAINER( sym )->parent );
+
 	assert( sym->type == SYM_ZOMBIE );
 
 	sym->type = SYM_PARAM;
-	compile->param = g_slist_append( compile->param, sym );
-	compile->nparam = g_slist_length( compile->param );
+	parent->param = g_slist_append( parent->param, sym );
+	parent->nparam = g_slist_length( parent->param );
 	symbol_made( sym );
 
 	return( TRUE );
@@ -934,8 +955,8 @@ symbol_recalculate_sub( Symbol *sym )
 
 #ifdef DEBUG_TIME
 	printf( "symbol_recalculate_sub: " );
-	symbol_name_print( sym );
-	printf( " - %gs\n", g_timer_elapsed( timer, NULL ) );
+	symbol_name_scope_print( sym );
+	printf( " %g\n", g_timer_elapsed( timer, NULL ) );
 #endif /*DEBUG_TIME*/
 
 	return( result );
@@ -952,6 +973,10 @@ static gboolean symbol_running = FALSE;
 void *
 symbol_recalculate_check( Symbol *sym )
 {
+#ifdef DEBUG
+	printf( "symbol_recalculate_check: %s\n", symbol_name( sym ) );
+#endif /*DEBUG*/
+
 	error_clear();
 	if( sym->expr->err ) {
 		expr_error_get( sym->expr );
