@@ -57,7 +57,9 @@ typedef enum {
 struct _Symbol {
 	Filemodel parent_class;
 
-	SymbolType type; 	/* The type of this symbol */
+	/* The type of this symbol.
+	 */
+	SymbolType type;
 
 	/* Track during parse. A list of pointers to pointers to this
 	 * symbol which we need to patch if we resolve to an outer scope.
@@ -78,13 +80,22 @@ struct _Symbol {
 	/* Recomputation links. Use these to work out what to build next.
 	 */
 	gboolean dirty;		/* True if this sym needs recalc */
-	GSList *parents;	/* Compile which refer to this sym */
+	GSList *parents;	/* Compiles which refer to this sym */
 
 	GSList *topchildren;	/* For top syms, all top-level children */
 	GSList *topparents;	/* For top syms, all top-level parents */
 	int ndirtychildren;	/* Number of dirty top syms we refer to */
 	gboolean leaf;		/* True for in recomp set */
 
+	/* This is a generated symbol, like $$result, $$fn1, whatever.
+	 */
+	gboolean generated;
+
+	/* A temporary intermediate symbol generated during parse to hold 
+	 * stuff until we need it. Don't generate code for these.
+	 */
+	gboolean placeholder;
+
 	/* X-tras for definitions.
 	 */
 	Tool *tool;		/* Tool and toolkit defined in */
@@ -138,6 +149,8 @@ void symbol_qualified_name_relative( Symbol *context,
 	Symbol *sym, BufInfo *buf );
 const char *symbol_name( Symbol *sym );
 void symbol_name_print( Symbol *sym );
+const char *symbol_name_scope( Symbol *sym );
+void symbol_name_scope_print( Symbol *sym );
 
 void symbol_new_value( Symbol *sym );
 
@@ -156,7 +169,7 @@ void symbol_not_defined( Symbol *sym );
 void *symbol_link_break( Symbol *child, Compile *compile );
 
 gboolean symbol_user_init( Symbol *sym );
-gboolean symbol_parameter_init( Symbol *sym, Compile *compile );
+gboolean symbol_parameter_init( Symbol *sym );
 gboolean symbol_parameter_builtin_init( Symbol *sym );
 
 gboolean symbol_busy( void );
