@@ -42,6 +42,21 @@ static FilemodelClass *parent_class = NULL;
  */
 #define MAX_NAME (256)
 
+void
+tool_error( Tool *tool, BufInfo *buf )
+{
+	if( tool->lineno != -1 ) {
+		buf_appends( buf, " (" );
+		if( FILEMODEL( tool->kit )->filename )
+			buf_appends( buf, FILEMODEL( tool->kit )->filename );
+		else
+			buf_appends( buf, IOBJECT( tool->kit )->name );
+		buf_appendf( buf, ":%d)", tool->lineno );
+	}
+}
+
+static void *toolitem_free( Toolitem *toolitem );
+
 /* Remove a tool. Also strip the sym, if any.
  */
 static void 
@@ -70,9 +85,18 @@ tool_dispose( GObject *gobject )
 		tool->sym = NULL;
 
 		symbol_strip( sym );
+
+		/* Anything that referred to this symbol is going to need a
+		 * recalc.
+		 */
 	}
 
-	filemodel_set_modified( FILEMODEL( tool->kit ), TRUE );
+	if( tool->kit ) {
+		filemodel_set_modified( FILEMODEL( tool->kit ), TRUE );
+		tool->kit = NULL;
+	}
+
+	IM_FREEF( toolitem_free, tool->toolitem );
 
 	G_OBJECT_CLASS( parent_class )->dispose( gobject );
 }
@@ -223,6 +247,10 @@ tool_get_type( void )
 static void
 tool_link( Tool *tool, Toolkit *kit, int pos, const char *name )
 {
+#ifdef DEBUG
+	printf( "tool_link: %s\n", name );
+#endif /*DEBUG*/
+
 	filemodel_set_modified( FILEMODEL( kit ), TRUE );
 	iobject_set( IOBJECT( tool ), name, NULL );
 	icontainer_child_add( ICONTAINER( kit ), ICONTAINER( tool ), pos );
