@@ -234,12 +234,11 @@ toolkitbrowser_init( Toolkitbrowser *toolkitbrowser )
 
 	toolkitbrowser->top = gtk_hbox_new( FALSE, 12 );
 	toolkitbrowser->entry = gtk_entry_new();
-	gtk_entry_set_width_chars( GTK_ENTRY( toolkitbrowser->entry ), 20 );
         gtk_signal_connect( GTK_OBJECT( toolkitbrowser->entry ), "changed", 
 		GTK_SIGNAL_FUNC( toolkitbrowser_entry_changed_cb ), 
 		toolkitbrowser );
 	gtk_box_pack_end( GTK_BOX( toolkitbrowser->top ), 
-		toolkitbrowser->entry, FALSE, FALSE, 2 );
+		toolkitbrowser->entry, TRUE, TRUE, 2 );
 	label = gtk_label_new( _( "Search:" ) );
 	gtk_box_pack_end( GTK_BOX( toolkitbrowser->top ), 
 		label, FALSE, FALSE, 0 );
