@@ -50,7 +50,7 @@ toolview_destroy( GtkObject *object )
 	tview = TOOLVIEW( object );
 
 #ifdef DEBUG
-	printf( "toolview_destroy: 0x%x\n", (unsigned int) object );
+	printf( "toolview_destroy: %p\n", object );
 #endif /*DEBUG*/
 
 	DESTROY_GTK( tview->item );
@@ -62,7 +62,7 @@ static void
 toolview_finalize( GObject *gobject )
 {
 #ifdef DEBUG
-	printf( "toolview_finalize: 0x%x\n", (unsigned int) gobject );
+	printf( "toolview_finalize: %p\n", gobject );
 #endif /*DEBUG*/
 
 	G_OBJECT_CLASS( parent_class )->finalize( gobject );
@@ -312,7 +312,7 @@ toolview_new( void )
 	Toolview *tview = gtk_type_new( TYPE_TOOLVIEW );
 
 #ifdef DEBUG
-	printf( "toolview_new: 0x%x\n", (unsigned int) tview );
+	printf( "toolview_new: %p\n", tview );
 #endif /*DEBUG*/
 
 	return( VIEW( tview ) );
