@@ -37,7 +37,7 @@
  */
 TraceFlags trace_flags = (TraceFlags) 0;
 
-static iWindowClass *parent_class = NULL;
+static LogClass *parent_class = NULL;
 
 /* All trace windows.
  */
@@ -210,60 +210,6 @@ trace_destroy( GtkObject *object )
 	trace_global_rethink();
 }
 
-static void
-trace_class_init( TraceClass *class )
-{
-	GtkObjectClass *object_class = (GtkObjectClass *) class;
-
-	parent_class = g_type_class_peek_parent( class );
-
-	object_class->destroy = trace_destroy;
-
-	/* Create signals.
-	 */
-
-	/* Init methods.
-	 */
-}
-
-static void
-trace_init( Trace *trace )
-{
-	trace->flags = 0;
-}
-
-GtkType
-trace_get_type( void )
-{
-	static GtkType trace_type = 0;
-
-	if( !trace_type ) {
-		static const GtkTypeInfo info = {
-			"Trace",
-			sizeof( Trace ),
-			sizeof( TraceClass ),
-			(GtkClassInitFunc) trace_class_init,
-			(GtkObjectInitFunc) trace_init,
-			/* reserved_1 */ NULL,
-			/* reserved_2 */ NULL,
-			(GtkClassInitFunc) NULL,
-		};
-
-		trace_type = gtk_type_unique( TYPE_IWINDOW, &info );
-	}
-
-	return( trace_type );
-}
-
-static void
-trace_clear_action_cb( GtkAction *action, Trace *trace )
-{
-	GtkTextView *text_view = GTK_TEXT_VIEW( trace->view );
-	GtkTextBuffer *text_buffer = gtk_text_view_get_buffer( text_view );
-
-	gtk_text_buffer_set_text( text_buffer, "", 0 );
-}
-
 static void
 trace_view_action_cb( GtkToggleAction *action, Trace *trace )
 {
@@ -291,7 +237,7 @@ static GtkActionEntry trace_actions[] = {
 	{ "Clear", 
 		NULL, N_( "_Clear" ), NULL, 
 		N_( "Clear trace window" ), 
-		G_CALLBACK( trace_clear_action_cb ) },
+		G_CALLBACK( log_clear_action_cb ) },
 
 	{ "Close", 
 		GTK_STOCK_CLOSE, N_( "_Close" ), NULL, 
@@ -353,63 +299,51 @@ static const char *trace_menubar_ui_description =
 "</ui>";
 
 static void
-trace_build( Trace *trace, GtkWidget *vbox )
+trace_class_init( TraceClass *class )
 {
-	GtkActionGroup *action_group;
-	GtkUIManager *ui_manager;
-	GtkAccelGroup *accel_group;
-	GError *error;
-	GtkWidget *mbar;
-	GtkWidget *swin;
-	PangoFontDescription *font_desc;
+	GtkObjectClass *object_class = (GtkObjectClass *) class;
+	LogClass *log_class = (LogClass *) class;
 
-        /* Make main menu bar
-         */
-	action_group = gtk_action_group_new( "TraceActions" );
-	gtk_action_group_set_translation_domain( action_group, 
-		GETTEXT_PACKAGE );
-	gtk_action_group_add_actions( action_group, 
-		trace_actions, G_N_ELEMENTS( trace_actions ), 
-		GTK_WINDOW( trace ) );
-	gtk_action_group_add_toggle_actions( action_group, 
-		trace_toggle_actions, G_N_ELEMENTS( trace_toggle_actions ), 
-		GTK_WINDOW( trace ) );
+	parent_class = g_type_class_peek_parent( class );
 
-	ui_manager = gtk_ui_manager_new();
-	gtk_ui_manager_insert_action_group( ui_manager, action_group, 0 );
+	object_class->destroy = trace_destroy;
 
-	accel_group = gtk_ui_manager_get_accel_group( ui_manager );
-	gtk_window_add_accel_group( GTK_WINDOW( trace ), accel_group );
+	log_class->actions = trace_actions;
+	log_class->n_actions = IM_NUMBER( trace_actions );
+	log_class->toggle_actions = trace_toggle_actions;
+	log_class->n_toggle_actions = IM_NUMBER( trace_toggle_actions );
+	log_class->action_name = "TraceActions";
+	log_class->ui_description = trace_menubar_ui_description;
+	log_class->menu_bar_name = "/TraceMenubar";
+}
 
-	error = NULL;
-	if( !gtk_ui_manager_add_ui_from_string( ui_manager,
-			trace_menubar_ui_description, -1, &error ) ) {
-		g_message( "building menus failed: %s", error->message );
-		g_error_free( error );
-		exit( EXIT_FAILURE );
+static void
+trace_init( Trace *trace )
+{
+	trace->flags = 0;
+}
+
+GtkType
+trace_get_type( void )
+{
+	static GtkType type = 0;
+
+	if( !type ) {
+		static const GtkTypeInfo info = {
+			"Trace",
+			sizeof( Trace ),
+			sizeof( TraceClass ),
+			(GtkClassInitFunc) trace_class_init,
+			(GtkObjectInitFunc) trace_init,
+			/* reserved_1 */ NULL,
+			/* reserved_2 */ NULL,
+			(GtkClassInitFunc) NULL,
+		};
+
+		type = gtk_type_unique( TYPE_LOG, &info );
 	}
 
-	mbar = gtk_ui_manager_get_widget( ui_manager, 
-		"/TraceMenubar" );
-	gtk_box_pack_start( GTK_BOX( vbox ), mbar, FALSE, FALSE, 0 );
-        gtk_widget_show( mbar );
-
-	swin = gtk_scrolled_window_new( NULL, NULL );
-	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( swin ),
-		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
-	gtk_box_pack_start( GTK_BOX( vbox ), swin, TRUE, TRUE, 0 );
-	gtk_widget_show( swin );
-
-	trace->view = gtk_text_view_new();
-	gtk_text_view_set_editable( GTK_TEXT_VIEW( trace->view ), FALSE );
-	gtk_text_view_set_cursor_visible( GTK_TEXT_VIEW( trace->view ), 
-		FALSE );
-	font_desc = pango_font_description_from_string( "Mono" );
-	gtk_widget_modify_font( trace->view, font_desc );
-	pango_font_description_free( font_desc );
-
-	gtk_container_add( GTK_CONTAINER( swin ), trace->view );
-	gtk_widget_show( trace->view );
+	return( type );
 }
 
 static void
@@ -417,8 +351,6 @@ trace_link( Trace *trace )
 {
         iwindow_set_title( IWINDOW( trace ), _( "Trace" ) );
 	gtk_window_set_default_size( GTK_WINDOW( trace ), 640, 480 );
-	iwindow_set_build( IWINDOW( trace ), 
-		(iWindowBuildFn) trace_build, NULL, NULL, NULL );
 	iwindow_set_size_prefs( IWINDOW( trace ), 
 		"TRACE_WIDTH", "TRACE_HEIGHT" );
 	iwindow_build( IWINDOW( trace ) );
@@ -440,19 +372,8 @@ trace_new( void )
 static void *
 trace_text_sub( Trace *trace, const char *buf, TraceFlags flags )
 {
-	if( !trace_block_count && trace->flags & flags ) {
-		GtkTextView *text_view = GTK_TEXT_VIEW( trace->view );
-		GtkTextBuffer *text_buffer = 
-			gtk_text_view_get_buffer( text_view );
-		GtkTextMark *mark = gtk_text_buffer_get_insert( text_buffer );
-		GtkTextIter iter;
-
-		gtk_text_buffer_get_end_iter( text_buffer, &iter );
-		gtk_text_buffer_move_mark( text_buffer, mark, &iter );
-		gtk_text_buffer_insert_at_cursor( text_buffer, buf, -1 );
-		gtk_text_view_scroll_to_mark( text_view, mark, 
-			0.0, TRUE, 0.5, 1 );
-	}
+	if( !trace_block_count && trace->flags & flags ) 
+		log_text( LOG( trace ), buf );
 
 	return( NULL );
 }
