@@ -39,12 +39,21 @@ tree_const_destroy( ParseConst *pc )
 	pc->type = PARSE_CONST_NONE;
 }
 
+void
+tree_const_copy( ParseConst *from, ParseConst *to )
+{
+	*to = *from;
+	if( to->type == PARSE_CONST_STR && to->val.str )
+		to->val.str = im_strdupn( to->val.str );
+}
+
 /* Free a parse node.
  */
 void *
 tree_node_destroy( ParseNode *n )
 {
 	switch( n->type ) {
+	case NODE_PATTERN_CLASS:
 	case NODE_TAG:
 		IM_FREE( n->tag );
 		break;
@@ -229,25 +238,20 @@ tree_unop_new( Compile *compile, UnOp op, ParseNode *a )
 	return( no );
 }
 
-/* Make a new leaf node. 
- */
 ParseNode *
 tree_leaf_new( Compile *compile, const char *name )
 {
 	ParseNode *no = tree_new( compile );
-	Symbol *sym = symbol_new_reference( compile, name );
 
-	/* Fill fields.
-	 */
 	no->type = NODE_LEAF;
-	no->leaf = sym;
+	no->leaf = symbol_new_reference( compile, name );
 
 	/* Have we a reference to a ZOMBIE? If yes, we may need to patch this
 	 * leaf to point to a new symbol. Add the leaf's pointer to the
 	 * refedat list on the ZOMBIE.
 	 */
-	if( sym->type == SYM_ZOMBIE )
-		(void) symbol_patch_add( (void **) &no->leaf, sym );
+	if( no->leaf->type == SYM_ZOMBIE )
+		(void) symbol_patch_add( (void **) &no->leaf, no->leaf );
 
 	return( no );
 }
@@ -264,6 +268,10 @@ tree_leafsym_new( Compile *compile, Symbol *sym )
 	no->type = NODE_LEAF;
 	no->leaf = sym;
 
+	/* Note that this compile refs this sym.
+	 */
+	compile_link_make( compile, sym );
+
 	/* Have we a reference to a ZOMBIE? If yes, we may need to patch this
 	 * leaf to point to a new symbol. Add the leaf's pointer to the
 	 * refedat list on the ZOMBIE.
@@ -353,3 +361,139 @@ tree_appl_new( Compile *compile, ParseNode *l, ParseNode *r )
 
 	return( no );
 }
+
+ParseNode *
+tree_pattern_class_new( Compile *compile, const char *class_name, ParseNode *l )
+{
+	ParseNode *no = tree_new( compile );
+
+	no->type = NODE_PATTERN_CLASS;
+	no->arg1 = l;
+	no->tag = im_strdupn( class_name );
+
+	return( no );
+}
+
+ParseNode *
+tree_map( Compile *compile, tree_map_fn fn, ParseNode *node, void *a, void *b ) 
+{
+	ParseNode *result;
+	GSList *l;
+
+	g_assert( node );
+
+	if( (result = fn( compile, node, a, b )) )
+		return( result );
+
+	switch( node->type ) {
+	case NODE_GENERATOR:
+		if( (result = tree_map( compile, fn, node->arg1, a, b )) )
+			return( result );
+		if( node->arg2 &&
+			(result = tree_map( compile, fn, node->arg2, a, b )) )
+			return( result );
+		if( node->arg3 &&
+			(result = tree_map( compile, fn, node->arg3, a, b )) )
+			return( result );
+		break;
+
+	case NODE_APPLY:
+	case NODE_BINOP:
+	case NODE_COMPOSE:
+		if( (result = tree_map( compile, fn, node->arg1, a, b )) ||
+			(result = tree_map( compile, fn, node->arg2, a, b )) )
+			return( result );
+		break;
+
+	case NODE_UOP:
+		if( (result = fn( compile, node->arg1, a, b )) )
+			return( result );
+		break;
+
+	case NODE_SUPER:
+	case NODE_LISTCONST:
+		for( l = node->elist; l; l = l->next ) {
+			ParseNode *arg = (ParseNode *) l->data;
+
+			if( (result = tree_map( compile, fn, arg, a, b )) )
+				return( result );
+		}
+		break;
+
+	case NODE_LEAF:
+	case NODE_CLASS:
+	case NODE_TAG:
+	case NODE_CONST:
+		break;
+
+	case NODE_NONE:
+	default:
+		assert( FALSE );
+	}
+
+	return( NULL );
+}
+
+/* Copy a tree to a new context. Make all symbols afresh ... you need to link
+ * after calling this. 
+ */
+ParseNode *
+tree_copy( Compile *compile, ParseNode *node )
+{
+	ParseNode *copy;
+	GSList *l;
+
+	assert( node );
+
+	switch( node->type ) {
+	case NODE_GENERATOR:
+	case NODE_APPLY:
+	case NODE_BINOP:
+	case NODE_COMPOSE:
+	case NODE_UOP:
+	case NODE_TAG:
+	case NODE_CONST:
+	case NODE_PATTERN_CLASS:
+		copy = tree_new( compile );
+		copy->type = node->type;
+		copy->uop = node->uop;
+		copy->biop = node->biop;
+		if( node->tag )
+			copy->tag = im_strdupn( node->tag );
+		tree_const_copy( &node->con, &copy->con );
+		if( node->arg1 )
+			copy->arg1 = tree_copy( compile, node->arg1 );
+		if( node->arg2 )
+			copy->arg2 = tree_copy( compile, node->arg2 );
+		if( node->arg3 )
+			copy->arg3 = tree_copy( compile, node->arg3 );
+		break;
+
+	case NODE_SUPER:
+	case NODE_LISTCONST:
+		copy = tree_new( compile );
+		for( l = node->elist; l; l = l->next ) {
+			ParseNode *arg = (ParseNode *) l->data;
+
+			copy->elist = g_slist_append( copy->elist, 
+				tree_copy( compile, arg ) );
+		}
+		copy->type = node->type;
+		break;
+
+	case NODE_CLASS:
+		copy = tree_class_new( compile );
+		break;
+
+	case NODE_LEAF:
+		copy = tree_leaf_new( compile, IOBJECT( node->leaf )->name );
+		break;
+
+	case NODE_NONE:
+	default:
+		assert( FALSE );
+	}
+
+	return( copy );
+}
+
