@@ -41,6 +41,7 @@ typedef enum {
 	NODE_GENERATOR,		/* A list generator */
 	NODE_COMPOSE,		/* Function composition */
 	NODE_SUPER,		/* Superclass constructor */
+	NODE_PATTERN_CLASS,	/* A class pattern match */
 	NODE_LISTCONST		/* A list expression "[1, 2, 3]" */
 } NodeTypes;
 
@@ -75,8 +76,10 @@ typedef enum {
 	BI_PNOTEQ,
 	BI_LESS,		/* Relational ops */
 	BI_LESSEQ,
+	BI_MORE,		
+	BI_MOREEQ,
 	BI_IF,			/* if-then-else */
-	BI_CONS			/* List cons */
+	BI_CONS			/* List cons ... has to be last, see below */
 } BinOp;
 
 /* Unary operators.
@@ -136,13 +139,15 @@ struct _ParseNode {
 
 	NodeTypes type;
 
-	/* Bundle for node types with up to three arguments.
-	 * Unary ops, binary ops, application and ifthenelse ops.
+	/* Bundle for node types with up to two arguments.
 	 */
 	BinOp biop;
 	UnOp uop;
 	ParseNode *arg1;
 	ParseNode *arg2;
+
+	/* Just for generators, eg. [a, b .. c]
+	 */
 	ParseNode *arg3;
 
 	/* A symbol reference.
@@ -186,5 +191,16 @@ ParseNode *tree_leafsym_new( Compile *compile, Symbol *sym );
 ParseNode *tree_const_new( Compile *compile, ParseConst n );
 ParseNode *tree_class_new( Compile *compile );
 ParseNode *tree_compose_new( Compile *compile, ParseNode *f, ParseNode *g );
+ParseNode *tree_pattern_new( Compile *compile );
+ParseNode *tree_pattern_class_new( Compile *compile, 
+	const char *class_name, ParseNode *l );
 
 void *tree_node_destroy( ParseNode *n );
+
+typedef ParseNode *(*tree_map_fn)( Compile *, ParseNode *, void *, void * );
+ParseNode *tree_map( Compile *compile, 
+	tree_map_fn fn, ParseNode *node, void *a, void *b );
+
+/* Copy a tree into a new context.
+ */
+ParseNode *tree_copy( Compile *compile, ParseNode *node );
