@@ -55,6 +55,16 @@
 	} \
 } while( 0 )
 
+/* Swap two pointers.
+ */
+#define SWAPP( A, B ) { \
+	void *swapp_t; \
+ 	\
+	swapp_t = (A); \
+	(A) = (B); \
+	(B) = swapp_t; \
+}
+
 /* A string in the process of being written to ... multiple calls to 
  * buf_append add to it, on overflow append "..." and block further writes.
  */
@@ -331,13 +341,4 @@ GSList *recent_load( const char *filename );
 void recent_free( GSList *recent );
 void recent_save( GSList *recent, const char *filename );
 
-/* Tag our thumbnail drag-n-drops with these. Start up a bit to leave room for 
- * filedrop.
- */
-enum {
-	TARGET_SYMBOL = 99
-};
-
-void set_symbol_drag_type( GtkWidget *widget );
-
 const char *get_savedir( void );
