@@ -105,16 +105,35 @@ view_reset_all( void )
 	(void) slist_map( view_resettable, (SListMapFn) view_reset, NULL );
 }
 
+/* Should a viewchild be displayed? If model->display is true, also give the
+ * enclosing view a chance to filter.
+ */
+static gboolean
+view_viewchild_display( ViewChild *viewchild )
+{
+	View *parent_view = viewchild->parent_view;
+	Model *child_model = viewchild->child_model;
+	ViewClass *parent_view_class = VIEW_GET_CLASS( parent_view );
+
+	if( child_model->display && parent_view_class->display )
+		return( parent_view_class->display( parent_view, 
+			child_model ) );
+
+	return( child_model->display );
+}
+
 /* One of the children of the model we watch has changed ... create or destroy
  * the child view as required.
  */
 static void
 view_viewchild_changed( Model *model, ViewChild *viewchild )
 {
+	gboolean display = view_viewchild_display( viewchild );
+
 	/* Don't write to the viewchild->view_child pointer ... let
 	 * view_real_child_add() and view_real_child_remove() do this.
 	 */
-	if( !model->display && viewchild->child_view ) {
+	if( !display && viewchild->child_view ) {
 #ifdef DEBUG_VIEWCHILD
 		printf( "view_viewchild_changed: %s \"%s\", removing view\n", 
 			G_OBJECT_TYPE_NAME( model ), 
@@ -123,22 +142,19 @@ view_viewchild_changed( Model *model, ViewChild *viewchild )
 
 		gtk_widget_destroy( GTK_WIDGET( viewchild->child_view ) );
 	}
-	else if( model->display && !viewchild->child_view ) {
-		View *child_view;
-
+	else if( display && !viewchild->child_view ) {
 #ifdef DEBUG_VIEWCHILD
 		printf( "view_viewchild_changed: %s \"%s\", adding view\n", 
 			G_OBJECT_TYPE_NAME( model ), 
 			NN( IOBJECT( model )->name ) );
 #endif /*DEBUG_VIEWCHILD*/
 
-		child_view = model_view_new( model );
-		view_link( child_view, model, viewchild->view );
+		model_view_new( model, viewchild->parent_view );
 	}
 }
 
 static ViewChild *
-view_viewchild_new( View *view, Model *child_model )
+view_viewchild_new( View *parent_view, Model *child_model )
 {
 	ViewChild *viewchild;
 
@@ -152,14 +168,15 @@ view_viewchild_new( View *view, Model *child_model )
 	if( !(viewchild = INEW( NULL, ViewChild )) )
 		return( NULL );
 
-	viewchild->view = view;
+	viewchild->parent_view = parent_view;
 	viewchild->child_model = child_model;
 	viewchild->child_model_changed_sid = 
 		g_signal_connect( child_model, "changed", 
 			G_CALLBACK( view_viewchild_changed ), viewchild );
 	viewchild->child_view = NULL;
 
-	view->managed = g_slist_append( view->managed, viewchild );
+	parent_view->managed = 
+		g_slist_append( parent_view->managed, viewchild );
 
 	/* Make a view for this child, if we need one.
 	 */
@@ -171,17 +188,18 @@ view_viewchild_new( View *view, Model *child_model )
 static void *
 view_viewchild_destroy( ViewChild *viewchild )
 {
-	View *view = viewchild->view;
+	View *parent_view = viewchild->parent_view;
 
 #ifdef DEBUG_VIEWCHILD
 	printf( "view_viewchild_destroy: view %s watching model %s\n",
-		G_OBJECT_TYPE_NAME( viewchild->view ), 
+		G_OBJECT_TYPE_NAME( viewchild->parent_view ), 
 		G_OBJECT_TYPE_NAME( viewchild->child_model ) );
 #endif /*DEBUG_VIEWCHILD*/
 
 	DESTROY_GTK( viewchild->child_view );
 	FREESID( viewchild->child_model_changed_sid, viewchild->child_model );
-	view->managed = g_slist_remove( view->managed, viewchild );
+	parent_view->managed = 
+		g_slist_remove( parent_view->managed, viewchild );
 
 	im_free( viewchild );
 
@@ -246,6 +264,26 @@ view_child_remove( View *child )
 	VIEW_GET_CLASS( parent )->child_remove( parent, child );
 }
 
+/* Child needs repositioning.
+ */
+void
+view_child_position( View *child )
+{
+	View *parent = child->parent;
+
+	VIEW_GET_CLASS( parent )->child_position( parent, child );
+}
+
+/* Pop child to front of stacking order.
+ */
+void
+view_child_front( View *child )
+{
+	View *parent = child->parent;
+
+	VIEW_GET_CLASS( parent )->child_front( parent, child );
+}
+
 /* Break link to model. 
  */
 void 
@@ -329,7 +367,7 @@ view_model_pos_changed( Model *model, View *view )
 /* Called for model scrollto signal ... try scrolling.
  */
 static void
-view_model_scrollto( Model *model, View *view )
+view_model_scrollto( Model *model, ModelScrollPosition position, View *view )
 {
 	assert( IS_MODEL( model ) );
 	assert( IS_VIEW( view ) );
@@ -338,7 +376,7 @@ view_model_scrollto( Model *model, View *view )
 	printf( "view_model_scrollto: %s\n", IOBJECT( model )->name );
 #endif /*DEBUG*/
 
-	view_scrollto( view );
+	view_scrollto( view, position );
 }
 
 /* Called for model layout signal ... try to lay out children.
@@ -528,6 +566,16 @@ view_real_child_remove( View *parent, View *child )
 	child->parent = NULL;
 }
 
+static void
+view_real_child_position( View *parent, View *child )
+{
+}
+
+static void
+view_real_child_front( View *parent, View *child )
+{
+}
+
 static void 
 view_real_reset( View *view )
 {
@@ -576,6 +624,9 @@ view_class_init( ViewClass *class )
 	class->link = view_real_link;
 	class->child_add = view_real_child_add;
 	class->child_remove = view_real_child_remove;
+	class->child_position = view_real_child_position;
+	class->child_front = view_real_child_front;
+	class->display = NULL;
 
 	class->reset = view_real_reset;
 	class->scan = view_real_scan;
@@ -653,12 +704,12 @@ view_scan( View *view )
 /* Trigger the scrollto method for a view.
  */
 void *
-view_scrollto( View *view )
+view_scrollto( View *view, ModelScrollPosition position )
 {
 	ViewClass *view_class = VIEW_GET_CLASS( view );
 
 	if( view_class->scrollto )
-		view_class->scrollto( view );
+		view_class->scrollto( view, position );
 
 	return( NULL );
 }
@@ -794,7 +845,7 @@ view_get_columnview( View *child )
 }
 
 /* A view has changed size ... rethink the enclosing column geo. Helps table
- * to work good.
+ * to not break.
  */
 void *
 view_resize( View *view )
