@@ -32,6 +32,7 @@
 /*
 #define DEBUG_TIME
 #define DEBUG
+#define DEBUG_HISTORY_SANITY
 #define DEBUG_HISTORY_MISS
 #define DEBUG_HISTORY
  */
@@ -41,10 +42,10 @@
  */
 
 /* Often want it off ... we get spurious complaints about leaks if an
- * operation has no images in or out (eg. im_version) becausew it'll never
+ * operation has no images in or out (eg. im_version) because it'll never
  * get GCed.
- */
 #undef DEBUG_LEAK
+ */
 
 /* Maxiumum number of args to a VIPS function.
  */
@@ -184,7 +185,7 @@ vips_type_find( im_arg_type type )
 	return( VIPS_NONE );
 }
 
-/* Hash from a vargv ... just look at input args.
+/* Hash from a vargv ... just look at input args and the function name.
  */
 static unsigned int
 vips_hash( VipsInfo *vi )
@@ -208,6 +209,11 @@ vips_hash( VipsInfo *vi )
 #define HASH_P( P ) hash = (hash << 1) | (GPOINTER_TO_UINT( P ));
 #define HASH_S( S ) hash = (hash << 1) | g_str_hash( S );
 
+	/* Add the function to the hash. We often call many functions on
+	 * the same args, we'd like these calls to hash to different numbers.
+	 */
+	HASH_P( vi->fn );
+
         for( i = 0; i < vi->fn->argc; i++ ) {
                 im_type_desc *vips = vi->fn->argv[i].desc;
 		VipsArgumentType vt = vips_type_find( vips->type );
@@ -263,13 +269,20 @@ vips_hash( VipsInfo *vi )
 			{
 				im_mask_object *mo = vi->vargv[i];
 				DOUBLEMASK *mask = mo->mask;
-				int ne = mask->xsize * mask->ysize;
-				int i;
 
-				for( i = 0; i < ne; i++ )
-					HASH_D( mask->coeff[i] );
-				HASH_D( mask->scale );
-				HASH_D( mask->offset );
+				/* mask can be NULL if we are called after 
+				 * vips_new() but before we've built the arg
+				 * list.
+				 */
+				if( mask ) {
+					int ne = mask->xsize * mask->ysize;
+					int j;
+
+					for( j = 0; j < ne; j++ )
+						HASH_D( mask->coeff[j] );
+					HASH_D( mask->scale );
+					HASH_D( mask->offset );
+				}
 
 				break;
 			}
@@ -278,13 +291,20 @@ vips_hash( VipsInfo *vi )
 			{
 				im_mask_object *mo = vi->vargv[i];
 				INTMASK *mask = mo->mask;
-				int ne = mask->xsize * mask->ysize;
-				int i;
 
-				for( i = 0; i < ne; i++ )
-					HASH_I( mask->coeff[i] );
-				HASH_I( mask->scale );
-				HASH_I( mask->offset );
+				/* mask can be NULL if we are called after 
+				 * vips_new() but before we've built the arg
+				 * list.
+				 */
+				if( mask ) {
+					int ne = mask->xsize * mask->ysize;
+					int j;
+
+					for( j = 0; j < ne; j++ )
+						HASH_I( mask->coeff[j] );
+					HASH_I( mask->scale );
+					HASH_I( mask->offset );
+				}
 
 				break;
 			}
@@ -409,14 +429,14 @@ vips_equal( VipsInfo *vi1, VipsInfo *vi2 )
 				DOUBLEMASK *mask1 = mo1->mask;
 				DOUBLEMASK *mask2 = mo2->mask;
 				int ne = mask1->xsize * mask2->ysize;
-				int i;
+				int j;
 	
 				if( mask1->xsize != mask2->xsize ||
 					mask1->ysize != mask2->ysize )
 					return( FALSE );
 
-				for( i = 0; i < ne; i++ )
-					if( mask1->coeff[i] != mask2->coeff[i] )
+				for( j = 0; j < ne; j++ )
+					if( mask1->coeff[j] != mask2->coeff[j] )
 						return( FALSE );
 
 				if( mask1->scale != mask2->scale )
@@ -436,14 +456,14 @@ vips_equal( VipsInfo *vi1, VipsInfo *vi2 )
 				INTMASK *mask1 = mo1->mask;
 				INTMASK *mask2 = mo2->mask;
 				int ne = mask1->xsize * mask2->ysize;
-				int i;
+				int j;
 	
 				if( mask1->xsize != mask2->xsize ||
 					mask1->ysize != mask2->ysize )
 					return( FALSE );
 
-				for( i = 0; i < ne; i++ )
-					if( mask1->coeff[i] != mask2->coeff[i] )
+				for( j = 0; j < ne; j++ )
+					if( mask1->coeff[j] != mask2->coeff[j] )
 						return( FALSE );
 
 				if( mask1->scale != mask2->scale )
@@ -488,11 +508,11 @@ vips_equal( VipsInfo *vi1, VipsInfo *vi2 )
 	return( TRUE );
 }
 
-#ifdef DEBUG_HISTORY
+#ifdef DEBUG_HISTORY_SANITY
 static void
 vips_history_sanity_sub( VipsInfo *vi )
 {
-	assert( g_slist_find( vips_history_lru->list, vi ) );
+	g_assert( g_slist_find( vips_history_lru->list, vi ) );
 }
 
 static void
@@ -508,11 +528,11 @@ vips_history_sanity( void )
 	for( p = vips_history_lru->list; p; p = p->next ) {
 		VipsInfo *vi = (VipsInfo *) p->data;
 
-		assert( g_hash_table_lookup( vips_history_table, vi ) );
+		g_assert( g_hash_table_lookup( vips_history_table, vi ) );
 
-		assert( vi->fn );
-		assert( vi->fn->argc > 0 && vi->fn->argc < 100 );
-		assert( vi->in_cache );
+		g_assert( vi->fn );
+		g_assert( vi->fn->argc > 0 && vi->fn->argc < 100 );
+		g_assert( vi->in_cache );
 	}
 
 	/* Everything that's on the history table should be in the LRU.
@@ -529,7 +549,7 @@ vips_history_sanity( void )
 		/* Need to build with DEBUG on before vips_info_all is
 		 * maintained.
 		 */
-		assert( g_slist_find( vips_info_all, vi ) );
+		g_assert( g_slist_find( vips_info_all, vi ) );
 	}
 
 	/* Everything on vips_info_all that's not in vips_history_table should
@@ -539,10 +559,10 @@ vips_history_sanity( void )
 		VipsInfo *vi = (VipsInfo *) p->data;
 
 		if( !g_hash_table_lookup( vips_history_table, vi ) )
-			assert( !vi->in_cache );
+			g_assert( !vi->in_cache );
 	}
 }
-#endif /*DEBUG_HISTORY*/
+#endif /*DEBUG_HISTORY_SANITY*/
 
 /* Is a function call in our history? Return the old one. 
  */
@@ -562,8 +582,10 @@ vips_history_lookup( VipsInfo *vi )
 #ifdef DEBUG_HISTORY
 	if( old_vi ) 
 		printf( "vips_history_lookup: found \"%s\"\n", old_vi->name );
-	vips_history_sanity();
 #endif /*DEBUG_HISTORY*/
+#ifdef DEBUG_HISTORY_SANITY
+	vips_history_sanity();
+#endif /*DEBUG_HISTORY_SANITY*/
 
 	return( old_vi );
 }
@@ -573,13 +595,15 @@ vips_history_lookup( VipsInfo *vi )
 static void
 vips_history_touch( VipsInfo *vi )
 {
-	assert( vi->in_cache );
+	g_assert( vi->in_cache );
 
 	queue_remove( vips_history_lru, vi );
 	queue_add( vips_history_lru, vi );
 
-#ifdef DEBUG_HISTORY
+#ifdef DEBUG_HISTORY_SANITY
 	vips_history_sanity();
+#endif /*DEBUG_HISTORY_SANITY*/
+#ifdef DEBUG_HISTORY
 	printf( "vips_history_touch: bumping \"%s\"\n", vi->name );
 #endif /*DEBUG_HISTORY*/
 }
@@ -587,19 +611,21 @@ vips_history_touch( VipsInfo *vi )
 static void
 vips_history_insert( VipsInfo *vi )
 {
-	assert( !g_hash_table_lookup( vips_history_table, vi ) );
-	assert( !vi->in_cache );
+	g_assert( !g_hash_table_lookup( vips_history_table, vi ) );
+	g_assert( !vi->in_cache );
 
 	g_hash_table_insert( vips_history_table, vi, vi );
 	vips_history_size += 1;
 
-	assert( g_hash_table_lookup( vips_history_table, vi ) );
+	g_assert( g_hash_table_lookup( vips_history_table, vi ) );
 
 	queue_add( vips_history_lru, vi );
 	vi->in_cache = TRUE;
 
-#ifdef DEBUG_HISTORY
+#ifdef DEBUG_HISTORY_SANITY
 	vips_history_sanity();
+#endif /*DEBUG_HISTORY_SANITY*/
+#ifdef DEBUG_HISTORY
 	printf( "vips_history_insert: adding \"%s\"\n", vi->name );
 #endif /*DEBUG_HISTORY*/
 }
@@ -620,9 +646,9 @@ vips_history_remove( VipsInfo *vi )
 #endif /*DEBUG_HISTORY*/
 	}
 
-#ifdef DEBUG_HISTORY
+#ifdef DEBUG_HISTORY_SANITY
 	vips_history_sanity();
-#endif /*DEBUG_HISTORY*/
+#endif /*DEBUG_HISTORY_SANITY*/
 }
 
 static void
@@ -631,7 +657,7 @@ vips_destroy( VipsInfo *vi )
 	int i;
 
 #ifdef DEBUG_HISTORY
-	printf( "vips_destroy: destroying \"%s\"\n", vi->name );
+	printf( "vips_destroy: destroying \"%s\" (%p)\n", vi->name, vi );
 #endif /*DEBUG_HISTORY*/
 
 	/* Are we in the history? Remove us.
@@ -714,7 +740,7 @@ vips_destroy( VipsInfo *vi )
 			break;
 
 		default:
-			assert( FALSE );
+			g_assert( FALSE );
 		}
 	}
 
@@ -738,9 +764,9 @@ vips_history_remove_lru( void )
 
 	vips_destroy( vi );
 
-#ifdef DEBUG_HISTORY
+#ifdef DEBUG_HISTORY_SANITY
 	vips_history_sanity();
-#endif /*DEBUG_HISTORY*/
+#endif /*DEBUG_HISTORY_SANITY*/
 }
 
 static void
@@ -772,17 +798,9 @@ vips_history_add( VipsInfo *vi )
 {
 	int i;
 
-	/* Is this function call already in the history? If it is, just touch
-	 * the time.
-	 */
-	if( vips_history_lookup( vi ) ) {
-		vips_history_touch( vi );
-		return;
-	}
-
 #ifdef DEBUG_HISTORY
-	printf( "vips_history_add: adding \"%s\", hash = 0x%x\n", 
-		vi->name, vi->hash );
+	printf( "vips_history_add: adding \"%s\" (%p), hash = %u\n", 
+		vi->name, vi, vi->hash );
 #endif /*DEBUG_HISTORY*/
 
 	vips_history_insert( vi );
@@ -874,7 +892,7 @@ vips_tochar( VipsInfo *vi, int i, BufInfo *buf )
 	}
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 }
 
@@ -1058,7 +1076,7 @@ vips_new( Reduce *rc, im_function *fn )
 	VipsInfo *vi;
 	int i;
 
-	assert( fn->argc < MAX_VIPS_ARGS - 1 );
+	g_assert( fn->argc < MAX_VIPS_ARGS - 1 );
 
 	if( !fn || !(vi = INEW( NULL, VipsInfo )) )
 		return( NULL );
@@ -1411,7 +1429,7 @@ vips_fromip( Reduce *rc, PElement *arg,
 	}
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 
 	return( TRUE );
@@ -1511,7 +1529,7 @@ vips_toip( VipsInfo *vi, int i, int *outiiindex, PElement *arg )
 
 	case VIPS_IMAGEVEC:
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 
 #ifdef DEBUG
@@ -1616,7 +1634,7 @@ vips_tobuf( VipsInfo *vi, int i, BufInfo *buf )
 		break;
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 }
 
@@ -1757,7 +1775,7 @@ vips_build_output( VipsInfo *vi, int i )
 	}
 
 	default:
-		assert( FALSE );
+		g_assert( FALSE );
 	}
 }
 
@@ -2123,7 +2141,7 @@ vips_fillva( VipsInfo *vi, va_list ap )
 		}
 
 		else
-			assert( FALSE );
+			g_assert( FALSE );
 	}
 }
 
@@ -2154,7 +2172,7 @@ vips_dispatch( VipsInfo *vi, PElement *out )
 #endif /*DEBUG_HISTORY*/
 	}
 	else {
-		/* No: call function and save result in history.
+		/* No: call function.
 		 */
 		int result;
 
@@ -2196,10 +2214,23 @@ vips_dispatch( VipsInfo *vi, PElement *out )
 		reduce_throw( rc );
 	}
 
-	/* Can we cache this operation? Memoise or destroy.
+	/* Add to our operation cache, if necessary.
 	 */
-	if( vi->fn->flags & IM_FN_NOCACHE )
+	if( vi->fn->flags & IM_FN_NOCACHE ) 
+		/* Uncacheable operation.
+		 */
 		vips_destroy( vi );
+	else if( vi->in_cache ) 
+		/* Already in the history. Just touch the time.
+		 */
+		vips_history_touch( vi );
+	else if( (old_vi = vips_history_lookup( vi )) ) {
+		/* We have an equal but older item there? This can happen with
+		 * nested calls. Touch the old one and destroy this one.
+		 */
+		vips_history_touch( old_vi );
+		vips_destroy( vi );
+	}
 	else
 		vips_history_add( vi );
 }
@@ -2271,6 +2302,10 @@ vips_run( Reduce *rc, Compile *compile,
 	printf( "vips_run: starting for %s\n", name );
 #endif /*DEBUG*/
 
+#ifdef DEBUG_HISTORY_SANITY
+	vips_history_sanity();
+#endif /*DEBUG_HISTORY_SANITY*/
+
 	if( !(vi = vips_new( rc, function )) )
 		reduce_throw( rc );
 
