@@ -118,17 +118,20 @@ vobject_idle_refresh( gpointer user_data )
 		vObject *vobject = VOBJECT( data );
 
 #ifdef DEBUG
-		printf( "vobject_idle_refresh: starting \"%s\" (0x%x)\n", 
-			G_OBJECT_TYPE_NAME( vobject ), (unsigned int) vobject );
+		printf( "vobject_idle_refresh: starting \"%s\" (%p)\n", 
+			G_OBJECT_TYPE_NAME( vobject ), vobject );
 #endif /*DEBUG*/
 
+		/* We must clear dirty before we _refresh() so that if the
+		 * _refresh() indirectly triggers another update, we will
+		 * _refresh() again.
+		 */
 		vobject->dirty = FALSE;
 		vobject_refresh( vobject );
 	}
 
 	vobject_idle_running = 0;
 
-
 	return( FALSE );
 }
 #endif /*DEBUG_TIME*/
@@ -141,8 +144,8 @@ vobject_refresh_queue( vObject *vobject )
 {
 	if( !vobject->dirty ) {
 #ifdef DEBUG
-		printf( "vobject_refresh_queue: %s (0x%x)", 
-			G_OBJECT_TYPE_NAME( vobject ), (unsigned int) vobject );
+		printf( "vobject_refresh_queue: %s (%p)", 
+			G_OBJECT_TYPE_NAME( vobject ), vobject );
 		if( vobject->iobject )
 			printf( ", iobject %s \"%s\"", 
 				G_OBJECT_TYPE_NAME( vobject->iobject ), 
