@@ -579,12 +579,19 @@ static gboolean
 watch_int_update( Watch *watch )
 {
 	WatchInt *watch_int = WATCH_INT( watch );
+	Expr *expr = watch->row->expr;
 	PElement *root = &watch->row->expr->root;
 
 #ifdef DEBUG
 	printf( "watch_int_update: %s\n", NN( IOBJECT( watch )->name ) );
 #endif /*DEBUG*/
 
+	/* Can get called during shutdown :-( main_watchgroup_changed_cb() can
+	 * call us before destroying the row, but after killing the expr.
+	 */
+	if( !expr )
+		return( FALSE );
+	root = &expr->root;
 	if( PEISNOVAL( root ) )
 		return( FALSE );
 	if( !PEISREAL( root ) ) {
@@ -715,7 +722,6 @@ watch_path_update( Watch *watch )
 	printf( "watch_path_update\n" );
 #endif /*DEBUG*/
 
-	value = NULL;
 	if( !heap_get_lstring( root, &value ) )
 		return( FALSE );
 
@@ -959,7 +965,7 @@ watch_string_update( Watch *watch )
 	if( !heap_get_string( root, value, 1024 ) )
 		return( FALSE );
 
-	SETSTR( watch_string->value, value );
+	IM_SETSTR( watch_string->value, value );
 
 	return( TRUE );
 }
