@@ -367,8 +367,8 @@ workspace_column_pick( Workspace *ws )
 	/* Make an empty column ... always at the top left.
 	 */
 	col = column_new( ws, "A" );
-	col->x = 3;
-	col->y = 3;
+	col->x = WORKSPACEVIEW_MARGIN_LEFT;
+	col->y = WORKSPACEVIEW_MARGIN_TOP;
 	workspace_column_select( ws, col );
 
 	return( col );
@@ -420,6 +420,7 @@ workspace_add_def( Workspace *ws, const char *str )
 {
 	Column *col = workspace_column_pick( ws );
 	Symbol *sym;
+	char *name;
 
 #ifdef DEBUG
 	printf( "workspace_add_def: %s\n", str );
@@ -427,16 +428,31 @@ workspace_add_def( Workspace *ws, const char *str )
 
         if( !str || strspn( str, WHITESPACE ) == strlen( str ) )
 		return( NULL );
-	if( !(sym = workspace_add_symbol( ws )) )
-		return( NULL );
+
+	/* Try parsing as a "fred = 12" style def. 
+	 */
 	attach_input_string( str );
-	if( !symbol_user_init( sym ) || !parse_rhs( sym->expr, PARSE_RHS ) ) {
-		/* Parse error.
+	if( (name = parse_test_define()) ) {
+		sym = symbol_new( ws->sym->expr->compile, name );
+		IM_FREE( name );
+		attach_input_string( str + 
+			IM_CLIP( 0, input_state.charpos - 1, strlen( str ) ) );
+	}
+	else {
+		/* That didn't work. Make a sym from the col name.
+		 */
+		sym = workspace_add_symbol( ws );
+		attach_input_string( str );
+	}
+
+	if( !symbol_user_init( sym ) || 
+		!parse_rhs( sym->expr, PARSE_RHS ) ) {
+		/* Another parse error.
 		 */
 		expr_error_get( sym->expr );
 
-		/* Block changes to error_string ... symbol_destroy() can set
-		 * this for compound objects.
+		/* Block changes to error_string ... symbol_destroy() 
+		 * can set this for compound objects.
 		 */
 		error_block();
 		IDESTROY( sym );
@@ -444,7 +460,11 @@ workspace_add_def( Workspace *ws, const char *str )
 
 		return( NULL );
 	}
-	(void) row_new( col->scol, sym, &sym->expr->root );
+
+	/* If we're redefining a sym, it might have a row already.
+	 */
+	if( !sym->expr->row )
+		(void) row_new( col->scol, sym, &sym->expr->root );
 	symbol_made( sym );
 	filemodel_set_modified( FILEMODEL( ws ), TRUE );
 
@@ -496,14 +516,41 @@ workspace_load_file( Workspace *ws, const char *filename )
 	return( NULL );
 }
 
+/* Bounding box of columns to be saved. Though we only really set top/left.
+ */
+static void *
+workspace_selected_save_box( Column *col, Rect *box )
+{
+	if( model_save_test( MODEL( col ) ) ) {
+		if( im_rect_isempty( box ) ) {
+			box->left = col->x;
+			box->top = col->y;
+			box->width = 100;
+			box->height = 100;
+		}
+		else {
+			box->left = IM_MIN( box->left, col->x );
+			box->top = IM_MIN( box->top, col->y );
+		}
+	}
+
+	return( NULL );
+}
+
 /* Save just the selected objects.
  */
 gboolean
 workspace_selected_save( Workspace *ws, const char *filename )
 {
 	WorkspaceSaveType save = ws->save_type;
+	Rect box = { 0 };
 
 	ws->save_type = WORKSPACE_SAVE_SELECTED;
+
+	workspace_map_column( ws, 
+		(column_map_fn) workspace_selected_save_box, &box );
+	filemodel_set_offset( FILEMODEL( ws ), box.left, box.top );
+
 	if( !filemodel_save_all( FILEMODEL( ws ), filename ) ) {
 		ws->save_type = save;
 		unlinkf( "%s", filename );
@@ -751,13 +798,19 @@ workspace_dispose( GObject *gobject )
 {
 	Workspace *ws;
 
+#ifdef DEBUG
+	printf( "workspace_dispose: %s\n", NN( IOBJECT( gobject )->name ) );
+#endif /*DEBUG*/
+
 	g_return_if_fail( gobject != NULL );
 	g_return_if_fail( IS_WORKSPACE( gobject ) );
 
 	ws = WORKSPACE( gobject );
 
-	IDESTROY( ws->sym );
 	IM_FREEF( g_source_remove, ws->auto_save_timeout );
+	UNREF( ws->kitg );
+	UNREF( ws->local_kitg );
+	IDESTROY( ws->sym );
 
 	G_OBJECT_CLASS( parent_class )->dispose( gobject );
 }
@@ -776,9 +829,9 @@ workspace_finalize( GObject *gobject )
 
 	ws = WORKSPACE( gobject );
 
-	UNREF( ws->kitg );
-
 	IM_FREE( ws->status );
+	IM_FREE( ws->local_defs );
+
 	workspace_all = g_slist_remove( workspace_all, ws );
 
 	G_OBJECT_CLASS( parent_class )->finalize( gobject );
@@ -831,6 +884,10 @@ workspace_link( Workspace *ws, Workspacegroup *wsg, const char *name )
 	(void) compile_new( sym->expr );
 	symbol_made( sym );
 	iobject_set( IOBJECT( ws ), name, NULL );
+
+	ws->local_kitg = toolkitgroup_new( ws->sym );
+	g_object_ref( G_OBJECT( ws->local_kitg ) );
+	iobject_sink( IOBJECT( ws->local_kitg ) );
 }
 
 static const char *
@@ -876,6 +933,7 @@ workspace_load( Model *model,
 {
 	Workspace *ws = WORKSPACE( model );
 	char buf[256];
+	char *txt;
 
 	assert( IS_WORKSPACEGROUP( parent ) );
 
@@ -899,6 +957,11 @@ workspace_load( Model *model,
 	(void) get_iprop( xnode, "window_width", &ws->window_width );
 	(void) get_iprop( xnode, "window_height", &ws->window_height );
 
+	(void) get_bprop( xnode, "lpane_open", &ws->lpane_open );
+	(void) get_iprop( xnode, "lpane_position", &ws->lpane_position );
+	(void) get_bprop( xnode, "rpane_open", &ws->rpane_open );
+	(void) get_iprop( xnode, "rpane_position", &ws->rpane_position );
+
 	if( get_sprop( xnode, "name", buf, 256 ) ) {
 		IM_SETSTR( IOBJECT( ws )->name, buf );
 	}
@@ -906,6 +969,13 @@ workspace_load( Model *model,
 		IM_SETSTR( IOBJECT( ws )->caption, buf );
 	}
 
+	/* Don't use get_sprop() and avoid a limit on def size.
+	 */
+	if( (txt = (char *) xmlGetProp( xnode, (xmlChar *) "local_defs" )) ) {
+		(void) workspace_local_set( ws, txt );
+		IM_FREEF( xmlFree, txt );
+	}
+
 	if( !MODEL_CLASS( parent_class )->load( model, state, parent, xnode ) )
 		return( FALSE );
 
@@ -926,6 +996,15 @@ workspace_save( Model *model, xmlNode *xnode )
 		!set_dprop( xthis, "offset", ws->offset ) ||
 		!set_prop( xthis, "window_width", "%d", ws->window_width ) ||
 		!set_prop( xthis, "window_height", "%d", ws->window_height ) ||
+		!set_prop( xthis, "lpane_position", "%d", 
+			ws->lpane_position ) ||
+		!set_prop( xthis, "lpane_open", 
+			bool_to_char( ws->lpane_open ) ) ||
+		!set_prop( xthis, "rpane_position", "%d", 
+			ws->rpane_position ) ||
+		!set_prop( xthis, "rpane_open", 
+			bool_to_char( ws->rpane_open ) ) ||
+		!set_sprop( xthis, "local_defs", ws->local_defs ) ||
 		!set_sprop( xthis, "name", IOBJECT( ws )->name ) ||
 		!set_sprop( xthis, "caption", IOBJECT( ws )->caption ) ) 
 		return( NULL );
@@ -1037,10 +1116,10 @@ workspace_load_compat( Workspace *ws, int major, int minor )
 	path = path_parse( pathname );
 	if( path_map_exact( path, "*.def", 
 		(path_map_fn) workspace_load_toolkit, ws->kitg ) ) {
-		path_free( path );
+		path_free2( path );
 		return( FALSE );
 	}
-	path_free( path );
+	path_free2( path );
 
 	ws->compat_major = major;
 	ws->compat_minor = minor;
@@ -1093,9 +1172,8 @@ workspace_top_load( Filemodel *filemodel,
 
 			/* Do we need broken region handling?
 			 */
-			if( filemodel->major == 7 || 
-				filemodel->minor == 8 ) 
-				ws->compatibility = TRUE;
+			if( filemodel->major == 7 && filemodel->minor == 8 ) 
+				ws->compat_78 = TRUE;
 		}
 
 		if( model_load( MODEL( ws ), state, parent, xnode ) )
@@ -1257,7 +1335,7 @@ workspace_init( Workspace *ws )
 	ws->errors = NULL;
         ws->mode = WORKSPACE_MODE_REGULAR;
 
-	ws->compatibility = FALSE;
+	ws->compat_78 = FALSE;
 	ws->compat_major = 0;
 	ws->compat_minor = 0;
 
@@ -1272,6 +1350,13 @@ workspace_init( Workspace *ws )
 	ws->window_width = 0;
 	ws->window_height = 0;
 
+	/* Overwritten by mainw.
+	 */
+	ws->lpane_open = WORKSPACE_RPANE_OPEN;
+	ws->lpane_position = WORKSPACE_RPANE_POSITION;
+	ws->rpane_open = WORKSPACE_LPANE_OPEN;
+	ws->rpane_position = WORKSPACE_LPANE_POSITION;
+
 	ws->auto_save_timeout = 0;
 
 	ws->status = NULL;
@@ -1281,6 +1366,11 @@ workspace_init( Workspace *ws )
 	ws->scale = 1.0;
 	ws->offset = 0.0;
 
+	ws->local_defs = im_strdupn( _( 
+		"// private definitions for this workspace\n" ) );
+	ws->local_kitg = NULL;
+	ws->local_kit = NULL;
+
 	filemodel_register( FILEMODEL( ws ) );
 
 	workspace_all = g_slist_prepend( workspace_all, ws );
@@ -1335,6 +1425,24 @@ workspace_new( Workspacegroup *wsg, const char *name )
 	return( ws );
 }
 
+/* Load into an empty workspace.
+ */
+static gboolean
+workspace_load_empty( Workspace *ws, Workspacegroup *wsg, const char *filename )
+{
+	g_assert( workspace_is_empty( ws ) );
+
+	ws->load_type = WORKSPACE_LOAD_TOP;
+	column_set_offset( WORKSPACEVIEW_MARGIN_LEFT, 
+		WORKSPACEVIEW_MARGIN_TOP );
+	if( !filemodel_load_all( FILEMODEL( ws ), MODEL( wsg ), filename ) ) 
+		return( FALSE );
+	filemodel_set_modified( FILEMODEL( ws ), FALSE );
+	filemodel_set_filename( FILEMODEL( ws ), filename );
+
+	return( TRUE );
+}
+
 /* New workspace from a file.
  */
 Workspace *
@@ -1347,19 +1455,11 @@ workspace_new_from_file( Workspacegroup *wsg, const char *filename )
 #endif /*DEBUG*/
 
 	ws = WORKSPACE( g_object_new( TYPE_WORKSPACE, NULL ) );
-	ws->load_type = WORKSPACE_LOAD_TOP;
-	if( !filemodel_load_all( FILEMODEL( ws ), MODEL( wsg ), filename ) ) {
+	if( !workspace_load_empty( ws, wsg, filename ) ) {
 		g_object_unref( G_OBJECT( ws ) );
 		return( NULL );
 	}
 
-	filemodel_set_modified( FILEMODEL( ws ), FALSE );
-	filemodel_set_filename( FILEMODEL( ws ), filename );
-
-#ifdef DEBUG
-	printf( "(set name = %s)\n", IOBJECT( ws )->name );
-#endif /*DEBUG*/
-
 	return( ws );
 }
 
@@ -1417,17 +1517,16 @@ workspace_merge_file( Workspace *ws, const char *filename )
 	if( workspace_is_empty( ws ) ) {
 		model_empty( MODEL( ws ) );
 
-		ws->load_type = WORKSPACE_LOAD_TOP;
-		if( !filemodel_load_all( FILEMODEL( ws ), 
-			MODEL( ICONTAINER( ws )->parent ), filename ) ) 
+		if( !workspace_load_empty( ws, 
+			WORKSPACEGROUP( ICONTAINER( ws )->parent ), 
+			filename ) ) 
 			return( FALSE );
-
-		filemodel_set_modified( FILEMODEL( ws ), FALSE );
-		filemodel_set_filename( FILEMODEL( ws ), filename );
 	}
 	else {
 		ws->load_type = WORKSPACE_LOAD_COLUMNS;
-		column_set_offset( IM_RECT_RIGHT( &ws->area ), 0 );
+		column_set_offset( 
+			IM_RECT_RIGHT( &ws->area ) + WORKSPACEVIEW_MARGIN_LEFT,
+			WORKSPACEVIEW_MARGIN_TOP );
 		if( !filemodel_load_all( FILEMODEL( ws ), 
 			MODEL( ICONTAINER( ws )->parent ), filename ) ) 
 			return( FALSE );
@@ -1715,7 +1814,7 @@ workspace_ungroup_row( Row *row )
 		return( row );
 	if( result ) {
 		if( !class_get_member( root, MEMBER_VALUE, NULL, &value ) || 
-			(length = heap_list_length( &value )) < 0 ) 
+			(length = heap_list_length_max( &value, 100 )) < 0 ) 
 			return( row );
 
 		for( i = 0; i < length; i++ )
@@ -1727,7 +1826,7 @@ workspace_ungroup_row( Row *row )
 		if( !heap_is_list( root, &result ) )
 			return( row );
 		if( result ) {
-			if( (length = heap_list_length( root )) < 0 ) 
+			if( (length = heap_list_length_max( root, 100 )) < 0 ) 
 				return( row );
 
 			for( i = 0; i < length; i++ )
@@ -1811,3 +1910,51 @@ workspace_set_mode( Workspace *ws, WorkspaceMode mode )
 			(icontainer_map_fn) iobject_changed, NULL );
 	}
 }
+
+/* New ws private defs.
+ */
+gboolean
+workspace_local_set( Workspace *ws, const char *txt )
+{
+	/* New kit for defs ... will destroy any old defs, since we can't have
+	 * two kits with the same name. Don't register it, we don't want it 
+	 * to be autosaved on quit.
+	 */
+	ws->local_kit = toolkit_new( ws->local_kitg, "Workspace Locals" );
+	filemodel_unregister( FILEMODEL( ws->local_kit ) );
+	IM_SETSTR( ws->local_defs, txt );
+	iobject_changed( IOBJECT( ws ) );
+
+	filemodel_set_modified( FILEMODEL( ws ), TRUE );
+	attach_input_string( txt );
+	if( !parse_toplevel( ws->local_kit, 0 ) ) 
+		return( FALSE );
+
+	return( TRUE );
+}
+
+gboolean
+workspace_local_set_from_file( Workspace *ws, const char *fname )
+{
+	iOpenFile *of;
+	char *txt;
+
+	if( !(of = file_open_read( "%s", fname )) ) 
+		return( FALSE );
+	if( !(txt = file_read( of )) ) {
+		file_close( of );
+		return( FALSE );
+	}
+	if( !workspace_local_set( ws, txt ) ) {
+		g_free( txt );
+		file_close( of );
+		return( FALSE );
+	}
+
+	filemodel_set_filename( FILEMODEL( ws->local_kit ), fname );
+
+	g_free( txt );
+	file_close( of );
+
+	return( TRUE );
+}
