@@ -83,12 +83,12 @@ struct _Workspace {
 	GSList *errors;		/* Rows with errors */
         WorkspaceMode mode;	/* Display mode */
 
-	/* Compatibility. 7.8.x used different names for Marks, and had strange
-	 * position behaviour
+	/* Compatibility. 7.8.x used different names for Marks and had strange
+	 * position behaviour.
 	 */
-	gboolean compatibility;	/* In 7.8.x compatibility mode */
+	gboolean compat_78;	/* In 7.8.x compatibility mode */
 
-	/* Other versions (7.10 etc.) need special compat toolkits. 0 here for
+	/* Some versions (7.10 etc.) need special compat toolkits. 0 here for
 	 * no compat toolkits loaded.
 	 */
 	int compat_major;
@@ -103,6 +103,10 @@ struct _Workspace {
 	Rect vp;		/* Viewport hint ... set by views */
 	int window_width;	/* Enclosing window size ... set by views */
 	int window_height;
+	gboolean lpane_open;	/* Pane model */
+	int lpane_position;	
+	gboolean rpane_open;
+	int rpane_position;
 
 	guint auto_save_timeout;/* Timeout for next autosave */
 
@@ -118,6 +122,13 @@ struct _Workspace {
 	 */
 	double scale;
 	double offset;
+
+	/* Workspace-local defs, displayed in the left pane. All in a private
+	 * kitg and kit.
+	 */
+	char *local_defs;
+	Toolkitgroup *local_kitg;
+	Toolkit *local_kit;
 };
 
 typedef struct _WorkspaceClass {
@@ -191,3 +202,5 @@ void workspace_set_status( Workspace *ws, const char *fmt, ... )
 
 void workspace_set_mode( Workspace *ws, WorkspaceMode mode );
 
+gboolean workspace_local_set( Workspace *ws, const char *txt );
+gboolean workspace_local_set_from_file( Workspace *ws, const char *fname );
