@@ -58,6 +58,11 @@ static void
 workspacegroup_child_add( iContainer *parent, iContainer *child, int pos )
 {
 	ICONTAINER_CLASS( parent_class )->child_add( parent, child, pos );
+
+#ifdef DEBUG
+	printf( "workspacegroup_child_add: added %s\n",
+		IOBJECT( child )->name );
+#endif /*DEBUG*/
 }
 
 static void
@@ -214,7 +219,7 @@ workspacegroup_workspace_new( Workspacegroup *wsg, GtkWidget *parent )
 		iwindow_true_cb, NULL, NULL, wsg );
 	idialog_add_ok( IDIALOG( ss ), 
 		workspacegroup_workspace_new_done_cb, _( "Create Workspace" ) );
-	idialog_set_parent( IDIALOG( ss ), parent );
+	iwindow_set_parent( IWINDOW( ss ), parent );
 	iwindow_build( IWINDOW( ss ) );
 
 	gtk_widget_show( ss );
